@@ -314,10 +314,6 @@ void FrameSequenceTracker::ReportSubmitFrame(
           << TRACKER_DCHECK_MSG;
     }
   }
-
-  if (has_missing_content) {
-    checkerboarding_.frames.push_back(frame_token);
-  }
 }
 
 void FrameSequenceTracker::ReportFrameEnd(
@@ -373,9 +369,6 @@ void FrameSequenceTracker::ReportFrameEnd(
     DCHECK_GT(impl_throughput().frames_expected,
               impl_throughput().frames_produced)
         << TRACKER_DCHECK_MSG;
-    DCHECK_GE(impl_throughput().frames_produced,
-              impl_throughput().frames_ontime)
-        << TRACKER_DCHECK_MSG;
     --impl_throughput().frames_expected;
     metrics()->NotifyNoUpdateForJankReporter(
         FrameInfo::SmoothEffectDrivingThread::kCompositor,
@@ -443,25 +436,14 @@ void FrameSequenceTracker::ReportFramePresented(
       (feedback.interval.is_zero() ? viz::BeginFrameArgs::DefaultInterval()
                                    : feedback.interval);
   DCHECK(!vsync_interval.is_zero()) << TRACKER_DCHECK_MSG;
-  base::TimeTicks safe_deadline_for_frame =
-      last_frame_presentation_timestamp_ + vsync_interval * 1.5;
-
   const bool was_presented = !feedback.failed();
   if (was_presented && submitted_frame_since_last_presentation) {
-    if (!last_frame_presentation_timestamp_.is_null() &&
-        (safe_deadline_for_frame < feedback.timestamp)) {
-      DCHECK_LE(impl_throughput().frames_ontime,
-                impl_throughput().frames_produced)
-          << TRACKER_DCHECK_MSG;
-      ++impl_throughput().frames_ontime;
-    }
-
     DCHECK_LT(impl_throughput().frames_produced,
               impl_throughput().frames_expected)
         << TRACKER_DCHECK_MSG;
     ++impl_throughput().frames_produced;
     if (metrics()->GetEffectiveThread() == ThreadType::kCompositor) {
-      metrics()->AdvanceTrace(feedback.timestamp);
+      metrics()->AdvanceTrace(feedback.timestamp, frame_token);
     }
 
     metrics()->ComputeJank(FrameInfo::SmoothEffectDrivingThread::kCompositor,
@@ -482,54 +464,12 @@ void FrameSequenceTracker::ReportFramePresented(
           << TRACKER_DCHECK_MSG;
       ++main_throughput().frames_produced;
       if (metrics()->GetEffectiveThread() == ThreadType::kMain) {
-        metrics()->AdvanceTrace(feedback.timestamp);
+        metrics()->AdvanceTrace(feedback.timestamp, frame_token);
       }
 
       metrics()->ComputeJank(FrameInfo::SmoothEffectDrivingThread::kMain,
                              frame_token, feedback.timestamp, vsync_interval);
     }
-    if (main_frames_.size() < size_before_erase) {
-      if (!last_frame_presentation_timestamp_.is_null() &&
-          (safe_deadline_for_frame < feedback.timestamp)) {
-        DCHECK_LE(main_throughput().frames_ontime,
-                  main_throughput().frames_produced)
-            << TRACKER_DCHECK_MSG;
-        ++main_throughput().frames_ontime;
-      }
-    }
-    last_frame_presentation_timestamp_ = feedback.timestamp;
-
-    if (checkerboarding_.last_frame_had_checkerboarding) {
-      DCHECK(!checkerboarding_.last_frame_timestamp.is_null())
-          << TRACKER_DCHECK_MSG;
-      DCHECK(!feedback.timestamp.is_null()) << TRACKER_DCHECK_MSG;
-
-      // |feedback.timestamp| is the timestamp when the latest frame was
-      // presented. |checkerboarding_.last_frame_timestamp| is the timestamp
-      // when the previous frame (which had checkerboarding) was presented. Use
-      // |feedback.interval| to compute the number of vsyncs that have passed
-      // between the two frames (since that is how many times the user saw that
-      // checkerboarded frame).
-      base::TimeDelta difference =
-          feedback.timestamp - checkerboarding_.last_frame_timestamp;
-      const auto& interval = feedback.interval.is_zero()
-                                 ? viz::BeginFrameArgs::DefaultInterval()
-                                 : feedback.interval;
-      DCHECK(!interval.is_zero()) << TRACKER_DCHECK_MSG;
-      constexpr base::TimeDelta kEpsilon = base::Milliseconds(1);
-      int64_t frames = (difference + kEpsilon).IntDiv(interval);
-      metrics_->add_checkerboarded_frames(frames);
-    }
-
-    const bool frame_had_checkerboarding =
-        base::Contains(checkerboarding_.frames, frame_token);
-    checkerboarding_.last_frame_had_checkerboarding = frame_had_checkerboarding;
-    checkerboarding_.last_frame_timestamp = feedback.timestamp;
-  }
-
-  while (!checkerboarding_.frames.empty() &&
-         !viz::FrameTokenGT(checkerboarding_.frames.front(), frame_token)) {
-    checkerboarding_.frames.pop_front();
   }
 }
 
@@ -608,8 +548,6 @@ void FrameSequenceTracker::ReportMainFrameCausedNoDamage(
   DCHECK_GT(main_throughput().frames_expected,
             main_throughput().frames_produced)
       << TRACKER_DCHECK_MSG;
-  DCHECK_GE(main_throughput().frames_produced, main_throughput().frames_ontime)
-      << TRACKER_DCHECK_MSG;
   last_no_main_damage_sequence_ = args.frame_id.sequence_number;
   --main_throughput().frames_expected;
   metrics()->NotifyNoUpdateForJankReporter(
@@ -698,7 +636,4 @@ void FrameSequenceTracker::AddSortedFrame(const viz::BeginFrameArgs& args,
     metrics_->AddSortedFrame(args, frame_info);
 }
 
-FrameSequenceTracker::CheckerboardingData::CheckerboardingData() = default;
-FrameSequenceTracker::CheckerboardingData::~CheckerboardingData() = default;
-
 }  // namespace cc
