@@ -13,19 +13,18 @@
 #include <vector>
 
 #include "base/auto_reset.h"
-#include "base/debug/crash_logging.h"
 #include "base/functional/bind.h"
 #include "base/memory/raw_ptr.h"
 #include "base/notreached.h"
 #include "base/task/single_thread_task_runner.h"
 #include "base/trace_event/trace_event.h"
 #include "base/trace_event/traced_value.h"
+#include "base/trace_event/typed_macros.h"
 #include "cc/base/devtools_instrumentation.h"
 #include "cc/base/features.h"
 #include "cc/benchmarks/benchmark_instrumentation.h"
 #include "cc/input/browser_controls_offset_manager.h"
 #include "cc/metrics/compositor_timing_history.h"
-#include "cc/metrics/jank_injector.h"
 #include "cc/paint/paint_worklet_layer_painter.h"
 #include "cc/trees/compositor_commit_data.h"
 #include "cc/trees/layer_tree_frame_sink.h"
@@ -35,11 +34,9 @@
 #include "cc/trees/proxy_main.h"
 #include "cc/trees/render_frame_metadata_observer.h"
 #include "cc/trees/task_runner_provider.h"
-#include "components/power_scheduler/power_mode_arbiter.h"
 #include "components/viz/common/frame_sinks/delay_based_time_source.h"
 #include "components/viz/common/frame_timing_details.h"
-#include "components/viz/common/gpu/context_provider.h"
-#include "gpu/command_buffer/client/gles2_interface.h"
+#include "components/viz/common/gpu/raster_context_provider.h"
 #include "services/metrics/public/cpp/ukm_recorder.h"
 
 namespace cc {
@@ -50,9 +47,6 @@ namespace {
 constexpr auto kSmoothnessTakesPriorityExpirationDelay =
     base::Milliseconds(250);
 
-// Make this less than kHungRendererDelay (15 sec).
-constexpr base::TimeDelta kHungCommitTimeout = base::Seconds(14);
-
 }  // namespace
 
 // Ensures that a CompletionEvent for commit is always signaled.
@@ -111,7 +105,6 @@ ProxyImpl::ProxyImpl(
 
   host_impl_ = layer_tree_host->CreateLayerTreeHostImpl(this);
   send_compositor_frame_ack_ = settings->send_compositor_frame_ack;
-  last_raster_priority_ = SAME_PRIORITY_FOR_BOTH_TREES;
 
   SchedulerSettings scheduler_settings(settings->ToSchedulerSettings());
   scheduler_settings.main_frame_before_commit_enabled =
@@ -126,8 +119,7 @@ ProxyImpl::ProxyImpl(
       this, scheduler_settings, layer_tree_host_id_,
       task_runner_provider_->ImplThreadTaskRunner(),
       std::move(compositor_timing_history),
-      host_impl_->compositor_frame_reporting_controller(),
-      power_scheduler::PowerModeArbiter::GetInstance());
+      host_impl_->compositor_frame_reporting_controller());
 
   DCHECK_EQ(scheduler_->visible(), host_impl_->visible());
 }
@@ -286,10 +278,11 @@ void ProxyImpl::FinishGLOnImpl(CompletionEvent* completion) {
   TRACE_EVENT0("cc", "ProxyImpl::FinishGLOnImplThread");
   DCHECK(IsImplThread());
   if (host_impl_->layer_tree_frame_sink()) {
-    viz::ContextProvider* context_provider =
+    auto* context_provider =
         host_impl_->layer_tree_frame_sink()->context_provider();
-    if (context_provider)
-      context_provider->ContextGL()->Finish();
+    if (context_provider) {
+      context_provider->RasterInterface()->Finish();
+    }
   }
   completion->Signal();
 }
@@ -379,9 +372,6 @@ void ProxyImpl::NotifyReadyToCommitOnImpl(
           completion_event, start_time, MainThreadTaskRunner(),
           proxy_main_weak_ptr_),
       std::move(commit_state), unsafe_state, commit_timestamps);
-  hung_commit_timer_.Start(
-      FROM_HERE, kHungCommitTimeout,
-      base::BindOnce(&ProxyImpl::OnHungCommit, base::Unretained(this)));
 
   // Extract metrics data from the layer tree host and send them to the
   // scheduler to pass them to the compositor_timing_history object.
@@ -393,17 +383,6 @@ void ProxyImpl::NotifyReadyToCommitOnImpl(
     scheduler_->SetNeedsBeginMainFrame();
 }
 
-void ProxyImpl::OnHungCommit() {
-  UMA_HISTOGRAM_BOOLEAN("Compositing.Renderer.CommitHung", true);
-  static auto* hung_commit_data = base::debug::AllocateCrashKeyString(
-      "hung_commit", base::debug::CrashKeySize::Size1024);
-  std::string debug_info = host_impl_->GetHungCommitDebugInfo() +
-                           scheduler_->GetHungCommitDebugInfo();
-  LOG(ERROR) << "commit hung: " << debug_info;
-  base::debug::SetCrashKeyString(hung_commit_data, debug_info);
-  scheduler_->TraceHungCommitDebugInfo();
-}
-
 void ProxyImpl::DidLoseLayerTreeFrameSinkOnImplThread() {
   TRACE_EVENT0("cc", "ProxyImpl::DidLoseLayerTreeFrameSinkOnImplThread");
   DCHECK(IsImplThread());
@@ -496,10 +475,6 @@ void ProxyImpl::SetVideoNeedsBeginFrames(bool needs_begin_frames) {
     scheduler_->SetVideoNeedsBeginFrames(needs_begin_frames);
 }
 
-bool ProxyImpl::HasInvalidationAnimation() const {
-  return host_impl_->mutator_host()->HasInvalidationAnimation();
-}
-
 bool ProxyImpl::IsInsideDraw() {
   return inside_draw_;
 }
@@ -507,76 +482,54 @@ bool ProxyImpl::IsInsideDraw() {
 void ProxyImpl::RenewTreePriority() {
   DCHECK(IsImplThread());
 
-  bool scroll_type_considered_interaction = false;
-  bool prefer_new_content = false;
+  bool precise_scrolling_in_progress =
+      host_impl_->GetActivelyScrollingType() == ActivelyScrollingType::kPrecise;
+
+  bool avoid_entering_smoothness =
+      host_impl_->CurrentScrollCheckerboardsDueToNoRecording() ||
+      (precise_scrolling_in_progress &&
+       host_impl_->IsCurrentScrollMainRepainted());
+
   bool non_scroll_interaction_in_progress =
       host_impl_->IsPinchGestureActive() ||
       host_impl_->page_scale_animation_active();
 
-  ActivelyScrollingType actively_scrolling_type =
-      host_impl_->GetActivelyScrollingType();
-
-  switch (actively_scrolling_type) {
-    case ActivelyScrollingType::kNone:
-      break;
-    case ActivelyScrollingType::kPrecise:
-      scroll_type_considered_interaction = true;
-      break;
-    case ActivelyScrollingType::kAnimated:
-      scroll_type_considered_interaction = base::FeatureList::IsEnabled(
-          features::kSchedulerSmoothnessForAnimatedScrolls);
-  }
-
-  bool user_interaction_in_progress =
-      non_scroll_interaction_in_progress || scroll_type_considered_interaction;
-
-  if (host_impl_->CurrentScrollCheckerboardsDueToNoRecording() &&
-      base::FeatureList::IsEnabled(
-          features::kPreferNewContentForCheckerboardedScrolls)) {
-    prefer_new_content = true;
-  }
-  if (scroll_type_considered_interaction &&
-      host_impl_->IsCurrentScrollMainRepainted()) {
-    prefer_new_content = true;
-  }
-
   // Schedule expiration if smoothness currently takes priority.
-  if (user_interaction_in_progress && !prefer_new_content)
+  if ((non_scroll_interaction_in_progress || precise_scrolling_in_progress) &&
+      !avoid_entering_smoothness) {
     smoothness_priority_expiration_notifier_.Schedule();
+  }
 
   // We use the same priority for both trees by default.
-  TreePriority scheduler_tree_priority = SAME_PRIORITY_FOR_BOTH_TREES;
-  TreePriority raster_tree_priority = SAME_PRIORITY_FOR_BOTH_TREES;
+  TreePriority tree_priority = SAME_PRIORITY_FOR_BOTH_TREES;
 
   // Smoothness takes priority if we have an expiration for it scheduled.
   if (smoothness_priority_expiration_notifier_.HasPendingNotification()) {
-    scheduler_tree_priority = SMOOTHNESS_TAKES_PRIORITY;
-    if (non_scroll_interaction_in_progress ||
-        actively_scrolling_type == ActivelyScrollingType::kPrecise ||
-        last_raster_priority_ == SMOOTHNESS_TAKES_PRIORITY)
-      raster_tree_priority = SMOOTHNESS_TAKES_PRIORITY;
+    tree_priority = SMOOTHNESS_TAKES_PRIORITY;
   }
 
-  last_raster_priority_ = raster_tree_priority;
-
   // New content takes priority in certain cases:
   // - When ui resources have been evicted.
   // - When the viewport is 0x0 (may be invalid/unset?)
   // - When the active scroll gesture requires main-thread repainting for the
   //   scroll offset change to be visible.
   if (host_impl_->active_tree()->GetDeviceViewport().size().IsEmpty() ||
-      host_impl_->EvictedUIResourcesExist() ||
-      (host_impl_->IsCurrentScrollMainRepainted() &&
-       base::FeatureList::IsEnabled(
-           features::kMainRepaintScrollPrefersNewContent))) {
+      host_impl_->EvictedUIResourcesExist()) {
     // Once we enter NEW_CONTENTS_TAKES_PRIORITY mode, visible tiles on active
     // tree might be freed. We need to set RequiresHighResToDraw to ensure that
     // high res tiles will be required to activate pending tree.
+    //
+    // TODO(crbug.com/1477299): It would be better to use this path during
+    // main-repainted scrolls (host_impl_->IsCurrentScrollMainRepainted()), but
+    // the RequiresHighResToDraw mode has a page-freezing bug triggered by
+    // animating scaled directly-composited images. To address the urgent
+    // regression, we have restored the pre-crbug.com/1418368 behavior, which
+    // makes main-repainted scrolls use SAME_PRIORITY_FOR_BOTH_TREES.
     host_impl_->SetRequiresHighResToDraw();
-    scheduler_tree_priority = raster_tree_priority = NEW_CONTENT_TAKES_PRIORITY;
+    tree_priority = NEW_CONTENT_TAKES_PRIORITY;
   }
 
-  host_impl_->SetTreePriority(raster_tree_priority);
+  host_impl_->SetTreePriority(tree_priority);
 
   // Only put the scheduler in impl latency prioritization mode if we don't
   // have a scroll listener. This gives the scroll listener a better chance of
@@ -586,7 +539,7 @@ void ProxyImpl::RenewTreePriority() {
       host_impl_->ScrollAffectsScrollHandler()
           ? ScrollHandlerState::SCROLL_AFFECTS_SCROLL_HANDLER
           : ScrollHandlerState::SCROLL_DOES_NOT_AFFECT_SCROLL_HANDLER;
-  scheduler_->SetTreePrioritiesAndScrollState(scheduler_tree_priority,
+  scheduler_->SetTreePrioritiesAndScrollState(tree_priority,
                                               scroll_handler_state);
 }
 
@@ -731,10 +684,6 @@ void ProxyImpl::ScheduledActionSendBeginMainFrame(
     const viz::BeginFrameArgs& args) {
   DCHECK(IsImplThread());
 
-  if (is_jank_injection_enabled_ && host_impl_->CanInjectJankOnMain()) {
-    jank_injector_.ScheduleJankIfNeeded(args, MainThreadTaskRunner());
-  }
-
   benchmark_instrumentation::ScopedBeginFrameTask begin_frame_task(
       benchmark_instrumentation::kSendBeginFrame,
       args.frame_id.sequence_number);
@@ -758,9 +707,15 @@ void ProxyImpl::ScheduledActionSendBeginMainFrame(
       (args.frame_id.sequence_number & 0xffffffff);
   host_impl_->WillSendBeginMainFrame();
   {
-    TRACE_EVENT_WITH_FLOW1(
-        "viz,benchmark", "Graphics.Pipeline", TRACE_ID_GLOBAL(args.trace_id),
-        TRACE_EVENT_FLAG_FLOW_IN, "step", "SendBeginMainFrame");
+    TRACE_EVENT(
+        "viz,benchmark,graphics.pipeline", "Graphics.Pipeline",
+        perfetto::TerminatingFlow::Global(args.trace_id),
+        [&](perfetto::EventContext ctx) {
+          auto* event = ctx.event<perfetto::protos::pbzero::ChromeTrackEvent>();
+          auto* data = event->set_chrome_graphics_pipeline();
+          data->set_step(perfetto::protos::pbzero::ChromeGraphicsPipeline::
+                             StepName::STEP_SEND_BEGIN_MAIN_FRAME);
+        });
     TRACE_EVENT_WITH_FLOW0("viz,benchmark",
                            "MainFrame.SendBeginMainFrameOnImpl",
                            TRACE_ID_LOCAL(begin_main_frame_state->trace_id),
@@ -825,7 +780,6 @@ void ProxyImpl::ScheduledActionCommit() {
   }
 
   data_for_commit_.reset();
-  hung_commit_timer_.Stop();
 }
 
 void ProxyImpl::ScheduledActionPostCommit() {
@@ -982,7 +936,6 @@ base::SingleThreadTaskRunner* ProxyImpl::MainThreadTaskRunner() {
 
 void ProxyImpl::SetSourceURL(ukm::SourceId source_id, const GURL& url) {
   DCHECK(IsImplThread());
-  is_jank_injection_enabled_ = JankInjector::IsEnabled(url);
   host_impl_->SetActiveURL(url, source_id);
 }
 
