@@ -7,6 +7,7 @@
 #include <stddef.h>
 #include <utility>
 
+#include "base/containers/contains.h"
 #include "base/debug/alias.h"
 #include "base/debug/leak_annotations.h"
 #include "base/functional/bind.h"
@@ -20,6 +21,7 @@
 #include "base/threading/thread_restrictions.h"
 #include "dbus/bus.h"
 #include "dbus/dbus_statistics.h"
+#include "dbus/error.h"
 #include "dbus/message.h"
 #include "dbus/object_path.h"
 #include "dbus/scoped_dbus_error.h"
@@ -127,41 +129,26 @@ ObjectProxy::~ObjectProxy() {
 // Originally we tried to make |method_call| a const reference, but we
 // gave up as dbus_connection_send_with_reply_and_block() takes a
 // non-const pointer of DBusMessage as the second parameter.
-std::unique_ptr<Response> ObjectProxy::CallMethodAndBlockWithErrorDetails(
-    MethodCall* method_call,
-    int timeout_ms,
-    ScopedDBusError* error) {
+base::expected<std::unique_ptr<Response>, Error>
+ObjectProxy::CallMethodAndBlock(MethodCall* method_call, int timeout_ms) {
   bus_->AssertOnDBusThread();
 
   if (!bus_->Connect() || !method_call->SetDestination(service_name_) ||
       !method_call->SetPath(object_path_)) {
-    return nullptr;
+    // Not an error from libdbus, so returns invalid error.
+    return base::unexpected(Error());
   }
 
-  DBusMessage* request_message = method_call->raw_message();
-
   // Send the message synchronously.
-  DBusMessage* response_message =
-      bus_->SendWithReplyAndBlock(request_message, timeout_ms, error->get());
-
+  auto result =
+      bus_->SendWithReplyAndBlock(method_call->raw_message(), timeout_ms);
   statistics::AddBlockingSentMethodCall(
       service_name_, method_call->GetInterface(), method_call->GetMember());
-
-  if (!response_message) {
+  if (!result.has_value()) {
     LogMethodCallFailure(method_call->GetInterface(), method_call->GetMember(),
-                         error->is_set() ? error->name() : "unknown error type",
-                         error->is_set() ? error->message() : "");
-    return nullptr;
+                         result.error().name(), result.error().message());
   }
-
-  return Response::FromRawMessage(response_message);
-}
-
-std::unique_ptr<Response> ObjectProxy::CallMethodAndBlock(
-    MethodCall* method_call,
-    int timeout_ms) {
-  ScopedDBusError error;
-  return CallMethodAndBlockWithErrorDetails(method_call, timeout_ms, &error);
+  return result;
 }
 
 void ObjectProxy::CallMethod(MethodCall* method_call,
@@ -301,9 +288,9 @@ void ObjectProxy::Detach() {
     bus_->RemoveFilterFunction(&ObjectProxy::HandleMessageThunk, this);
 
   for (const auto& match_rule : match_rules_) {
-    ScopedDBusError error;
-    bus_->RemoveMatch(match_rule, error.get());
-    if (error.is_set()) {
+    Error error;
+    bus_->RemoveMatch(match_rule, &error);
+    if (error.IsValid()) {
       // There is nothing we can do to recover, so just print the error.
       LOG(ERROR) << "Failed to remove match rule: " << match_rule;
     }
@@ -624,10 +611,10 @@ bool ObjectProxy::AddMatchRuleWithCallback(
   DCHECK(!absolute_signal_name.empty());
   bus_->AssertOnDBusThread();
 
-  if (match_rules_.find(match_rule) == match_rules_.end()) {
-    ScopedDBusError error;
-    bus_->AddMatch(match_rule, error.get());
-    if (error.is_set()) {
+  if (!base::Contains(match_rules_, match_rule)) {
+    dbus::Error error;
+    bus_->AddMatch(match_rule, &error);
+    if (error.IsValid()) {
       LOG(ERROR) << "Failed to add match rule \"" << match_rule << "\". Got "
                  << error.name() << ": " << error.message();
       return false;
@@ -638,11 +625,11 @@ bool ObjectProxy::AddMatchRuleWithCallback(
       method_table_[absolute_signal_name].push_back(signal_callback);
       return true;
     }
-  } else {
-    // We already have the match rule.
-    method_table_[absolute_signal_name].push_back(signal_callback);
-    return true;
   }
+
+  // We already have the match rule.
+  method_table_[absolute_signal_name].push_back(signal_callback);
+  return true;
 }
 
 bool ObjectProxy::AddMatchRuleWithoutCallback(
@@ -652,12 +639,13 @@ bool ObjectProxy::AddMatchRuleWithoutCallback(
   DCHECK(!absolute_signal_name.empty());
   bus_->AssertOnDBusThread();
 
-  if (match_rules_.find(match_rule) != match_rules_.end())
+  if (base::Contains(match_rules_, match_rule)) {
     return true;
+  }
 
-  ScopedDBusError error;
-  bus_->AddMatch(match_rule, error.get());
-  if (error.is_set()) {
+  Error error;
+  bus_->AddMatch(match_rule, &error);
+  if (error.IsValid()) {
     LOG(ERROR) << "Failed to add match rule \"" << match_rule << "\". Got "
                << error.name() << ": " << error.message();
     return false;
