@@ -10,6 +10,7 @@
 #include <memory>
 #include <utility>
 
+#include "base/containers/contains.h"
 #include "base/files/file_util.h"
 #include "base/functional/bind.h"
 #include "base/json/json_reader.h"
@@ -127,8 +128,8 @@ void GenerateMultipartBody(MultipartType multipart_type,
     while (true) {
       boundary = net::GenerateMimeMultipartBoundary();
       bool conflict_with_content = false;
-      for (auto& part : parts) {
-        if (part.data.find(boundary, 0) != std::string::npos) {
+      for (const auto& part : parts) {
+        if (base::Contains(part.data, boundary)) {
           conflict_with_content = true;
           break;
         }
@@ -152,7 +153,7 @@ void GenerateMultipartBody(MultipartType multipart_type,
   output->data.clear();
   if (data_offset)
     data_offset->clear();
-  for (auto& part : parts) {
+  for (const auto& part : parts) {
     output->data.append(base::StringPrintf(
         kMultipartItemHeaderFormat, boundary.c_str(), part.type.c_str()));
     if (data_offset)
