@@ -208,7 +208,7 @@ class OAuth2AccessTokenManager::Fetcher : public OAuth2AccessTokenConsumer {
   // Fetcher, since this Fetcher is destructed in the dtor of the
   // OAuth2AccessTokenManager or is scheduled for deletion at the end of
   // OnGetTokenFailure/OnGetTokenSuccess (whichever comes first).
-  const raw_ptr<OAuth2AccessTokenManager, DanglingUntriaged>
+  const raw_ptr<OAuth2AccessTokenManager, AcrossTasksDanglingUntriaged>
       oauth2_access_token_manager_;
   scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory_;
   const CoreAccountId account_id_;
