@@ -4,6 +4,7 @@
 
 #include "google_apis/gaia/oauth2_access_token_manager.h"
 
+#include "base/containers/contains.h"
 #include "base/memory/ref_counted.h"
 #include "base/test/bind.h"
 #include "base/test/task_environment.h"
@@ -38,8 +39,7 @@ class FakeOAuth2AccessTokenManagerDelegate
       const CoreAccountId& account_id,
       scoped_refptr<network::SharedURLLoaderFactory> url_loader_factory,
       OAuth2AccessTokenConsumer* consumer) override {
-    EXPECT_NE(account_ids_to_refresh_tokens_.find(account_id),
-              account_ids_to_refresh_tokens_.end());
+    EXPECT_TRUE(base::Contains(account_ids_to_refresh_tokens_, account_id));
     return GaiaAccessTokenFetcher::
         CreateExchangeRefreshTokenForAccessTokenInstance(
             consumer, url_loader_factory,
@@ -47,8 +47,7 @@ class FakeOAuth2AccessTokenManagerDelegate
   }
 
   bool HasRefreshToken(const CoreAccountId& account_id) const override {
-    return account_ids_to_refresh_tokens_.find(account_id) !=
-           account_ids_to_refresh_tokens_.end();
+    return base::Contains(account_ids_to_refresh_tokens_, account_id);
   }
 
   scoped_refptr<network::SharedURLLoaderFactory> GetURLLoaderFactory()
