@@ -368,13 +368,13 @@ class HeadlessWebContentsBeginFrameControlTest : public HeadlessBrowserTest {
     command_line->AppendSwitch(::switches::kDisableNewContentRenderingTimeout);
     command_line->AppendSwitch(cc::switches::kDisableCheckerImaging);
     command_line->AppendSwitch(cc::switches::kDisableThreadedAnimation);
-    command_line->AppendSwitch(blink::switches::kDisableThreadedScrolling);
   }
 
   void RunTest() {
-    browser_context_ = browser()->CreateBrowserContextBuilder().Build();
-    browser()->SetDefaultBrowserContext(browser_context_);
-    browser_devtools_client_.AttachToBrowser();
+    browser()->SetDefaultBrowserContext(
+        browser()->CreateBrowserContextBuilder().Build());
+    SimpleDevToolsProtocolClient browser_devtools_client;
+    browser_devtools_client.AttachToBrowser();
 
     EXPECT_TRUE(embedded_test_server()->Start());
 
@@ -383,7 +383,7 @@ class HeadlessWebContentsBeginFrameControlTest : public HeadlessBrowserTest {
     params.Set("width", 200);
     params.Set("height", 200);
     params.Set("enableBeginFrameControl", true);
-    browser_devtools_client_.SendCommand(
+    browser_devtools_client.SendCommand(
         "Target.createTarget", std::move(params),
         base::BindOnce(
             &HeadlessWebContentsBeginFrameControlTest::OnTargetCreated,
@@ -391,7 +391,7 @@ class HeadlessWebContentsBeginFrameControlTest : public HeadlessBrowserTest {
 
     RunAsynchronousTest();
 
-    browser_devtools_client_.DetachClient();
+    browser_devtools_client.DetachClient();
   }
 
   void OnTargetCreated(base::Value::Dict result) {
@@ -402,15 +402,6 @@ class HeadlessWebContentsBeginFrameControlTest : public HeadlessBrowserTest {
         browser()->GetWebContentsForDevToolsAgentHostId(targetId));
 
     devtools_client_.AttachToWebContents(web_contents_->web_contents());
-
-    devtools_client_.SendCommand(
-        "Page.stopLoading",
-        base::BindOnce(
-            &HeadlessWebContentsBeginFrameControlTest::OnLoadingStopped,
-            base::Unretained(this)));
-  }
-
-  void OnLoadingStopped(base::Value::Dict) {
     devtools_client_.AddEventHandler("Page.loadEventFired",
                                      on_load_event_fired_handler_);
 
@@ -435,12 +426,10 @@ class HeadlessWebContentsBeginFrameControlTest : public HeadlessBrowserTest {
     devtools_client_.RemoveEventHandler("Page.loadEventFired",
                                         on_load_event_fired_handler_);
 
-    page_ready_ = true;
     StartFrames();
   }
 
   void BeginFrame(bool screenshot) {
-    frame_in_flight_ = true;
     num_begin_frames_++;
 
     base::Value::Dict params;
@@ -459,17 +448,6 @@ class HeadlessWebContentsBeginFrameControlTest : public HeadlessBrowserTest {
         "has_damage", DictBool(result, "result.hasDamage"),
         "has_screenshot_data", DictString(result, "result.screenshotData"));
 
-    // Post OnFrameFinished call so that any pending OnNeedsBeginFramesChanged
-    // call will be executed first.
-    browser()->BrowserMainThread()->PostTask(
-        FROM_HERE,
-        base::BindOnce(
-            &HeadlessWebContentsBeginFrameControlTest::NotifyOnFrameFinished,
-            base::Unretained(this), std::move(result)));
-  }
-
-  void NotifyOnFrameFinished(base::Value::Dict result) {
-    frame_in_flight_ = false;
     OnFrameFinished(std::move(result));
   }
 
@@ -481,16 +459,11 @@ class HeadlessWebContentsBeginFrameControlTest : public HeadlessBrowserTest {
             base::Unretained(this)));
   }
 
-  raw_ptr<HeadlessBrowserContext, DanglingUntriaged> browser_context_ =
-      nullptr;  // Not owned.
-  raw_ptr<HeadlessWebContentsImpl, DanglingUntriaged> web_contents_ =
+  raw_ptr<HeadlessWebContentsImpl, AcrossTasksDanglingUntriaged> web_contents_ =
       nullptr;  // Not owned.
 
-  bool page_ready_ = false;
-  bool frame_in_flight_ = false;
   int num_begin_frames_ = 0;
 
-  SimpleDevToolsProtocolClient browser_devtools_client_;
   SimpleDevToolsProtocolClient devtools_client_;
 
   SimpleDevToolsProtocolClient::EventCallback on_load_event_fired_handler_ =
@@ -629,7 +602,9 @@ class HeadlessWebContentsBeginFrameControlViewportTest
   }
 };
 
-HEADLESS_DEVTOOLED_TEST_F(HeadlessWebContentsBeginFrameControlViewportTest);
+// TODO(crbug.com/1459385): Turning this off since it's flaking regularly.
+DISABLED_HEADLESS_DEVTOOLED_TEST_F(
+    HeadlessWebContentsBeginFrameControlViewportTest);
 
 #endif  // !BUILDFLAG(IS_MAC)
 
