@@ -7,19 +7,20 @@
 
 #include "net/cert/pki/cert_errors.h"
 #include "net/cert/pki/parsed_certificate.h"
-#include "net/cert/x509_util.h"
-#include "third_party/boringssl/src/include/openssl/span.h"
+#include "third_party/boringssl/src/include/openssl/base.h"
+#include "third_party/boringssl/src/include/openssl/pool.h"
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size) {
   net::CertErrors errors;
   std::shared_ptr<const net::ParsedCertificate> cert =
       net::ParsedCertificate::Create(
-          net::x509_util::CreateCryptoBuffer(base::make_span(data, size)), {},
-          &errors);
+          bssl::UniquePtr<CRYPTO_BUFFER>(
+              CRYPTO_BUFFER_new(data, size, nullptr)),
+          {}, &errors);
 
   // Severe errors must be provided iff the parsing failed.
-  CHECK_EQ(errors.ContainsAnyErrorWithSeverity(net::CertError::SEVERITY_HIGH),
-           cert == nullptr);
+  BSSL_CHECK(errors.ContainsAnyErrorWithSeverity(
+                 net::CertError::SEVERITY_HIGH) == (cert == nullptr));
 
   return 0;
 }
