@@ -12,6 +12,7 @@
 #include "base/files/file_util.h"
 #include "base/hash/sha1.h"
 #include "base/pickle.h"
+#include "base/strings/strcat.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
@@ -33,6 +34,8 @@ using base::Time;
 
 namespace net {
 
+namespace {
+
 // Certificates for test data. They're obtained with:
 //
 // $ openssl s_client -connect [host]:443 -showcerts > /tmp/host.pem < /dev/null
@@ -81,22 +84,18 @@ void CheckGoogleCert(const scoped_refptr<X509Certificate>& google_cert,
   EXPECT_EQ("Mountain View", subject.locality_name);
   EXPECT_EQ("California", subject.state_or_province_name);
   EXPECT_EQ("US", subject.country_name);
-  EXPECT_EQ(0U, subject.street_addresses.size());
   ASSERT_EQ(1U, subject.organization_names.size());
   EXPECT_EQ("Google Inc", subject.organization_names[0]);
   EXPECT_EQ(0U, subject.organization_unit_names.size());
-  EXPECT_EQ(0U, subject.domain_components.size());
 
   const CertPrincipal& issuer = google_cert->issuer();
   EXPECT_EQ("Thawte SGC CA", issuer.common_name);
   EXPECT_EQ("", issuer.locality_name);
   EXPECT_EQ("", issuer.state_or_province_name);
   EXPECT_EQ("ZA", issuer.country_name);
-  EXPECT_EQ(0U, issuer.street_addresses.size());
   ASSERT_EQ(1U, issuer.organization_names.size());
   EXPECT_EQ("Thawte Consulting (Pty) Ltd.", issuer.organization_names[0]);
   EXPECT_EQ(0U, issuer.organization_unit_names.size());
-  EXPECT_EQ(0U, issuer.domain_components.size());
 
   // Use DoubleT because its epoch is the same on all platforms
   const Time& valid_start = google_cert->valid_start();
@@ -110,6 +109,18 @@ void CheckGoogleCert(const scoped_refptr<X509Certificate>& google_cert,
 
 }
 
+void ExpectX509CertificateMembersEqual(
+    const scoped_refptr<X509Certificate>& a,
+    const scoped_refptr<X509Certificate>& b) {
+  EXPECT_TRUE(a->subject().EqualsForTesting(b->subject()));
+  EXPECT_TRUE(a->issuer().EqualsForTesting(b->issuer()));
+  EXPECT_EQ(a->valid_start(), b->valid_start());
+  EXPECT_EQ(a->valid_expiry(), b->valid_expiry());
+  EXPECT_EQ(a->serial_number(), b->serial_number());
+}
+
+}  // namespace
+
 TEST(X509CertificateTest, GoogleCertParsing) {
   scoped_refptr<X509Certificate> google_cert(
       X509Certificate::CreateFromBytes(google_der));
@@ -129,25 +140,21 @@ TEST(X509CertificateTest, WebkitCertParsing) {
   EXPECT_EQ("Cupertino", subject.locality_name);
   EXPECT_EQ("California", subject.state_or_province_name);
   EXPECT_EQ("US", subject.country_name);
-  EXPECT_EQ(0U, subject.street_addresses.size());
   ASSERT_EQ(1U, subject.organization_names.size());
   EXPECT_EQ("Apple Inc.", subject.organization_names[0]);
   ASSERT_EQ(1U, subject.organization_unit_names.size());
   EXPECT_EQ("Mac OS Forge", subject.organization_unit_names[0]);
-  EXPECT_EQ(0U, subject.domain_components.size());
 
   const CertPrincipal& issuer = webkit_cert->issuer();
   EXPECT_EQ("Go Daddy Secure Certification Authority", issuer.common_name);
   EXPECT_EQ("Scottsdale", issuer.locality_name);
   EXPECT_EQ("Arizona", issuer.state_or_province_name);
   EXPECT_EQ("US", issuer.country_name);
-  EXPECT_EQ(0U, issuer.street_addresses.size());
   ASSERT_EQ(1U, issuer.organization_names.size());
   EXPECT_EQ("GoDaddy.com, Inc.", issuer.organization_names[0]);
   ASSERT_EQ(1U, issuer.organization_unit_names.size());
   EXPECT_EQ("http://certificates.godaddy.com/repository",
-      issuer.organization_unit_names[0]);
-  EXPECT_EQ(0U, issuer.domain_components.size());
+            issuer.organization_unit_names[0]);
 
   // Use DoubleT because its epoch is the same on all platforms
   const Time& valid_start = webkit_cert->valid_start();
@@ -181,24 +188,20 @@ TEST(X509CertificateTest, ThawteCertParsing) {
   EXPECT_EQ("Mountain View", subject.locality_name);
   EXPECT_EQ("California", subject.state_or_province_name);
   EXPECT_EQ("US", subject.country_name);
-  EXPECT_EQ(0U, subject.street_addresses.size());
   ASSERT_EQ(1U, subject.organization_names.size());
   EXPECT_EQ("Thawte Inc", subject.organization_names[0]);
   EXPECT_EQ(0U, subject.organization_unit_names.size());
-  EXPECT_EQ(0U, subject.domain_components.size());
 
   const CertPrincipal& issuer = thawte_cert->issuer();
   EXPECT_EQ("thawte Extended Validation SSL CA", issuer.common_name);
   EXPECT_EQ("", issuer.locality_name);
   EXPECT_EQ("", issuer.state_or_province_name);
   EXPECT_EQ("US", issuer.country_name);
-  EXPECT_EQ(0U, issuer.street_addresses.size());
   ASSERT_EQ(1U, issuer.organization_names.size());
   EXPECT_EQ("thawte, Inc.", issuer.organization_names[0]);
   ASSERT_EQ(1U, issuer.organization_unit_names.size());
   EXPECT_EQ("Terms of use at https://www.thawte.com/cps (c)06",
             issuer.organization_unit_names[0]);
-  EXPECT_EQ(0U, issuer.domain_components.size());
 
   // Use DoubleT because its epoch is the same on all platforms
   const Time& valid_start = thawte_cert->valid_start();
@@ -224,13 +227,10 @@ TEST(X509CertificateTest, MultivalueRDN) {
   EXPECT_EQ("", subject.locality_name);
   EXPECT_EQ("", subject.state_or_province_name);
   EXPECT_EQ("US", subject.country_name);
-  EXPECT_EQ(0U, subject.street_addresses.size());
   ASSERT_EQ(1U, subject.organization_names.size());
   EXPECT_EQ("Chromium", subject.organization_names[0]);
   ASSERT_EQ(1U, subject.organization_unit_names.size());
   EXPECT_EQ("Chromium net_unittests", subject.organization_unit_names[0]);
-  ASSERT_EQ(1U, subject.domain_components.size());
-  EXPECT_EQ("Chromium", subject.domain_components[0]);
 }
 
 // Test that characters which would normally be escaped in the string form,
@@ -248,14 +248,11 @@ TEST(X509CertificateTest, UnescapedSpecialCharacters) {
   EXPECT_EQ("Mountain View", subject.locality_name);
   EXPECT_EQ("California", subject.state_or_province_name);
   EXPECT_EQ("US", subject.country_name);
-  ASSERT_EQ(1U, subject.street_addresses.size());
-  EXPECT_EQ("1600 Amphitheatre Parkway", subject.street_addresses[0]);
   ASSERT_EQ(1U, subject.organization_names.size());
   EXPECT_EQ("Chromium = \"net_unittests\"", subject.organization_names[0]);
   ASSERT_EQ(2U, subject.organization_unit_names.size());
   EXPECT_EQ("net_unittests", subject.organization_unit_names[0]);
   EXPECT_EQ("Chromium", subject.organization_unit_names[1]);
-  EXPECT_EQ(0U, subject.domain_components.size());
 }
 
 TEST(X509CertificateTest, InvalidPrintableStringIsUtf8) {
@@ -695,6 +692,79 @@ TEST(X509CertificateTest, Cache) {
             cert3->intermediate_buffers().size());
 }
 
+TEST(X509CertificateTest, CloneWithDifferentIntermediates) {
+  CertificateList certs = CreateCertificateListFromFile(
+      GetTestCertsDirectory(), "multi-root-chain1.pem",
+      X509Certificate::FORMAT_PEM_CERT_SEQUENCE);
+  ASSERT_EQ(4u, certs.size());
+
+  auto leaf_with_no_intermediates = certs[0];
+
+  {
+    auto cloned =
+        leaf_with_no_intermediates->CloneWithDifferentIntermediates({});
+    // Intermediates are equal, so should return a reference to the same object.
+    EXPECT_EQ(leaf_with_no_intermediates.get(), cloned.get());
+  }
+  {
+    std::vector<bssl::UniquePtr<CRYPTO_BUFFER>> intermediates;
+    intermediates.push_back(bssl::UpRef(certs[1]->cert_buffer()));
+    intermediates.push_back(bssl::UpRef(certs[2]->cert_buffer()));
+    auto cloned = leaf_with_no_intermediates->CloneWithDifferentIntermediates(
+        std::move(intermediates));
+    ASSERT_TRUE(cloned);
+    EXPECT_NE(leaf_with_no_intermediates.get(), cloned.get());
+    EXPECT_EQ(leaf_with_no_intermediates->cert_buffer(), cloned->cert_buffer());
+    ExpectX509CertificateMembersEqual(leaf_with_no_intermediates, cloned);
+    ASSERT_EQ(2u, cloned->intermediate_buffers().size());
+    EXPECT_TRUE(x509_util::CryptoBufferEqual(
+        certs[1]->cert_buffer(), cloned->intermediate_buffers()[0].get()));
+    EXPECT_TRUE(x509_util::CryptoBufferEqual(
+        certs[2]->cert_buffer(), cloned->intermediate_buffers()[1].get()));
+  }
+
+  std::vector<bssl::UniquePtr<CRYPTO_BUFFER>> leaf_intermediates;
+  leaf_intermediates.push_back(bssl::UpRef(certs[1]->cert_buffer()));
+  leaf_intermediates.push_back(bssl::UpRef(certs[2]->cert_buffer()));
+  auto leaf_with_intermediates = X509Certificate::CreateFromBuffer(
+      bssl::UpRef(certs[0]->cert_buffer()), std::move(leaf_intermediates));
+  ASSERT_TRUE(leaf_with_intermediates);
+
+  {
+    auto cloned = leaf_with_intermediates->CloneWithDifferentIntermediates({});
+    EXPECT_NE(leaf_with_intermediates.get(), cloned.get());
+    EXPECT_EQ(leaf_with_intermediates->cert_buffer(), cloned->cert_buffer());
+    ExpectX509CertificateMembersEqual(leaf_with_intermediates, cloned);
+    ASSERT_EQ(0u, cloned->intermediate_buffers().size());
+  }
+  {
+    std::vector<bssl::UniquePtr<CRYPTO_BUFFER>> intermediates;
+    intermediates.push_back(bssl::UpRef(certs[1]->cert_buffer()));
+    intermediates.push_back(bssl::UpRef(certs[2]->cert_buffer()));
+    auto cloned = leaf_with_intermediates->CloneWithDifferentIntermediates(
+        std::move(intermediates));
+    // Intermediates are equal, so should return a reference to the same object.
+    EXPECT_EQ(leaf_with_intermediates.get(), cloned.get());
+  }
+  {
+    std::vector<bssl::UniquePtr<CRYPTO_BUFFER>> intermediates;
+    intermediates.push_back(bssl::UpRef(certs[2]->cert_buffer()));
+    intermediates.push_back(bssl::UpRef(certs[1]->cert_buffer()));
+    auto cloned = leaf_with_intermediates->CloneWithDifferentIntermediates(
+        std::move(intermediates));
+    // Intermediates are different (same buffers but in different order).
+    ASSERT_TRUE(cloned);
+    EXPECT_NE(leaf_with_intermediates.get(), cloned.get());
+    EXPECT_EQ(leaf_with_intermediates->cert_buffer(), cloned->cert_buffer());
+    ExpectX509CertificateMembersEqual(leaf_with_intermediates, cloned);
+    ASSERT_EQ(2u, cloned->intermediate_buffers().size());
+    EXPECT_TRUE(x509_util::CryptoBufferEqual(
+        certs[2]->cert_buffer(), cloned->intermediate_buffers()[0].get()));
+    EXPECT_TRUE(x509_util::CryptoBufferEqual(
+        certs[1]->cert_buffer(), cloned->intermediate_buffers()[1].get()));
+  }
+}
+
 TEST(X509CertificateTest, Pickle) {
   bssl::UniquePtr<CRYPTO_BUFFER> google_cert_handle =
       x509_util::CreateCryptoBuffer(google_der);
@@ -1328,17 +1398,15 @@ INSTANTIATE_TEST_SUITE_P(All,
                          testing::ValuesIn(kNameVerifyTestData));
 
 const struct PublicKeyInfoTestData {
-  const char* cert_file;
+  const char* file_name;
   size_t expected_bits;
   X509Certificate::PublicKeyType expected_type;
 } kPublicKeyInfoTestData[] = {
-    {"768-rsa-ee-by-768-rsa-intermediate.pem", 768,
-     X509Certificate::kPublicKeyTypeRSA},
-    {"1024-rsa-ee-by-768-rsa-intermediate.pem", 1024,
-     X509Certificate::kPublicKeyTypeRSA},
-    {"prime256v1-ecdsa-ee-by-1024-rsa-intermediate.pem", 256,
-     X509Certificate::kPublicKeyTypeECDSA},
-    {"large_key.pem", 8200, X509Certificate::kPublicKeyTypeRSA},
+    {"rsa-768", 768, X509Certificate::kPublicKeyTypeRSA},
+    {"rsa-1024", 1024, X509Certificate::kPublicKeyTypeRSA},
+    {"rsa-2048", 2048, X509Certificate::kPublicKeyTypeRSA},
+    {"rsa-8200", 8200, X509Certificate::kPublicKeyTypeRSA},
+    {"ec-prime256v1", 256, X509Certificate::kPublicKeyTypeECDSA},
 };
 
 class X509CertificatePublicKeyInfoTest
@@ -1348,15 +1416,16 @@ class X509CertificatePublicKeyInfoTest
 TEST_P(X509CertificatePublicKeyInfoTest, GetPublicKeyInfo) {
   PublicKeyInfoTestData data = GetParam();
 
-  scoped_refptr<X509Certificate> cert(
-      ImportCertFromFile(GetTestCertsDirectory(), data.cert_file));
-  ASSERT_TRUE(cert.get());
+  auto [leaf, root] = CertBuilder::CreateSimpleChain2();
+
+  ASSERT_TRUE(leaf->UseKeyFromFile(GetTestCertsDirectory().AppendASCII(
+      base::StrCat({data.file_name, "-1.key"}))));
 
   size_t actual_bits = 0;
   X509Certificate::PublicKeyType actual_type =
       X509Certificate::kPublicKeyTypeUnknown;
 
-  X509Certificate::GetPublicKeyInfo(cert->cert_buffer(), &actual_bits,
+  X509Certificate::GetPublicKeyInfo(leaf->GetCertBuffer(), &actual_bits,
                                     &actual_type);
 
   EXPECT_EQ(data.expected_bits, actual_bits);
