@@ -30,18 +30,15 @@ std::string BytesForNSSCert(CERTCertificate* cert) {
 
 TEST(X509UtilNSSTest, IsSameCertificate) {
   ScopedCERTCertificate google_nss_cert(
-      x509_util::CreateCERTCertificateFromBytes(google_der,
-                                                std::size(google_der)));
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(google_nss_cert);
 
   ScopedCERTCertificate google_nss_cert2(
-      x509_util::CreateCERTCertificateFromBytes(google_der,
-                                                std::size(google_der)));
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(google_nss_cert2);
 
   ScopedCERTCertificate webkit_nss_cert(
-      x509_util::CreateCERTCertificateFromBytes(webkit_der,
-                                                std::size(webkit_der)));
+      x509_util::CreateCERTCertificateFromBytes(webkit_der));
   ASSERT_TRUE(webkit_nss_cert);
 
   scoped_refptr<X509Certificate> google_x509_cert(
@@ -77,8 +74,8 @@ TEST(X509UtilNSSTest, IsSameCertificate) {
 }
 
 TEST(X509UtilNSSTest, CreateCERTCertificateFromBytes) {
-  ScopedCERTCertificate google_cert(x509_util::CreateCERTCertificateFromBytes(
-      google_der, std::size(google_der)));
+  ScopedCERTCertificate google_cert(
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(google_cert);
   EXPECT_STREQ(
       "CN=www.google.com,O=Google Inc,L=Mountain View,ST=California,C=US",
@@ -86,11 +83,11 @@ TEST(X509UtilNSSTest, CreateCERTCertificateFromBytes) {
 }
 
 TEST(X509UtilNSSTest, CreateCERTCertificateFromBytesGarbage) {
-  static const uint8_t garbage_data[] = "garbage";
-  EXPECT_EQ(nullptr,
-            x509_util::CreateCERTCertificateFromBytes(garbage_data, 0));
   EXPECT_EQ(nullptr, x509_util::CreateCERTCertificateFromBytes(
-                         garbage_data, std::size(garbage_data)));
+                         base::span<const uint8_t>()));
+
+  static const uint8_t garbage_data[] = "garbage";
+  EXPECT_EQ(nullptr, x509_util::CreateCERTCertificateFromBytes(garbage_data));
 }
 
 TEST(X509UtilNSSTest, CreateCERTCertificateFromX509Certificate) {
@@ -192,8 +189,8 @@ TEST(X509UtilNSSTest, CreateCERTCertificateListFromBytes) {
 }
 
 TEST(X509UtilNSSTest, DupCERTCertificate) {
-  ScopedCERTCertificate cert(x509_util::CreateCERTCertificateFromBytes(
-      google_der, std::size(google_der)));
+  ScopedCERTCertificate cert(
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(cert);
 
   ScopedCERTCertificate cert2 = x509_util::DupCERTCertificate(cert.get());
@@ -209,11 +206,11 @@ TEST(X509UtilNSSTest, DupCERTCertificate) {
 }
 
 TEST(X509UtilNSSTest, DupCERTCertificateList) {
-  ScopedCERTCertificate cert(x509_util::CreateCERTCertificateFromBytes(
-      google_der, std::size(google_der)));
+  ScopedCERTCertificate cert(
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(cert);
-  ScopedCERTCertificate cert2(x509_util::CreateCERTCertificateFromBytes(
-      webkit_der, std::size(webkit_der)));
+  ScopedCERTCertificate cert2(
+      x509_util::CreateCERTCertificateFromBytes(webkit_der));
   ASSERT_TRUE(cert2);
   ScopedCERTCertificateList certs;
   certs.push_back(std::move(cert));
@@ -242,8 +239,8 @@ TEST(X509UtilNSSTest, DupCERTCertificateList_EmptyList) {
 }
 
 TEST(X509UtilNSSTest, CreateX509CertificateFromCERTCertificate_NoChain) {
-  ScopedCERTCertificate nss_cert(x509_util::CreateCERTCertificateFromBytes(
-      google_der, std::size(google_der)));
+  ScopedCERTCertificate nss_cert(
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(nss_cert);
   scoped_refptr<X509Certificate> x509_cert =
       x509_util::CreateX509CertificateFromCERTCertificate(nss_cert.get());
@@ -253,8 +250,8 @@ TEST(X509UtilNSSTest, CreateX509CertificateFromCERTCertificate_NoChain) {
 }
 
 TEST(X509UtilNSSTest, CreateX509CertificateFromCERTCertificate_EmptyChain) {
-  ScopedCERTCertificate nss_cert(x509_util::CreateCERTCertificateFromBytes(
-      google_der, std::size(google_der)));
+  ScopedCERTCertificate nss_cert(
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(nss_cert);
   scoped_refptr<X509Certificate> x509_cert =
       x509_util::CreateX509CertificateFromCERTCertificate(
@@ -265,11 +262,11 @@ TEST(X509UtilNSSTest, CreateX509CertificateFromCERTCertificate_EmptyChain) {
 }
 
 TEST(X509UtilNSSTest, CreateX509CertificateFromCERTCertificate_WithChain) {
-  ScopedCERTCertificate nss_cert(x509_util::CreateCERTCertificateFromBytes(
-      google_der, std::size(google_der)));
+  ScopedCERTCertificate nss_cert(
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(nss_cert);
-  ScopedCERTCertificate nss_cert2(x509_util::CreateCERTCertificateFromBytes(
-      webkit_der, std::size(webkit_der)));
+  ScopedCERTCertificate nss_cert2(
+      x509_util::CreateCERTCertificateFromBytes(webkit_der));
   ASSERT_TRUE(nss_cert2);
 
   std::vector<CERTCertificate*> chain;
@@ -287,11 +284,11 @@ TEST(X509UtilNSSTest, CreateX509CertificateFromCERTCertificate_WithChain) {
 }
 
 TEST(X509UtilNSSTest, CreateX509CertificateListFromCERTCertificates) {
-  ScopedCERTCertificate nss_cert(x509_util::CreateCERTCertificateFromBytes(
-      google_der, std::size(google_der)));
+  ScopedCERTCertificate nss_cert(
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(nss_cert);
-  ScopedCERTCertificate nss_cert2(x509_util::CreateCERTCertificateFromBytes(
-      webkit_der, std::size(webkit_der)));
+  ScopedCERTCertificate nss_cert2(
+      x509_util::CreateCERTCertificateFromBytes(webkit_der));
   ASSERT_TRUE(nss_cert2);
   ScopedCERTCertificateList nss_certs;
   nss_certs.push_back(std::move(nss_cert));
@@ -315,8 +312,8 @@ TEST(X509UtilNSSTest, CreateX509CertificateListFromCERTCertificates_EmptyList) {
 }
 
 TEST(X509UtilNSSTest, GetDEREncoded) {
-  ScopedCERTCertificate google_cert(x509_util::CreateCERTCertificateFromBytes(
-      google_der, std::size(google_der)));
+  ScopedCERTCertificate google_cert(
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(google_cert);
   std::string der_encoded;
   ASSERT_TRUE(x509_util::GetDEREncoded(google_cert.get(), &der_encoded));
@@ -398,8 +395,8 @@ TEST(X509UtilNSSTest, ParseClientSubjectAltNames) {
 }
 
 TEST(X509UtilNSSTest, GetValidityTimes) {
-  ScopedCERTCertificate google_cert(x509_util::CreateCERTCertificateFromBytes(
-      google_der, std::size(google_der)));
+  ScopedCERTCertificate google_cert(
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(google_cert);
 
   base::Time not_before, not_after;
@@ -414,8 +411,8 @@ TEST(X509UtilNSSTest, GetValidityTimes) {
 }
 
 TEST(X509UtilNSSTest, GetValidityTimesOptionalArgs) {
-  ScopedCERTCertificate google_cert(x509_util::CreateCERTCertificateFromBytes(
-      google_der, std::size(google_der)));
+  ScopedCERTCertificate google_cert(
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(google_cert);
 
   base::Time not_before;
@@ -438,8 +435,8 @@ TEST(X509UtilNSSTest, CalculateFingerprint256) {
        0xaa, 0x53, 0x32, 0x80, 0x8f, 0xbf, 0x8a, 0x24, 0x7d, 0x98, 0xec,
        0x7f, 0x77, 0x49, 0x38, 0x42, 0x81, 0x26, 0x7f, 0xed, 0x38}};
 
-  ScopedCERTCertificate google_cert(x509_util::CreateCERTCertificateFromBytes(
-      google_der, std::size(google_der)));
+  ScopedCERTCertificate google_cert(
+      x509_util::CreateCERTCertificateFromBytes(google_der));
   ASSERT_TRUE(google_cert);
 
   EXPECT_EQ(google_fingerprint,
