@@ -120,6 +120,10 @@ class NET_EXPORT CanonicalCookie {
   // will be unpartitioned even when the cookie line has the Partitioned
   // attribute.
   //
+  // The `block_truncated` argument indicates whether the '\0', '\n', and '\r'
+  // characters should cause the cookie to fail to be created if present
+  // (instead of truncating `cookie_line` at the first occurrence).
+  //
   // If a cookie is returned, |cookie->IsCanonical()| will be true.
   static std::unique_ptr<CanonicalCookie> Create(
       const GURL& url,
@@ -127,6 +131,7 @@ class NET_EXPORT CanonicalCookie {
       const base::Time& creation_time,
       absl::optional<base::Time> server_time,
       absl::optional<CookiePartitionKey> cookie_partition_key,
+      bool block_truncated = true,
       CookieInclusionStatus* status = nullptr);
 
   // Create a canonical cookie based on sanitizing the passed inputs in the
@@ -396,7 +401,7 @@ class NET_EXPORT CanonicalCookie {
                                     const base::Time& server_time);
 
   // Per rfc6265bis the maximum expiry date is no further than 400 days in the
-  // future. Clamping only occurs when kClampCookieExpiryTo400Days is enabled.
+  // future.
   static base::Time ValidateAndAdjustExpiryDate(
       const base::Time& expiry_date,
       const base::Time& creation_date);
@@ -462,6 +467,8 @@ class NET_EXPORT CanonicalCookie {
   static std::string BuildCookieAttributesLine(const CanonicalCookie& cookie);
 
  private:
+  FRIEND_TEST_ALL_PREFIXES(CanonicalCookieTest,
+                           TestGetAndAdjustPortForTrustworthyUrls);
   FRIEND_TEST_ALL_PREFIXES(CanonicalCookieTest, TestPrefixHistograms);
   FRIEND_TEST_ALL_PREFIXES(CanonicalCookieTest, TestHasHiddenPrefixName);
 
@@ -516,6 +523,19 @@ class NET_EXPORT CanonicalCookie {
   CookieEffectiveSameSite GetEffectiveSameSite(
       CookieAccessSemantics access_semantics) const;
 
+  // Returns the appropriate port value for the given `source_url` depending on
+  // if the url is considered trustworthy or not.
+  //
+  // This function normally returns source_url.EffectiveIntPort(), but it can
+  // return a different port value if:
+  // * `source_url`'s scheme isn't cryptographically secure
+  // * `url_is_trustworthy` is true
+  // * `source_url`'s port is the default port for the scheme i.e.: 80
+  // If all these conditions are true then the returned value will be 443 to
+  // indicate that we're treating `source_url` as if it was secure.
+  static int GetAndAdjustPortForTrustworthyUrls(const GURL& source_url,
+                                                bool url_is_trustworthy);
+
   // Checks for values that could be misinterpreted as a cookie name prefix.
   static bool HasHiddenPrefixName(const base::StringPiece cookie_value);
 
