@@ -109,39 +109,33 @@ TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchSessionControl) {
   EXPECT_TRUE(delete_info.Matches(
       *persistent_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_TRUE(delete_info.Matches(
       *session_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   delete_info.session_control =
       CookieDeletionInfo::SessionControl::PERSISTENT_COOKIES;
   EXPECT_TRUE(delete_info.Matches(
       *persistent_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       *session_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   delete_info.session_control =
       CookieDeletionInfo::SessionControl::SESSION_COOKIES;
   EXPECT_FALSE(delete_info.Matches(
       *persistent_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_TRUE(delete_info.Matches(
       *session_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 }
 
 TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchHost) {
@@ -176,49 +170,41 @@ TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchHost) {
   EXPECT_TRUE(delete_info.Matches(
       *domain_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_TRUE(delete_info.Matches(
       *host_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   delete_info.host = "thehost.hosting.com";
   EXPECT_FALSE(delete_info.Matches(
       *domain_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_TRUE(delete_info.Matches(
       *host_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   delete_info.host = "otherhost.hosting.com";
   EXPECT_FALSE(delete_info.Matches(
       *domain_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       *host_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   delete_info.host = "thehost.otherhosting.com";
   EXPECT_FALSE(delete_info.Matches(
       *domain_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       *host_cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 }
 
 TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchName) {
@@ -250,13 +236,11 @@ TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchName) {
   EXPECT_TRUE(delete_info.Matches(
       *cookie1,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       *cookie2,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 }
 
 TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchValue) {
@@ -288,13 +272,11 @@ TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchValue) {
   EXPECT_FALSE(delete_info.Matches(
       *cookie1,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_TRUE(delete_info.Matches(
       *cookie2,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 }
 
 TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchUrl) {
@@ -315,31 +297,27 @@ TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchUrl) {
   EXPECT_TRUE(delete_info.Matches(
       *cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   delete_info.url = GURL("https://www.example.com/another/path");
   EXPECT_FALSE(delete_info.Matches(
       *cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   delete_info.url = GURL("http://www.example.com/path");
   // Secure cookie on http:// URL -> no match.
   EXPECT_FALSE(delete_info.Matches(
       *cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   // Secure cookie on http:// URL, but delegate says treat is as trustworhy ->
   // match.
   EXPECT_TRUE(delete_info.Matches(
       *cookie,
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/true,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/true}));
 }
 
 TEST(CookieDeletionInfoTest, CookieDeletionInfoDomainMatchesDomain) {
@@ -370,8 +348,7 @@ TEST(CookieDeletionInfoTest, CookieDeletionInfoDomainMatchesDomain) {
   EXPECT_TRUE(delete_info.Matches(
       create_cookie("example.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   const char kExtensionHostname[] = "mgndgikekgjfcpckkfioiadnlibdjbkf";
 
@@ -382,38 +359,31 @@ TEST(CookieDeletionInfoTest, CookieDeletionInfoDomainMatchesDomain) {
   EXPECT_TRUE(delete_info.Matches(
       create_cookie(".example.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_TRUE(delete_info.Matches(
       create_cookie("example.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_TRUE(delete_info.Matches(
       create_cookie(".another.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_TRUE(delete_info.Matches(
       create_cookie("192.168.0.1"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       create_cookie(".nomatch.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       create_cookie("192.168.0.2"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       create_cookie(kExtensionHostname),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 }
 
 TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchesDomainList) {
@@ -439,41 +409,34 @@ TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchesDomainList) {
   EXPECT_TRUE(delete_info.Matches(
       create_cookie("anything.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   // With only an "to_delete" list.
-  delete_info.domains_and_ips_to_delete =
-      std::set<std::string>({"includea.com", "includeb.com"});
+  delete_info.domains_and_ips_to_delete = {"includea.com", "includeb.com"};
   EXPECT_TRUE(delete_info.Matches(
       create_cookie("includea.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_TRUE(delete_info.Matches(
       create_cookie("includeb.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       create_cookie("anything.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   // With only an "to_ignore" list.
-  delete_info.domains_and_ips_to_delete.clear();
-  delete_info.domains_and_ips_to_ignore.insert("exclude.com");
+  delete_info.domains_and_ips_to_delete.reset();
+  delete_info.domains_and_ips_to_ignore = {"exclude.com"};
   EXPECT_TRUE(delete_info.Matches(
       create_cookie("anything.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       create_cookie("exclude.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 
   // Now with both lists populated.
   //
@@ -487,31 +450,41 @@ TEST(CookieDeletionInfoTest, CookieDeletionInfoMatchesDomainList) {
   //              |           right.com |
   //              |                     |
   //              +---------------------+
-  delete_info.domains_and_ips_to_delete =
-      std::set<std::string>({"left.com", "mid.com"});
-  delete_info.domains_and_ips_to_ignore =
-      std::set<std::string>({"mid.com", "right.com"});
+  delete_info.domains_and_ips_to_delete = {"left.com", "mid.com"};
+  delete_info.domains_and_ips_to_ignore = {"mid.com", "right.com"};
 
   EXPECT_TRUE(delete_info.Matches(
       create_cookie("left.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       create_cookie("mid.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       create_cookie("right.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_FALSE(delete_info.Matches(
       create_cookie("outside.com"),
       CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
+
+  // An empty list of deleted domains shouldn't delete anything.
+  delete_info.domains_and_ips_to_delete = std::set<std::string>();
+  delete_info.domains_and_ips_to_ignore.reset();
+  EXPECT_FALSE(delete_info.Matches(
+      create_cookie("outside.com"),
+      CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
+                         /*delegate_treats_url_as_trustworthy=*/false}));
+
+  // An empty list of ignored domains should delete everything.
+  delete_info.domains_and_ips_to_delete.reset();
+  delete_info.domains_and_ips_to_ignore = std::set<std::string>();
+  EXPECT_TRUE(delete_info.Matches(
+      create_cookie("inside.com"),
+      CookieAccessParams{net::CookieAccessSemantics::UNKNOWN,
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 }
 
 // Test that Matches() works regardless of the cookie access semantics (because
@@ -528,18 +501,15 @@ TEST(CookieDeletionInfoTest, MatchesWithCookieAccessSemantics) {
   EXPECT_TRUE(delete_info.Matches(
       *cookie,
       CookieAccessParams{CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_TRUE(delete_info.Matches(
       *cookie,
       CookieAccessParams{CookieAccessSemantics::LEGACY,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
   EXPECT_TRUE(delete_info.Matches(
       *cookie,
       CookieAccessParams{CookieAccessSemantics::NONLEGACY,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+                         /*delegate_treats_url_as_trustworthy=*/false}));
 }
 
 TEST(CookieDeletionInfoTest, MatchesCookiePartitionKeyCollection) {
@@ -592,13 +562,11 @@ TEST(CookieDeletionInfoTest, MatchesCookiePartitionKeyCollection) {
     CookieDeletionInfo delete_info;
     delete_info.cookie_partition_key_collection =
         test_case.filter_cookie_partition_key_collection;
-    EXPECT_EQ(
-        test_case.expects_match,
-        delete_info.Matches(
-            *cookie, CookieAccessParams{
-                         net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+    EXPECT_EQ(test_case.expects_match,
+              delete_info.Matches(
+                  *cookie, CookieAccessParams{
+                               net::CookieAccessSemantics::UNKNOWN,
+                               /*delegate_treats_url_as_trustworthy=*/false}));
   }
 }
 
@@ -635,13 +603,11 @@ TEST(CookieDeletionInfoTest, MatchesExcludeUnpartitionedCookies) {
         /*server_time=*/absl::nullopt, test_case.cookie_partition_key);
     CookieDeletionInfo delete_info;
     delete_info.partitioned_state_only = test_case.partitioned_state_only;
-    EXPECT_EQ(
-        test_case.expects_match,
-        delete_info.Matches(
-            *cookie, CookieAccessParams{
-                         net::CookieAccessSemantics::UNKNOWN,
-                         /*delegate_treats_url_as_trustworthy=*/false,
-                         CookieSamePartyStatus::kNoSamePartyEnforcement}));
+    EXPECT_EQ(test_case.expects_match,
+              delete_info.Matches(
+                  *cookie, CookieAccessParams{
+                               net::CookieAccessSemantics::UNKNOWN,
+                               /*delegate_treats_url_as_trustworthy=*/false}));
   }
 }
 
