@@ -19,7 +19,6 @@
 #include "net/cookies/cookie_constants.h"
 #include "net/cookies/cookie_options.h"
 #include "net/cookies/cookie_util.h"
-#include "net/first_party_sets/same_party_context.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/abseil-cpp/absl/types/optional.h"
@@ -1814,180 +1813,6 @@ TEST(CookieUtilTest, IsCookieAccessResultInclude) {
   EXPECT_TRUE(cookie_util::IsCookieAccessResultInclude(CookieAccessResult()));
 }
 
-TEST(CookieUtilTest, GetSamePartyStatus_NotInSet) {
-  const bool same_party_attribute_enabled = true;
-  CookieOptions options;
-  options.set_is_in_nontrivial_first_party_set(false);
-
-  for (bool same_party : {false, true}) {
-    for (bool secure : {false, true}) {
-      for (bool httponly : {false, true}) {
-        for (CookieSameSite same_site : {
-                 CookieSameSite::NO_RESTRICTION,
-                 CookieSameSite::LAX_MODE,
-                 CookieSameSite::STRICT_MODE,
-                 CookieSameSite::UNSPECIFIED,
-             }) {
-          for (SamePartyContext::Type party_context_type : {
-                   SamePartyContext::Type::kCrossParty,
-                   SamePartyContext::Type::kSameParty,
-               }) {
-            base::Time now = base::Time::Now();
-            std::unique_ptr<CanonicalCookie> cookie =
-                CanonicalCookie::CreateUnsafeCookieForTesting(
-                    "cookie", "tasty", "example.test", "/", now, now, now, now,
-                    secure, httponly, same_site,
-                    CookiePriority::COOKIE_PRIORITY_DEFAULT, same_party);
-
-            options.set_same_party_context(
-                SamePartyContext(party_context_type));
-            EXPECT_EQ(CookieSamePartyStatus::kNoSamePartyEnforcement,
-                      cookie_util::GetSamePartyStatus(
-                          *cookie, options, same_party_attribute_enabled));
-          }
-        }
-      }
-    }
-  }
-}
-
-TEST(CookieUtilTest, GetSamePartyStatus_FeatureDisabled) {
-  const bool same_party_attribute_enabled = false;
-  CookieOptions options;
-  options.set_is_in_nontrivial_first_party_set(true);
-
-  for (bool same_party : {false, true}) {
-    for (bool secure : {false, true}) {
-      for (bool httponly : {false, true}) {
-        for (CookieSameSite same_site : {
-                 CookieSameSite::NO_RESTRICTION,
-                 CookieSameSite::LAX_MODE,
-                 CookieSameSite::STRICT_MODE,
-                 CookieSameSite::UNSPECIFIED,
-             }) {
-          for (SamePartyContext::Type party_context_type : {
-                   SamePartyContext::Type::kCrossParty,
-                   SamePartyContext::Type::kSameParty,
-               }) {
-            base::Time now = base::Time::Now();
-            std::unique_ptr<CanonicalCookie> cookie =
-                CanonicalCookie::CreateUnsafeCookieForTesting(
-                    "cookie", "tasty", "example.test", "/", now, now, now, now,
-                    secure, httponly, same_site,
-                    CookiePriority::COOKIE_PRIORITY_DEFAULT, same_party);
-
-            options.set_same_party_context(
-                SamePartyContext(party_context_type));
-            EXPECT_EQ(CookieSamePartyStatus::kNoSamePartyEnforcement,
-                      cookie_util::GetSamePartyStatus(
-                          *cookie, options, same_party_attribute_enabled));
-          }
-        }
-      }
-    }
-  }
-}
-
-TEST(CookieUtilTest, GetSamePartyStatus_NotSameParty) {
-  CookieOptions options;
-  options.set_is_in_nontrivial_first_party_set(true);
-
-  for (bool secure : {false, true}) {
-    for (bool httponly : {false, true}) {
-      for (CookieSameSite same_site : {
-               CookieSameSite::NO_RESTRICTION,
-               CookieSameSite::LAX_MODE,
-               CookieSameSite::STRICT_MODE,
-               CookieSameSite::UNSPECIFIED,
-           }) {
-        for (SamePartyContext::Type party_context_type : {
-                 SamePartyContext::Type::kCrossParty,
-                 SamePartyContext::Type::kSameParty,
-             }) {
-          base::Time now = base::Time::Now();
-          std::unique_ptr<CanonicalCookie> cookie =
-              CanonicalCookie::CreateUnsafeCookieForTesting(
-                  "cookie", "tasty", "example.test", "/", now, now, now, now,
-                  secure, httponly, same_site,
-                  CookiePriority::COOKIE_PRIORITY_DEFAULT,
-                  false /* same_party */);
-
-          options.set_same_party_context(SamePartyContext(party_context_type));
-          EXPECT_EQ(CookieSamePartyStatus::kNoSamePartyEnforcement,
-                    cookie_util::GetSamePartyStatus(
-                        *cookie, options,
-                        /*same_party_attribute_enabled=*/true));
-        }
-      }
-    }
-  }
-}
-
-TEST(CookieUtilTest, GetSamePartyStatus_SamePartySemantics) {
-  CookieOptions options;
-  options.set_is_in_nontrivial_first_party_set(true);
-
-  // Note: some SameParty cookie configurations (e.g. non-Secure cookies) are
-  // skipped, because they are invalid.
-  for (bool httponly : {false, true}) {
-    for (CookieSameSite same_site : {
-             CookieSameSite::NO_RESTRICTION,
-             CookieSameSite::LAX_MODE,
-             CookieSameSite::UNSPECIFIED,
-         }) {
-      for (CookieOptions::SameSiteCookieContext::ContextType same_site_context :
-           {
-               CookieOptions::SameSiteCookieContext::ContextType::CROSS_SITE,
-               CookieOptions::SameSiteCookieContext::ContextType::SAME_SITE_LAX,
-               CookieOptions::SameSiteCookieContext::ContextType::
-                   SAME_SITE_LAX_METHOD_UNSAFE,
-               CookieOptions::SameSiteCookieContext::ContextType::
-                   SAME_SITE_STRICT,
-           }) {
-        for (CookieOptions::SameSiteCookieContext::ContextType
-                 schemeful_same_site_context :
-             {
-                 CookieOptions::SameSiteCookieContext::ContextType::CROSS_SITE,
-                 CookieOptions::SameSiteCookieContext::ContextType::
-                     SAME_SITE_LAX,
-                 CookieOptions::SameSiteCookieContext::ContextType::
-                     SAME_SITE_LAX_METHOD_UNSAFE,
-                 CookieOptions::SameSiteCookieContext::ContextType::
-                     SAME_SITE_STRICT,
-             }) {
-          if (same_site_context < schemeful_same_site_context)
-            continue;
-          options.set_same_site_cookie_context(
-              CookieOptions::SameSiteCookieContext(
-                  same_site_context, schemeful_same_site_context));
-
-          base::Time now = base::Time::Now();
-          std::unique_ptr<CanonicalCookie> cookie =
-              CanonicalCookie::CreateUnsafeCookieForTesting(
-                  "cookie", "tasty", "example.test", "/", now, now, now, now,
-                  true /* secure */, httponly, same_site,
-                  CookiePriority::COOKIE_PRIORITY_DEFAULT,
-                  true /* same_party */);
-
-          options.set_same_party_context(
-              SamePartyContext(SamePartyContext::Type::kCrossParty));
-          EXPECT_EQ(CookieSamePartyStatus::kEnforceSamePartyExclude,
-                    cookie_util::GetSamePartyStatus(
-                        *cookie, options,
-                        /*same_party_attribute_enabled=*/true));
-
-          options.set_same_party_context(
-              SamePartyContext(SamePartyContext::Type::kSameParty));
-          EXPECT_EQ(CookieSamePartyStatus::kEnforceSamePartyInclude,
-                    cookie_util::GetSamePartyStatus(
-                        *cookie, options,
-                        /*same_party_attribute_enabled=*/true));
-        }
-      }
-    }
-  }
-}
-
 }  // namespace
 
 }  // namespace net
