@@ -395,7 +395,6 @@ class BidirectionalStreamQuicImplTest
       public WithTaskEnvironment {
  protected:
   static const bool kFin = true;
-  static const bool kIncludeVersion = true;
 
   // Holds a packet to be written to the wire, and the IO mode that should
   // be used by the mock socket when performing the write.
@@ -404,7 +403,7 @@ class BidirectionalStreamQuicImplTest
         : mode(mode), packet(packet) {}
     PacketToWrite(IoMode mode, int rv) : mode(mode), packet(nullptr), rv(rv) {}
     IoMode mode;
-    raw_ptr<quic::QuicReceivedPacket> packet;
+    raw_ptr<quic::QuicReceivedPacket, DanglingUntriaged> packet;
     int rv;
   };
 
@@ -535,8 +534,8 @@ class BidirectionalStreamQuicImplTest
             kQuicYieldAfterDurationMilliseconds),
         /*cert_verify_flags=*/0, quic::test::DefaultQuicConfig(),
         std::make_unique<TestQuicCryptoClientConfigHandle>(&crypto_config_),
-        "CONNECTION_UNKNOWN", dns_start, dns_end,
-        std::make_unique<quic::QuicClientPushPromiseIndex>(), nullptr,
+        dns_start, dns_end,
+        std::make_unique<quic::QuicClientPushPromiseIndex>(),
         base::DefaultTickClock::GetInstance(),
         base::SingleThreadTaskRunner::GetCurrentDefault().get(),
         /*socket_performance_watcher=*/nullptr, HostResolverEndpointResult(),
@@ -570,23 +569,20 @@ class BidirectionalStreamQuicImplTest
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructServerDataPacket(
       uint64_t packet_number,
-      bool should_include_version,
       bool fin,
       absl::string_view data) {
     std::unique_ptr<quic::QuicReceivedPacket> packet(
-        server_maker_.MakeDataPacket(packet_number, stream_id_,
-                                     should_include_version, fin, data));
+        server_maker_.MakeDataPacket(packet_number, stream_id_, fin, data));
     DVLOG(2) << "packet(" << packet_number << "): " << std::endl
              << quiche::QuicheTextUtils::HexDump(packet->AsStringPiece());
     return packet;
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructClientDataPacket(
-      bool should_include_version,
       bool fin,
       absl::string_view data) {
-    return client_maker_.MakeDataPacket(++packet_number_, stream_id_,
-                                        should_include_version, fin, data);
+    return client_maker_.MakeDataPacket(++packet_number_, stream_id_, fin,
+                                        data);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructRequestHeadersPacket(
@@ -606,7 +602,7 @@ class BidirectionalStreamQuicImplTest
         ConvertRequestPriorityToQuicPriority(request_priority);
     std::unique_ptr<quic::QuicReceivedPacket> packet(
         client_maker_.MakeRequestHeadersPacket(
-            ++packet_number_, stream_id, kIncludeVersion, fin, priority,
+            ++packet_number_, stream_id, fin, priority,
             std::move(request_headers_), spdy_headers_frame_length));
     DVLOG(2) << "packet(" << packet_number_ << "): " << std::endl
              << quiche::QuicheTextUtils::HexDump(packet->AsStringPiece());
@@ -623,7 +619,7 @@ class BidirectionalStreamQuicImplTest
         ConvertRequestPriorityToQuicPriority(request_priority);
     std::unique_ptr<quic::QuicReceivedPacket> packet(
         client_maker_.MakeRequestHeadersAndMultipleDataFramesPacket(
-            ++packet_number_, stream_id_, kIncludeVersion, fin, priority,
+            ++packet_number_, stream_id_, fin, priority,
             std::move(request_headers_), spdy_headers_frame_length, data));
     DVLOG(2) << "packet(" << packet_number_ << "): " << std::endl
              << quiche::QuicheTextUtils::HexDump(packet->AsStringPiece());
@@ -647,8 +643,8 @@ class BidirectionalStreamQuicImplTest
       spdy::Http2HeaderBlock response_headers,
       size_t* spdy_headers_frame_length) {
     return server_maker_.MakeResponseHeadersPacket(
-        packet_number, stream_id, !kIncludeVersion, fin,
-        std::move(response_headers), spdy_headers_frame_length);
+        packet_number, stream_id, fin, std::move(response_headers),
+        spdy_headers_frame_length);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructResponseTrailersPacket(
@@ -656,34 +652,30 @@ class BidirectionalStreamQuicImplTest
       bool fin,
       spdy::Http2HeaderBlock trailers,
       size_t* spdy_headers_frame_length) {
-    return server_maker_.MakeResponseHeadersPacket(
-        packet_number, stream_id_, !kIncludeVersion, fin, std::move(trailers),
-        spdy_headers_frame_length);
+    return server_maker_.MakeResponseHeadersPacket(packet_number, stream_id_,
+                                                   fin, std::move(trailers),
+                                                   spdy_headers_frame_length);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructClientRstStreamPacket() {
-    return ConstructRstStreamCancelledPacket(++packet_number_, !kIncludeVersion,
-                                             &client_maker_);
+    return ConstructRstStreamCancelledPacket(++packet_number_, &client_maker_);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructServerRstStreamPacket(
       uint64_t packet_number) {
-    return ConstructRstStreamCancelledPacket(packet_number, !kIncludeVersion,
-                                             &server_maker_);
+    return ConstructRstStreamCancelledPacket(packet_number, &server_maker_);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket>
   ConstructClientEarlyRstStreamPacket() {
-    return ConstructRstStreamCancelledPacket(++packet_number_, kIncludeVersion,
-                                             &client_maker_);
+    return ConstructRstStreamCancelledPacket(++packet_number_, &client_maker_);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructRstStreamCancelledPacket(
       uint64_t packet_number,
-      bool include_version,
       QuicTestPacketMaker* maker) {
     std::unique_ptr<quic::QuicReceivedPacket> packet(maker->MakeRstPacket(
-        packet_number, include_version, stream_id_, quic::QUIC_STREAM_CANCELLED,
+        packet_number, stream_id_, quic::QUIC_STREAM_CANCELLED,
         /*include_stop_sending_if_v99=*/true));
     DVLOG(2) << "packet(" << packet_number << "): " << std::endl
              << quiche::QuicheTextUtils::HexDump(packet->AsStringPiece());
@@ -694,21 +686,19 @@ class BidirectionalStreamQuicImplTest
   ConstructClientAckAndRstStreamPacket(uint64_t largest_received,
                                        uint64_t smallest_received) {
     return client_maker_.MakeAckAndRstPacket(
-        ++packet_number_, !kIncludeVersion, stream_id_,
-        quic::QUIC_STREAM_CANCELLED, largest_received, smallest_received);
+        ++packet_number_, stream_id_, quic::QUIC_STREAM_CANCELLED,
+        largest_received, smallest_received);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructAckAndDataPacket(
       uint64_t packet_number,
-      bool should_include_version,
       uint64_t largest_received,
       uint64_t smallest_received,
       bool fin,
       absl::string_view data,
       QuicTestPacketMaker* maker) {
     std::unique_ptr<quic::QuicReceivedPacket> packet(
-        maker->MakeAckAndDataPacket(packet_number, should_include_version,
-                                    stream_id_, largest_received,
+        maker->MakeAckAndDataPacket(packet_number, stream_id_, largest_received,
                                     smallest_received, fin, data));
     DVLOG(2) << "packet(" << packet_number << "): " << std::endl
              << quiche::QuicheTextUtils::HexDump(packet->AsStringPiece());
@@ -779,7 +769,7 @@ class BidirectionalStreamQuicImplTest
   scoped_refptr<TestTaskRunner> runner_;
   std::unique_ptr<MockWrite[]> mock_writes_;
   quic::MockClock clock_;
-  raw_ptr<quic::QuicConnection> connection_;
+  raw_ptr<quic::QuicConnection, DanglingUntriaged> connection_;
   std::unique_ptr<QuicChromiumConnectionHelper> helper_;
   std::unique_ptr<QuicChromiumAlarmFactory> alarm_factory_;
   TransportSecurityState transport_security_state_;
@@ -863,8 +853,7 @@ TEST_P(BidirectionalStreamQuicImplTest, GetRequest) {
   const char kResponseBody[] = "Hello world!";
   // Server sends data.
   std::string header = ConstructDataHeader(strlen(kResponseBody));
-  ProcessPacket(ConstructServerDataPacket(3, !kIncludeVersion, !kFin,
-                                          header + kResponseBody));
+  ProcessPacket(ConstructServerDataPacket(3, !kFin, header + kResponseBody));
   EXPECT_EQ(12, cb.WaitForResult());
 
   EXPECT_EQ(std::string(kResponseBody), delegate->data_received());
@@ -997,8 +986,8 @@ TEST_P(BidirectionalStreamQuicImplTest, CoalesceDataBuffersNotHeadersFrame) {
   AddWrite(ConstructRequestHeadersPacketInner(
       GetNthClientInitiatedBidirectionalStreamId(0), !kFin, DEFAULT_PRIORITY,
       &spdy_request_headers_frame_length));
-  AddWrite(ConstructClientDataPacket(kIncludeVersion, !kFin,
-                                     header + kBody1 + header2 + kBody2));
+  AddWrite(
+      ConstructClientDataPacket(!kFin, header + kBody1 + header2 + kBody2));
 
   // Ack server's data packet.
   AddWrite(ConstructClientAckPacket(3, 1));
@@ -1009,8 +998,7 @@ TEST_P(BidirectionalStreamQuicImplTest, CoalesceDataBuffersNotHeadersFrame) {
   std::string header4 = ConstructDataHeader(kBody4.length());
   std::string header5 = ConstructDataHeader(kBody5.length());
   AddWrite(ConstructClientDataPacket(
-      !kIncludeVersion, kFin,
-      header3 + kBody3 + header4 + kBody4 + header5 + kBody5));
+      kFin, header3 + kBody3 + header4 + kBody4 + header5 + kBody5));
 
   Initialize();
 
@@ -1063,8 +1051,7 @@ TEST_P(BidirectionalStreamQuicImplTest, CoalesceDataBuffersNotHeadersFrame) {
   const char kResponseBody[] = "Hello world!";
   std::string header6 = ConstructDataHeader(strlen(kResponseBody));
   // Server sends data.
-  ProcessPacket(ConstructServerDataPacket(3, !kIncludeVersion, !kFin,
-                                          header6 + kResponseBody));
+  ProcessPacket(ConstructServerDataPacket(3, !kFin, header6 + kResponseBody));
 
   EXPECT_EQ(static_cast<int>(strlen(kResponseBody)), cb.WaitForResult());
 
@@ -1125,8 +1112,7 @@ TEST_P(BidirectionalStreamQuicImplTest,
   AddWrite(ConstructClientAckPacket(3, 1));
   const char kBody2[] = "really small";
   std::string header2 = ConstructDataHeader(strlen(kBody2));
-  AddWrite(ConstructClientDataPacket(!kIncludeVersion, kFin,
-                                     header2 + std::string(kBody2)));
+  AddWrite(ConstructClientDataPacket(kFin, header2 + std::string(kBody2)));
 
   Initialize();
 
@@ -1171,8 +1157,7 @@ TEST_P(BidirectionalStreamQuicImplTest,
   const char kResponseBody[] = "Hello world!";
   // Server sends data.
   std::string header3 = ConstructDataHeader(strlen(kResponseBody));
-  ProcessPacket(ConstructServerDataPacket(3, !kIncludeVersion, !kFin,
-                                          header3 + kResponseBody));
+  ProcessPacket(ConstructServerDataPacket(3, !kFin, header3 + kResponseBody));
 
   EXPECT_EQ(static_cast<int>(strlen(kResponseBody)), cb.WaitForResult());
 
@@ -1234,8 +1219,7 @@ TEST_P(BidirectionalStreamQuicImplTest,
   std::string header4 = ConstructDataHeader(kBody4.length());
   std::string header5 = ConstructDataHeader(kBody5.length());
   AddWrite(ConstructClientDataPacket(
-      !kIncludeVersion, kFin,
-      header3 + kBody3 + header4 + kBody4 + header5 + kBody5));
+      kFin, header3 + kBody3 + header4 + kBody4 + header5 + kBody5));
 
   Initialize();
 
@@ -1283,8 +1267,7 @@ TEST_P(BidirectionalStreamQuicImplTest,
   const char kResponseBody[] = "Hello world!";
   std::string header6 = ConstructDataHeader(strlen(kResponseBody));
   // Server sends data.
-  ProcessPacket(ConstructServerDataPacket(3, !kIncludeVersion, !kFin,
-                                          header6 + kResponseBody));
+  ProcessPacket(ConstructServerDataPacket(3, !kFin, header6 + kResponseBody));
 
   EXPECT_EQ(static_cast<int>(strlen(kResponseBody)), cb.WaitForResult());
 
@@ -1412,8 +1395,7 @@ TEST_P(BidirectionalStreamQuicImplTest, PostRequest) {
       GetNthClientInitiatedBidirectionalStreamId(0), !kFin, DEFAULT_PRIORITY,
       &spdy_request_headers_frame_length));
   std::string header = ConstructDataHeader(strlen(kUploadData));
-  AddWrite(ConstructClientDataPacket(kIncludeVersion, kFin,
-                                     header + std::string(kUploadData)));
+  AddWrite(ConstructClientDataPacket(kFin, header + std::string(kUploadData)));
 
   AddWrite(ConstructClientAckPacket(3, 1));
 
@@ -1459,8 +1441,7 @@ TEST_P(BidirectionalStreamQuicImplTest, PostRequest) {
   const char kResponseBody[] = "Hello world!";
   std::string header2 = ConstructDataHeader(strlen(kResponseBody));
   // Server sends data.
-  ProcessPacket(ConstructServerDataPacket(3, !kIncludeVersion, !kFin,
-                                          header2 + kResponseBody));
+  ProcessPacket(ConstructServerDataPacket(3, !kFin, header2 + kResponseBody));
 
   EXPECT_EQ(static_cast<int>(strlen(kResponseBody)), cb.WaitForResult());
 
@@ -1539,8 +1520,7 @@ TEST_P(BidirectionalStreamQuicImplTest, EarlyDataOverrideRequest) {
   const char kResponseBody[] = "Hello world!";
   // Server sends data.
   std::string header = ConstructDataHeader(strlen(kResponseBody));
-  ProcessPacket(ConstructServerDataPacket(3, !kIncludeVersion, !kFin,
-                                          header + kResponseBody));
+  ProcessPacket(ConstructServerDataPacket(3, !kFin, header + kResponseBody));
   EXPECT_EQ(12, cb.WaitForResult());
 
   EXPECT_EQ(std::string(kResponseBody), delegate->data_received());
@@ -1597,11 +1577,11 @@ TEST_P(BidirectionalStreamQuicImplTest, InterleaveReadDataAndSendData) {
       &spdy_request_headers_frame_length));
 
   std::string header = ConstructDataHeader(strlen(kUploadData));
-  AddWrite(ConstructAckAndDataPacket(++packet_number_, !kIncludeVersion, 2, 1,
-                                     !kFin, header + std::string(kUploadData),
+  AddWrite(ConstructAckAndDataPacket(++packet_number_, 2, 1, !kFin,
+                                     header + std::string(kUploadData),
                                      &client_maker_));
-  AddWrite(ConstructAckAndDataPacket(++packet_number_, !kIncludeVersion, 3, 3,
-                                     kFin, header + std::string(kUploadData),
+  AddWrite(ConstructAckAndDataPacket(++packet_number_, 3, 3, kFin,
+                                     header + std::string(kUploadData),
                                      &client_maker_));
   Initialize();
 
@@ -1648,9 +1628,8 @@ TEST_P(BidirectionalStreamQuicImplTest, InterleaveReadDataAndSendData) {
   std::string header2 = ConstructDataHeader(strlen(kResponseBody));
   // Server sends a data packet
   int server_ack = 2;
-  ProcessPacket(ConstructAckAndDataPacket(3, !kIncludeVersion, server_ack++, 1,
-                                          !kFin, header2 + kResponseBody,
-                                          &server_maker_));
+  ProcessPacket(ConstructAckAndDataPacket(
+      3, server_ack++, 1, !kFin, header2 + kResponseBody, &server_maker_));
 
   EXPECT_EQ(static_cast<int64_t>(strlen(kResponseBody)), cb.WaitForResult());
   EXPECT_EQ(std::string(kResponseBody), delegate->data_received());
@@ -1662,10 +1641,10 @@ TEST_P(BidirectionalStreamQuicImplTest, InterleaveReadDataAndSendData) {
   TestCompletionCallback cb2;
   rv = delegate->ReadData(cb2.callback());
   EXPECT_THAT(rv, IsError(ERR_IO_PENDING));
-  ProcessPacket(
-      ConstructAckAndDataPacket(4, !kIncludeVersion, server_ack++, 1, kFin,
+  ProcessPacket(ConstructAckAndDataPacket(4, server_ack++, 1, kFin,
 
-                                header2 + kResponseBody, &server_maker_));
+                                          header2 + kResponseBody,
+                                          &server_maker_));
 
   EXPECT_EQ(static_cast<int64_t>(strlen(kResponseBody)), cb2.WaitForResult());
 
@@ -1715,7 +1694,7 @@ TEST_P(BidirectionalStreamQuicImplTest, ServerSendsRstAfterHeaders) {
   // Server sends a Rst. Since the stream has sent fin, the rst is one way in
   // IETF QUIC.
   ProcessPacket(server_maker_.MakeRstPacket(
-      1, !kIncludeVersion, GetNthClientInitiatedBidirectionalStreamId(0),
+      1, GetNthClientInitiatedBidirectionalStreamId(0),
       quic::QUIC_STREAM_CANCELLED,
       /*include_stop_sending_if_v99=*/false));
 
@@ -1785,7 +1764,7 @@ TEST_P(BidirectionalStreamQuicImplTest, ServerSendsRstAfterReadData) {
   // Server sends a Rst. Since the stream has sent fin, the rst is one way in
   // IETF QUIC.
   ProcessPacket(server_maker_.MakeRstPacket(
-      3, !kIncludeVersion, GetNthClientInitiatedBidirectionalStreamId(0),
+      3, GetNthClientInitiatedBidirectionalStreamId(0),
       quic::QUIC_STREAM_CANCELLED,
       /*include_stop_sending_if_v99=*/false));
 
@@ -2129,8 +2108,7 @@ TEST_P(BidirectionalStreamQuicImplTest, DeleteStreamDuringOnDataRead) {
   const char kResponseBody[] = "Hello world!";
   std::string header = ConstructDataHeader(strlen(kResponseBody));
   // Server sends data.
-  ProcessPacket(ConstructServerDataPacket(3, !kIncludeVersion, !kFin,
-                                          header + kResponseBody));
+  ProcessPacket(ConstructServerDataPacket(3, !kFin, header + kResponseBody));
   EXPECT_EQ(static_cast<int64_t>(strlen(kResponseBody)), cb.WaitForResult());
 
   base::RunLoop().RunUntilIdle();
@@ -2150,7 +2128,7 @@ TEST_P(BidirectionalStreamQuicImplTest, AsyncFinRead) {
       GetNthClientInitiatedBidirectionalStreamId(0), !kFin, DEFAULT_PRIORITY,
       &spdy_request_headers_frame_length));
   std::string header = ConstructDataHeader(strlen(kBody));
-  AddWrite(ConstructClientDataPacket(kIncludeVersion, kFin, header + kBody));
+  AddWrite(ConstructClientDataPacket(kFin, header + kBody));
   AddWrite(ConstructClientAckPacket(3, 1));
 
   Initialize();
@@ -2201,8 +2179,7 @@ TEST_P(BidirectionalStreamQuicImplTest, AsyncFinRead) {
 
   // Server sends data with the fin set, which should result in the stream
   // being closed and hence no RST_STREAM will be sent.
-  ProcessPacket(ConstructServerDataPacket(3, !kIncludeVersion, kFin,
-                                          header2 + kResponseBody));
+  ProcessPacket(ConstructServerDataPacket(3, kFin, header2 + kResponseBody));
   EXPECT_EQ(static_cast<int64_t>(strlen(kResponseBody)), cb.WaitForResult());
 
   base::RunLoop().RunUntilIdle();
@@ -2262,8 +2239,7 @@ TEST_P(BidirectionalStreamQuicImplTest, DeleteStreamDuringOnTrailersReceived) {
 
   // Server sends data.
   std::string header = ConstructDataHeader(strlen(kResponseBody));
-  ProcessPacket(ConstructServerDataPacket(3, !kIncludeVersion, !kFin,
-                                          header + kResponseBody));
+  ProcessPacket(ConstructServerDataPacket(3, !kFin, header + kResponseBody));
 
   EXPECT_EQ(static_cast<int64_t>(strlen(kResponseBody)), cb.WaitForResult());
   EXPECT_EQ(std::string(kResponseBody), delegate->data_received());
