@@ -6,7 +6,9 @@
 
 #include "base/containers/contains.h"
 #include "base/containers/cxx20_erase.h"
+#include "base/feature_list.h"
 #include "base/memory/raw_ptr.h"
+#include "base/metrics/field_trial_params.h"
 #include "base/metrics/histogram_functions.h"
 #include "base/task/single_thread_task_runner.h"
 #include "net/base/address_list.h"
@@ -22,6 +24,7 @@
 #include "net/spdy/spdy_http_utils.h"
 #include "net/third_party/quiche/src/quiche/quic/core/http/web_transport_http3.h"
 #include "net/third_party/quiche/src/quiche/quic/core/quic_connection.h"
+#include "net/third_party/quiche/src/quiche/quic/core/quic_types.h"
 #include "net/third_party/quiche/src/quiche/quic/core/quic_utils.h"
 #include "net/url_request/url_request_context.h"
 #include "url/scheme_host_port.h"
@@ -29,6 +32,7 @@
 namespace net {
 
 namespace {
+
 // From
 // https://wicg.github.io/web-transport/#dom-quictransportconfiguration-server_certificate_fingerprints
 constexpr int kCustomCertificateMaxValidityDays = 14;
@@ -37,6 +41,23 @@ constexpr int kCustomCertificateMaxValidityDays = 14;
 // being closed.
 constexpr base::TimeDelta kMaxCloseTimeout = base::Seconds(2);
 
+// Enables custom congestion control for WebTransport over HTTP/3.
+BASE_FEATURE(kWebTransportCongestionControl,
+             "WebTransportCongestionControl",
+             base::FEATURE_DISABLED_BY_DEFAULT);
+constexpr base::FeatureParam<quic::CongestionControlType>::Option
+    kWebTransportCongestionControlAlgorithms[] = {
+        {quic::kCubicBytes, "CUBIC"},
+        {quic::kRenoBytes, "Reno"},
+        {quic::kBBR, "BBRv1"},
+        {quic::kBBRv2, "BBRv2"},
+};
+constexpr base::FeatureParam<quic::CongestionControlType>
+    kWebTransportCongestionControlAlgorithm{
+        &kWebTransportCongestionControl, /*name=*/"algorithm",
+        /*default_value=*/quic::kCubicBytes,
+        &kWebTransportCongestionControlAlgorithms};
+
 std::set<std::string> HostsFromOrigins(std::set<HostPortPair> origins) {
   std::set<std::string> hosts;
   for (const auto& origin : origins) {
@@ -69,12 +90,15 @@ std::unique_ptr<quic::ProofVerifier> CreateProofVerifier(
     URLRequestContext* context,
     const WebTransportParameters& parameters) {
   if (parameters.server_certificate_fingerprints.empty()) {
+    std::set<std::string> hostnames_to_allow_unknown_roots = HostsFromOrigins(
+        context->quic_context()->params()->origins_to_force_quic_on);
+    if (context->quic_context()->params()->webtransport_developer_mode) {
+      hostnames_to_allow_unknown_roots.insert("");
+    }
     return std::make_unique<ProofVerifierChromium>(
         context->cert_verifier(), context->ct_policy_enforcer(),
         context->transport_security_state(), context->sct_auditing_delegate(),
-        HostsFromOrigins(
-            context->quic_context()->params()->origins_to_force_quic_on),
-        anonymization_key);
+        std::move(hostnames_to_allow_unknown_roots), anonymization_key);
   }
 
   auto verifier =
@@ -122,6 +146,8 @@ class ConnectStream : public quic::QuicSpdyClientStream {
                 DedicatedWebTransportHttp3Client* client)
       : quic::QuicSpdyClientStream(id, session, type), client_(client) {}
 
+  ~ConnectStream() override { client_->OnConnectStreamDeleted(); }
+
   void OnInitialHeadersComplete(
       bool fin,
       size_t frame_len,
@@ -181,7 +207,17 @@ class DedicatedWebTransportHttp3ClientSession
     return true;
   }
 
-  bool ShouldNegotiateWebTransport() override { return true; }
+  quic::WebTransportHttp3VersionSet LocallySupportedWebTransportVersions()
+      const override {
+    quic::WebTransportHttp3VersionSet versions =
+        quic::WebTransportHttp3VersionSet(
+            {quic::WebTransportHttp3Version::kDraft02});
+    if (base::FeatureList::IsEnabled(features::kEnableWebTransportDraft07)) {
+      versions.Set(quic::WebTransportHttp3Version::kDraft07);
+    }
+    return versions;
+  }
+
   quic::HttpDatagramSupport LocalHttpDatagramSupport() override {
     return quic::HttpDatagramSupport::kRfcAndDraft04;
   }
@@ -221,9 +257,7 @@ class WebTransportVisitorProxy : public quic::WebTransportVisitor {
   explicit WebTransportVisitorProxy(quic::WebTransportVisitor* visitor)
       : visitor_(visitor) {}
 
-  void OnSessionReady(const spdy::Http2HeaderBlock& block) override {
-    visitor_->OnSessionReady(block);
-  }
+  void OnSessionReady() override { visitor_->OnSessionReady(); }
   void OnSessionClosed(quic::WebTransportSessionError error_code,
                        const std::string& error_message) override {
     visitor_->OnSessionClosed(error_code, error_message);
@@ -253,6 +287,74 @@ bool IsTerminalState(WebTransportState state) {
          state == WebTransportState::FAILED;
 }
 
+// These values are persisted to logs. Entries should not be renumbered and
+// numeric values should never be reused.
+enum class NegotiatedHttpDatagramVersion {
+  kNone = 0,
+  kDraft04 = 1,
+  kRfc = 2,
+  kMaxValue = kRfc,
+};
+
+void RecordNegotiatedHttpDatagramSupport(quic::HttpDatagramSupport support) {
+  NegotiatedHttpDatagramVersion negotiated;
+  switch (support) {
+    case quic::HttpDatagramSupport::kNone:
+      negotiated = NegotiatedHttpDatagramVersion::kNone;
+      break;
+    case quic::HttpDatagramSupport::kDraft04:
+      negotiated = NegotiatedHttpDatagramVersion::kDraft04;
+      break;
+    case quic::HttpDatagramSupport::kRfc:
+      negotiated = NegotiatedHttpDatagramVersion::kRfc;
+      break;
+    case quic::HttpDatagramSupport::kRfcAndDraft04:
+      NOTREACHED();
+      return;
+  }
+  base::UmaHistogramEnumeration(
+      "Net.WebTransport.NegotiatedHttpDatagramVersion", negotiated);
+}
+
+const char* WebTransportHttp3VersionString(
+    quic::WebTransportHttp3Version version) {
+  switch (version) {
+    case quic::WebTransportHttp3Version::kDraft02:
+      return "draft-02";
+    case quic::WebTransportHttp3Version::kDraft07:
+      return "draft-07";
+  }
+}
+
+enum class NegotiatedWebTransportVersion {
+  kDraft02 = 0,
+  kDraft07 = 1,
+  kMaxValue = kDraft07,
+};
+
+void RecordNegotiatedWebTransportVersion(
+    quic::WebTransportHttp3Version version) {
+  NegotiatedWebTransportVersion negotiated;
+  switch (version) {
+    case quic::WebTransportHttp3Version::kDraft02:
+      negotiated = NegotiatedWebTransportVersion::kDraft02;
+      break;
+    case quic::WebTransportHttp3Version::kDraft07:
+      negotiated = NegotiatedWebTransportVersion::kDraft07;
+      break;
+  }
+  base::UmaHistogramEnumeration(
+      "Net.WebTransport.NegotiatedWebTransportVersion", negotiated);
+}
+
+void AdjustSendAlgorithm(quic::QuicConnection& connection) {
+  if (!base::FeatureList::IsEnabled(kWebTransportCongestionControl)) {
+    return;
+  }
+  connection.sent_packet_manager().SetSendAlgorithm(
+      kWebTransportCongestionControlAlgorithm.Get());
+}
+
 }  // namespace
 
 DedicatedWebTransportHttp3Client::DedicatedWebTransportHttp3Client(
@@ -281,6 +383,7 @@ DedicatedWebTransportHttp3Client::DedicatedWebTransportHttp3Client(
       crypto_config_(
           CreateProofVerifier(anonymization_key_, context, parameters),
           /* session_cache */ nullptr) {
+  ConfigureQuicCryptoClientConfig(crypto_config_);
   net_log_.BeginEvent(
       NetLogEventType::QUIC_SESSION_WEBTRANSPORT_CLIENT_ALIVE, [&] {
         base::Value::Dict dict;
@@ -314,6 +417,7 @@ void DedicatedWebTransportHttp3Client::Connect() {
 
 void DedicatedWebTransportHttp3Client::Close(
     const absl::optional<WebTransportCloseInfo>& close_info) {
+  CHECK(session());
   base::TimeDelta probe_timeout = base::Microseconds(
       connection_->sent_packet_manager().GetPtoDelay().ToMicroseconds());
   // Wait for at least three PTOs similar to what's used in
@@ -513,7 +617,8 @@ void DedicatedWebTransportHttp3Client::CreateConnection() {
   }
 
   packet_reader_ = std::make_unique<QuicChromiumPacketReader>(
-      socket_.get(), quic_context_->clock(), this, kQuicYieldAfterPacketsRead,
+      std::move(socket_), quic_context_->clock(), this,
+      kQuicYieldAfterPacketsRead,
       quic::QuicTime::Delta::FromMilliseconds(
           kQuicYieldAfterDurationMilliseconds),
       net_log_);
@@ -521,6 +626,7 @@ void DedicatedWebTransportHttp3Client::CreateConnection() {
   event_logger_ = std::make_unique<QuicEventLogger>(session_.get(), net_log_);
   connection_->set_debug_visitor(event_logger_.get());
   connection_->set_creator_debug_delegate(event_logger_.get());
+  AdjustSendAlgorithm(*connection_);
 
   session_->Initialize();
   packet_reader_->StartReading();
@@ -609,6 +715,12 @@ void DedicatedWebTransportHttp3Client::OnConnectStreamAborted() {
   TransitionToState(WebTransportState::FAILED);
 }
 
+void DedicatedWebTransportHttp3Client::OnConnectStreamDeleted() {
+  // `web_transport_session_` is owned by ConnectStream. Clear so that it
+  // doesn't get dangling.
+  web_transport_session_ = nullptr;
+}
+
 void DedicatedWebTransportHttp3Client::OnCloseTimeout() {
   SetErrorIfNecessary(ERR_TIMED_OUT);
   TransitionToState(WebTransportState::FAILED);
@@ -623,7 +735,6 @@ int DedicatedWebTransportHttp3Client::DoSendRequest() {
   if (stream == nullptr) {
     return ERR_QUIC_PROTOCOL_ERROR;
   }
-  connect_stream_ = stream;
 
   spdy::Http2HeaderBlock headers;
   DCHECK_EQ(url_.scheme(), url::kHttpsScheme);
@@ -724,9 +835,25 @@ void DedicatedWebTransportHttp3Client::SetErrorIfNecessary(
   }
 }
 
-void DedicatedWebTransportHttp3Client::OnSessionReady(
-    const spdy::Http2HeaderBlock& /*spdy_headers*/) {
+void DedicatedWebTransportHttp3Client::OnSessionReady() {
+  CHECK(session_->SupportsWebTransport());
+
   session_ready_ = true;
+
+  RecordNegotiatedWebTransportVersion(
+      *session_->SupportedWebTransportVersion());
+  RecordNegotiatedHttpDatagramSupport(session_->http_datagram_support());
+  net_log_.AddEvent(NetLogEventType::QUIC_SESSION_WEBTRANSPORT_SESSION_READY,
+                    [&] {
+                      base::Value::Dict dict;
+                      dict.Set("http_datagram_version",
+                               quic::HttpDatagramSupportToString(
+                                   session_->http_datagram_support()));
+                      dict.Set("webtransport_http3_version",
+                               WebTransportHttp3VersionString(
+                                   *session_->SupportedWebTransportVersion()));
+                      return dict;
+                    });
 }
 
 void DedicatedWebTransportHttp3Client::OnSessionClosed(
