@@ -33,20 +33,6 @@ namespace net {
 
 namespace {
 
-void UpdatePublicResetAddressMismatchHistogram(
-    const IPEndPoint& server_hello_address,
-    const IPEndPoint& public_reset_address) {
-  int sample = GetAddressMismatch(server_hello_address, public_reset_address);
-  // We are seemingly talking to an older server that does not support the
-  // feature, so we can't report the results in the histogram.
-  if (sample < 0) {
-    return;
-  }
-  UMA_HISTOGRAM_ENUMERATION("Net.QuicSession.PublicResetAddressMismatch2",
-                            static_cast<QuicAddressMismatch>(sample),
-                            QUIC_ADDRESS_MISMATCH_MAX);
-}
-
 // If |address| is an IPv4-mapped IPv6 address, returns ADDRESS_FAMILY_IPV4
 // instead of ADDRESS_FAMILY_IPV6. Othewise, behaves like GetAddressFamily().
 AddressFamily GetRealAddressFamily(const IPAddress& address) {
@@ -58,11 +44,9 @@ AddressFamily GetRealAddressFamily(const IPAddress& address) {
 
 QuicConnectionLogger::QuicConnectionLogger(
     quic::QuicSession* session,
-    const char* const connection_description,
     std::unique_ptr<SocketPerformanceWatcher> socket_performance_watcher,
     const NetLogWithSource& net_log)
     : session_(session),
-      connection_description_(connection_description),
       socket_performance_watcher_(std::move(socket_performance_watcher)),
       event_logger_(session, net_log) {}
 
@@ -101,8 +85,6 @@ QuicConnectionLogger::~QuicConnectionLogger() {
           duplicate_stream_frame_per_thousand, 1, 1000, 75);
     }
   }
-
-  RecordAggregatePacketLossRate();
 }
 
 void QuicConnectionLogger::OnFrameAddedToPacket(const quic::QuicFrame& frame) {
@@ -178,7 +160,8 @@ void QuicConnectionLogger::OnPacketSent(
     quic::EncryptionLevel encryption_level,
     const quic::QuicFrames& retransmittable_frames,
     const quic::QuicFrames& nonretransmittable_frames,
-    quic::QuicTime sent_time) {
+    quic::QuicTime sent_time,
+    uint32_t batch_id) {
   // 4.4.1.4.  Minimum Packet Size
   // The payload of a UDP datagram carrying the Initial packet MUST be
   // expanded to at least 1200 octets
@@ -215,7 +198,7 @@ void QuicConnectionLogger::OnPacketSent(
   event_logger_.OnPacketSent(packet_number, packet_length, has_crypto_handshake,
                              transmission_type, encryption_level,
                              retransmittable_frames, nonretransmittable_frames,
-                             sent_time);
+                             sent_time, batch_id);
 }
 
 void QuicConnectionLogger::OnPacketLoss(
@@ -395,11 +378,6 @@ void QuicConnectionLogger::OnIncomingAck(
                               least_unacked_sent_packet);
 }
 
-void QuicConnectionLogger::OnStopWaitingFrame(
-    const quic::QuicStopWaitingFrame& frame) {
-  event_logger_.OnStopWaitingFrame(frame);
-}
-
 void QuicConnectionLogger::OnRstStreamFrame(
     const quic::QuicRstStreamFrame& frame) {
   base::UmaHistogramSparse("Net.QuicSession.RstStreamErrorCodeServer",
@@ -470,13 +448,6 @@ void QuicConnectionLogger::OnCoalescedPacketSent(
   event_logger_.OnCoalescedPacketSent(coalesced_packet, length);
 }
 
-void QuicConnectionLogger::OnPublicResetPacket(
-    const quic::QuicPublicResetPacket& packet) {
-  UpdatePublicResetAddressMismatchHistogram(
-      local_address_from_shlo_, ToIPEndPoint(packet.client_address));
-  event_logger_.OnPublicResetPacket(packet);
-}
-
 void QuicConnectionLogger::OnVersionNegotiationPacket(
     const quic::QuicVersionNegotiationPacket& packet) {
   event_logger_.OnVersionNegotiationPacket(packet);
@@ -498,8 +469,9 @@ void QuicConnectionLogger::OnCryptoHandshakeMessageReceived(
 
       int sample = GetAddressMismatch(local_address_from_shlo_,
                                       local_address_from_self_);
-      // We are seemingly talking to an older server that does not support the
-      // feature, so we can't report the results in the histogram.
+      // If `sample` is negative, we are seemingly talking to an older server
+      // that does not support the feature, so we can't report the results in
+      // the histogram.
       if (sample >= 0) {
         UMA_HISTOGRAM_ENUMERATION("Net.QuicSession.SelfShloAddressMismatch",
                                   static_cast<QuicAddressMismatch>(sample),
@@ -542,16 +514,6 @@ void QuicConnectionLogger::OnCertificateVerified(
   event_logger_.OnCertificateVerified(result);
 }
 
-base::HistogramBase* QuicConnectionLogger::Get6PacketHistogram(
-    const char* which_6) const {
-  // This histogram takes a binary encoding of the 6 consecutive packets
-  // received.  As a result, there are 64 possible sample-patterns.
-  string prefix("Net.QuicSession.6PacketsPatternsReceived_");
-  return base::LinearHistogram::FactoryGet(
-      prefix + which_6 + connection_description_, 1, 64, 65,
-      base::HistogramBase::kUmaTargetedHistogramFlag);
-}
-
 float QuicConnectionLogger::ReceivedPacketLossRate() const {
   if (!largest_received_packet_number_.IsInitialized())
     return 0.0f;
@@ -593,23 +555,9 @@ void QuicConnectionLogger::OnZeroRttRejected(int reason) {
   event_logger_.OnZeroRttRejected(reason);
 }
 
-void QuicConnectionLogger::RecordAggregatePacketLossRate() const {
-  // For short connections under 22 packets in length, we'll rely on the
-  // Net.QuicSession.21CumulativePacketsReceived_* histogram to indicate packet
-  // loss rates.  This way we avoid tremendously anomalous contributions to our
-  // histogram.  (e.g., if we only got 5 packets, but lost 1, we'd otherwise
-  // record a 20% loss in this histogram!). We may still get some strange data
-  // (1 loss in 22 is still high :-/).
-  if (!largest_received_packet_number_.IsInitialized() ||
-      largest_received_packet_number_ - first_received_packet_number_ < 22)
-    return;
-
-  string prefix("Net.QuicSession.PacketLossRate_");
-  base::HistogramBase* histogram = base::Histogram::FactoryGet(
-      prefix + connection_description_, 1, 1000, 75,
-      base::HistogramBase::kUmaTargetedHistogramFlag);
-  histogram->Add(static_cast<base::HistogramBase::Sample>(
-      ReceivedPacketLossRate() * 1000));
+void QuicConnectionLogger::OnEncryptedClientHelloSent(
+    std::string_view client_hello) {
+  event_logger_.OnEncryptedClientHelloSent(client_hello);
 }
 
 }  // namespace net
