@@ -256,8 +256,8 @@ class QuicProxyClientSocketTest
             kQuicYieldAfterDurationMilliseconds),
         /*cert_verify_flags=*/0, quic::test::DefaultQuicConfig(),
         std::make_unique<TestQuicCryptoClientConfigHandle>(&crypto_config_),
-        "CONNECTION_UNKNOWN", dns_start, dns_end,
-        std::make_unique<quic::QuicClientPushPromiseIndex>(), nullptr,
+        dns_start, dns_end,
+        std::make_unique<quic::QuicClientPushPromiseIndex>(),
         base::DefaultTickClock::GetInstance(),
         base::SingleThreadTaskRunner::GetCurrentDefault().get(),
         /*socket_performance_watcher=*/nullptr, HostResolverEndpointResult(),
@@ -324,8 +324,8 @@ class QuicProxyClientSocketTest
       uint64_t largest_received,
       uint64_t smallest_received) {
     return client_maker_.MakeAckAndRstPacket(
-        packet_number, !kIncludeVersion, client_data_stream_id1_, error_code,
-        largest_received, smallest_received,
+        packet_number, client_data_stream_id1_, error_code, largest_received,
+        smallest_received,
         /*include_stop_sending_if_v99=*/false);
   }
 
@@ -335,16 +335,16 @@ class QuicProxyClientSocketTest
       uint64_t largest_received,
       uint64_t smallest_received) {
     return client_maker_.MakeAckAndRstPacket(
-        packet_number, !kIncludeVersion, client_data_stream_id1_, error_code,
-        largest_received, smallest_received,
+        packet_number, client_data_stream_id1_, error_code, largest_received,
+        smallest_received,
         /*include_stop_sending_if_v99=*/true);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructRstPacket(
       uint64_t packet_number,
       quic::QuicRstStreamErrorCode error_code) {
-    return client_maker_.MakeRstPacket(packet_number, !kIncludeVersion,
-                                       client_data_stream_id1_, error_code,
+    return client_maker_.MakeRstPacket(packet_number, client_data_stream_id1_,
+                                       error_code,
                                        /*include_stop_sending_if_v99=*/true);
   }
 
@@ -354,7 +354,7 @@ class QuicProxyClientSocketTest
     spdy::Http2HeaderBlock block;
     PopulateConnectRequestIR(&block);
     return client_maker_.MakeRequestHeadersPacket(
-        packet_number, client_data_stream_id1_, kIncludeVersion, !kFin,
+        packet_number, client_data_stream_id1_, !kFin,
         ConvertRequestPriorityToQuicPriority(request_priority),
         std::move(block), nullptr);
   }
@@ -372,7 +372,7 @@ class QuicProxyClientSocketTest
       block[header.first] = header.second;
     }
     return client_maker_.MakeRequestHeadersPacket(
-        packet_number, client_data_stream_id1_, kIncludeVersion, !kFin,
+        packet_number, client_data_stream_id1_, !kFin,
         ConvertRequestPriorityToQuicPriority(request_priority),
         std::move(block), nullptr);
   }
@@ -384,7 +384,7 @@ class QuicProxyClientSocketTest
     PopulateConnectRequestIR(&block);
     block["proxy-authorization"] = "Basic Zm9vOmJhcg==";
     return client_maker_.MakeRequestHeadersPacket(
-        packet_number, client_data_stream_id1_, kIncludeVersion, !kFin,
+        packet_number, client_data_stream_id1_, !kFin,
         ConvertRequestPriorityToQuicPriority(request_priority),
         std::move(block), nullptr);
   }
@@ -393,7 +393,7 @@ class QuicProxyClientSocketTest
       uint64_t packet_number,
       absl::string_view data) {
     return client_maker_.MakeDataPacket(packet_number, client_data_stream_id1_,
-                                        !kIncludeVersion, !kFin, data);
+                                        !kFin, data);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructAckAndDataPacket(
@@ -402,8 +402,8 @@ class QuicProxyClientSocketTest
       uint64_t smallest_received,
       absl::string_view data) {
     return client_maker_.MakeAckAndDataPacket(
-        packet_number, !kIncludeVersion, client_data_stream_id1_,
-        largest_received, smallest_received, !kFin, data);
+        packet_number, client_data_stream_id1_, largest_received,
+        smallest_received, !kFin, data);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructAckPacket(
@@ -419,8 +419,8 @@ class QuicProxyClientSocketTest
   std::unique_ptr<quic::QuicReceivedPacket> ConstructServerRstPacket(
       uint64_t packet_number,
       quic::QuicRstStreamErrorCode error_code) {
-    return server_maker_.MakeRstPacket(packet_number, !kIncludeVersion,
-                                       client_data_stream_id1_, error_code,
+    return server_maker_.MakeRstPacket(packet_number, client_data_stream_id1_,
+                                       error_code,
                                        /*include_stop_sending_if_v99=*/true);
   }
 
@@ -428,14 +428,14 @@ class QuicProxyClientSocketTest
       uint64_t packet_number,
       absl::string_view data) {
     return server_maker_.MakeDataPacket(packet_number, client_data_stream_id1_,
-                                        !kIncludeVersion, !kFin, data);
+                                        !kFin, data);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructServerDataFinPacket(
       uint64_t packet_number,
       absl::string_view data) {
     return server_maker_.MakeDataPacket(packet_number, client_data_stream_id1_,
-                                        !kIncludeVersion, kFin, data);
+                                        kFin, data);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket> ConstructServerConnectReplyPacket(
@@ -446,8 +446,8 @@ class QuicProxyClientSocketTest
     block[":status"] = "200";
 
     return server_maker_.MakeResponseHeadersPacket(
-        packet_number, client_data_stream_id1_, !kIncludeVersion, fin,
-        std::move(block), header_length);
+        packet_number, client_data_stream_id1_, fin, std::move(block),
+        header_length);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket>
@@ -462,8 +462,7 @@ class QuicProxyClientSocketTest
     }
 
     return server_maker_.MakeResponseHeadersPacket(
-        packet_number, client_data_stream_id1_, !kIncludeVersion, fin,
-        std::move(block), nullptr);
+        packet_number, client_data_stream_id1_, fin, std::move(block), nullptr);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket>
@@ -472,8 +471,7 @@ class QuicProxyClientSocketTest
     block[":status"] = "407";
     block["proxy-authenticate"] = "Basic realm=\"MyRealm1\"";
     return server_maker_.MakeResponseHeadersPacket(
-        packet_number, client_data_stream_id1_, !kIncludeVersion, fin,
-        std::move(block), nullptr);
+        packet_number, client_data_stream_id1_, fin, std::move(block), nullptr);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket>
@@ -483,8 +481,7 @@ class QuicProxyClientSocketTest
     block["location"] = kRedirectUrl;
     block["set-cookie"] = "foo=bar";
     return server_maker_.MakeResponseHeadersPacket(
-        packet_number, client_data_stream_id1_, !kIncludeVersion, fin,
-        std::move(block), nullptr);
+        packet_number, client_data_stream_id1_, fin, std::move(block), nullptr);
   }
 
   std::unique_ptr<quic::QuicReceivedPacket>
@@ -493,8 +490,7 @@ class QuicProxyClientSocketTest
     block[":status"] = "500";
 
     return server_maker_.MakeResponseHeadersPacket(
-        packet_number, client_data_stream_id1_, !kIncludeVersion, fin,
-        std::move(block), nullptr);
+        packet_number, client_data_stream_id1_, fin, std::move(block), nullptr);
   }
 
   void AssertConnectSucceeds() {
