@@ -123,9 +123,8 @@ void QuicTestPacketMaker::set_hostname(const std::string& host) {
 }
 
 std::unique_ptr<quic::QuicReceivedPacket>
-QuicTestPacketMaker::MakeConnectivityProbingPacket(uint64_t num,
-                                                   bool include_version) {
-  InitializeHeader(num, include_version);
+QuicTestPacketMaker::MakeConnectivityProbingPacket(uint64_t num) {
+  InitializeHeader(num);
 
   if (!version_.HasIetfQuicFrames()) {
     AddQuicPingFrame();
@@ -141,18 +140,16 @@ QuicTestPacketMaker::MakeConnectivityProbingPacket(uint64_t num,
 }
 
 std::unique_ptr<quic::QuicReceivedPacket> QuicTestPacketMaker::MakePingPacket(
-    uint64_t num,
-    bool include_version) {
-  InitializeHeader(num, include_version);
+    uint64_t num) {
+  InitializeHeader(num);
   AddQuicPingFrame();
   return BuildPacket();
 }
 
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeRetireConnectionIdPacket(uint64_t num,
-                                                  bool include_version,
                                                   uint64_t sequence_number) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
   AddQuicRetireConnectionIdFrame(sequence_number);
   return BuildPacket();
 }
@@ -160,11 +157,10 @@ QuicTestPacketMaker::MakeRetireConnectionIdPacket(uint64_t num,
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeNewConnectionIdPacket(
     uint64_t num,
-    bool include_version,
     const quic::QuicConnectionId& cid,
     uint64_t sequence_number,
     uint64_t retire_prior_to) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
   AddQuicNewConnectionIdFrame(
       cid, sequence_number, retire_prior_to,
       quic::QuicUtils::GenerateStatelessResetToken(cid));
@@ -174,13 +170,12 @@ QuicTestPacketMaker::MakeNewConnectionIdPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckAndNewConnectionIdPacket(
     uint64_t num,
-    bool include_version,
     uint64_t largest_received,
     uint64_t smallest_received,
     const quic::QuicConnectionId& cid,
     uint64_t sequence_number,
     uint64_t retire_prior_to) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
   AddQuicAckFrame(largest_received, smallest_received);
   AddQuicNewConnectionIdFrame(
       cid, sequence_number, retire_prior_to,
@@ -191,7 +186,7 @@ QuicTestPacketMaker::MakeAckAndNewConnectionIdPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeDummyCHLOPacket(uint64_t packet_num) {
   SetEncryptionLevel(quic::ENCRYPTION_INITIAL);
-  InitializeHeader(packet_num, /*include_version=*/true);
+  InitializeHeader(packet_num);
 
   quic::CryptoHandshakeMessage message =
       MockCryptoClientStream::GetDummyCHLOMessage();
@@ -215,10 +210,9 @@ QuicTestPacketMaker::MakeDummyCHLOPacket(uint64_t packet_num) {
 
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckAndPingPacket(uint64_t num,
-                                          bool include_version,
                                           uint64_t largest_received,
                                           uint64_t smallest_received) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
   AddQuicAckFrame(largest_received, smallest_received);
   AddQuicPingFrame();
   return BuildPacket();
@@ -227,11 +221,10 @@ QuicTestPacketMaker::MakeAckAndPingPacket(uint64_t num,
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckAndRetireConnectionIdPacket(
     uint64_t num,
-    bool include_version,
     uint64_t largest_received,
     uint64_t smallest_received,
     uint64_t sequence_number) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
   AddQuicAckFrame(largest_received, smallest_received);
   AddQuicRetireConnectionIdFrame(sequence_number);
   return BuildPacket();
@@ -240,10 +233,9 @@ QuicTestPacketMaker::MakeAckAndRetireConnectionIdPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeRetransmissionAndRetireConnectionIdPacket(
     uint64_t num,
-    bool include_version,
     const std::vector<uint64_t>& original_packet_numbers,
     uint64_t sequence_number) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
   for (auto it : original_packet_numbers) {
     for (auto frame : saved_frames_[quic::QuicPacketNumber(it)]) {
       if (!MaybeCoalesceStreamFrame(frame)) {
@@ -258,40 +250,36 @@ QuicTestPacketMaker::MakeRetransmissionAndRetireConnectionIdPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeStreamsBlockedPacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamCount stream_count,
     bool unidirectional) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
   AddQuicStreamsBlockedFrame(1, stream_count, unidirectional);
   return BuildPacket();
 }
 
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeMaxStreamsPacket(uint64_t num,
-                                          bool include_version,
                                           quic::QuicStreamCount stream_count,
                                           bool unidirectional) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
   AddQuicMaxStreamsFrame(1, stream_count, unidirectional);
   return BuildPacket();
 }
 
 std::unique_ptr<quic::QuicReceivedPacket> QuicTestPacketMaker::MakeRstPacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId stream_id,
     quic::QuicRstStreamErrorCode error_code) {
-  return MakeRstPacket(num, include_version, stream_id, error_code,
+  return MakeRstPacket(num, stream_id, error_code,
                        /*include_stop_sending_if_v99=*/true);
 }
 
 std::unique_ptr<quic::QuicReceivedPacket> QuicTestPacketMaker::MakeRstPacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId stream_id,
     quic::QuicRstStreamErrorCode error_code,
     bool include_stop_sending_if_v99) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   if (include_stop_sending_if_v99 && version_.HasIetfQuicFrames()) {
     AddQuicStopSendingFrame(stream_id, error_code);
@@ -307,12 +295,11 @@ std::unique_ptr<quic::QuicReceivedPacket> QuicTestPacketMaker::MakeRstPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeRstAndDataPacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId rst_stream_id,
     quic::QuicRstStreamErrorCode rst_error_code,
     quic::QuicStreamId data_stream_id,
     absl::string_view data) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   if (version_.HasIetfQuicFrames()) {
     AddQuicStopSendingFrame(rst_stream_id, rst_error_code);
@@ -328,14 +315,13 @@ std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeRetransmissionRstAndDataPacket(
     const std::vector<uint64_t>& original_packet_numbers,
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId rst_stream_id,
     quic::QuicRstStreamErrorCode rst_error_code,
     quic::QuicStreamId data_stream_id,
     absl::string_view data,
     uint64_t retransmit_frame_count) {
   DCHECK(save_packet_frames_);
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   uint64_t frame_count = 0;
   for (auto it : original_packet_numbers) {
@@ -363,12 +349,11 @@ QuicTestPacketMaker::MakeRetransmissionRstAndDataPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeDataAndRstPacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId data_stream_id,
     absl::string_view data,
     quic::QuicStreamId rst_stream_id,
     quic::QuicRstStreamErrorCode rst_error_code) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   AddQuicStreamFrame(data_stream_id, /* fin = */ false, data);
   if (version_.HasIetfQuicFrames()) {
@@ -382,14 +367,13 @@ QuicTestPacketMaker::MakeDataAndRstPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeDataRstAndAckPacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId data_stream_id,
     absl::string_view data,
     quic::QuicStreamId rst_stream_id,
     quic::QuicRstStreamErrorCode rst_error_code,
     uint64_t largest_received,
     uint64_t smallest_received) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   AddQuicAckFrame(largest_received, smallest_received);
 
@@ -405,26 +389,24 @@ QuicTestPacketMaker::MakeDataRstAndAckPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckAndRstPacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId stream_id,
     quic::QuicRstStreamErrorCode error_code,
     uint64_t largest_received,
     uint64_t smallest_received) {
-  return MakeAckAndRstPacket(num, include_version, stream_id, error_code,
-                             largest_received, smallest_received,
+  return MakeAckAndRstPacket(num, stream_id, error_code, largest_received,
+                             smallest_received,
                              /*include_stop_sending_if_v99=*/true);
 }
 
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckAndRstPacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId stream_id,
     quic::QuicRstStreamErrorCode error_code,
     uint64_t largest_received,
     uint64_t smallest_received,
     bool include_stop_sending_if_v99) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   AddQuicAckFrame(largest_received, smallest_received);
 
@@ -442,16 +424,15 @@ QuicTestPacketMaker::MakeAckAndRstPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeRstAckAndConnectionClosePacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId stream_id,
     quic::QuicRstStreamErrorCode error_code,
     uint64_t largest_received,
     uint64_t smallest_received,
     quic::QuicErrorCode quic_error,
     const std::string& quic_error_details) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
-    AddQuicAckFrame(largest_received, smallest_received);
+  AddQuicAckFrame(largest_received, smallest_received);
 
   if (version_.HasIetfQuicFrames()) {
     AddQuicStopSendingFrame(stream_id, error_code);
@@ -465,7 +446,6 @@ QuicTestPacketMaker::MakeRstAckAndConnectionClosePacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeRstAckAndDataPacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId stream_id,
     quic::QuicRstStreamErrorCode error_code,
     uint64_t largest_received,
@@ -473,7 +453,7 @@ QuicTestPacketMaker::MakeRstAckAndDataPacket(
     quic::QuicStreamId data_id,
     bool fin,
     absl::string_view data) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   AddQuicRstStreamFrame(stream_id, error_code);
 
@@ -484,7 +464,6 @@ QuicTestPacketMaker::MakeRstAckAndDataPacket(
 
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckDataAndRst(uint64_t num,
-                                       bool include_version,
                                        quic::QuicStreamId stream_id,
                                        quic::QuicRstStreamErrorCode error_code,
                                        uint64_t largest_received,
@@ -492,7 +471,7 @@ QuicTestPacketMaker::MakeAckDataAndRst(uint64_t num,
                                        quic::QuicStreamId data_id,
                                        bool fin,
                                        absl::string_view data) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   AddQuicAckFrame(largest_received, smallest_received);
   AddQuicStreamFrame(data_id, fin, data);
@@ -506,7 +485,6 @@ QuicTestPacketMaker::MakeAckDataAndRst(uint64_t num,
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckRstAndDataPacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId stream_id,
     quic::QuicRstStreamErrorCode error_code,
     uint64_t largest_received,
@@ -514,7 +492,7 @@ QuicTestPacketMaker::MakeAckRstAndDataPacket(
     quic::QuicStreamId data_id,
     bool fin,
     absl::string_view data) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   AddQuicAckFrame(largest_received, smallest_received);
   AddQuicRstStreamFrame(stream_id, error_code);
@@ -530,7 +508,7 @@ QuicTestPacketMaker::MakeAckAndRetransmissionPacket(
     uint64_t smallest_received,
     const std::vector<uint64_t>& original_packet_numbers) {
   DCHECK(save_packet_frames_);
-  InitializeHeader(packet_number, /*include_version=*/false);
+  InitializeHeader(packet_number);
   AddQuicAckFrame(first_received, largest_received, smallest_received);
   for (auto it : original_packet_numbers) {
     for (auto frame : saved_frames_[quic::QuicPacketNumber(it)]) {
@@ -546,10 +524,9 @@ QuicTestPacketMaker::MakeAckAndRetransmissionPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeCombinedRetransmissionPacket(
     const std::vector<uint64_t>& original_packet_numbers,
-    uint64_t new_packet_number,
-    bool should_include_version) {
+    uint64_t new_packet_number) {
   DCHECK(save_packet_frames_);
-  InitializeHeader(new_packet_number, should_include_version);
+  InitializeHeader(new_packet_number);
   for (auto it : original_packet_numbers) {
     for (auto& frame : CloneFrames(saved_frames_[quic::QuicPacketNumber(it)])) {
       if (frame.type != quic::PADDING_FRAME) {
@@ -566,12 +543,11 @@ QuicTestPacketMaker::MakeCombinedRetransmissionPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeRstAndConnectionClosePacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId stream_id,
     quic::QuicRstStreamErrorCode error_code,
     quic::QuicErrorCode quic_error,
     const std::string& quic_error_details) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   if (version_.HasIetfQuicFrames()) {
     AddQuicStopSendingFrame(stream_id, error_code);
@@ -586,14 +562,13 @@ QuicTestPacketMaker::MakeRstAndConnectionClosePacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeDataRstAndConnectionClosePacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId data_stream_id,
     absl::string_view data,
     quic::QuicStreamId rst_stream_id,
     quic::QuicRstStreamErrorCode error_code,
     quic::QuicErrorCode quic_error,
     const std::string& quic_error_details) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   AddQuicStreamFrame(data_stream_id, /* fin = */ false, data);
   if (version_.HasIetfQuicFrames()) {
@@ -609,7 +584,6 @@ QuicTestPacketMaker::MakeDataRstAndConnectionClosePacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeDataRstAckAndConnectionClosePacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId data_stream_id,
     absl::string_view data,
     quic::QuicStreamId rst_stream_id,
@@ -618,7 +592,7 @@ QuicTestPacketMaker::MakeDataRstAckAndConnectionClosePacket(
     uint64_t smallest_received,
     quic::QuicErrorCode quic_error,
     const std::string& quic_error_details) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   AddQuicAckFrame(largest_received, smallest_received);
 
@@ -636,7 +610,6 @@ QuicTestPacketMaker::MakeDataRstAckAndConnectionClosePacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeDataRstAckAndConnectionClosePacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId data_stream_id,
     absl::string_view data,
     quic::QuicStreamId rst_stream_id,
@@ -646,7 +619,7 @@ QuicTestPacketMaker::MakeDataRstAckAndConnectionClosePacket(
     quic::QuicErrorCode quic_error,
     const std::string& quic_error_details,
     uint64_t frame_type) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
 
   AddQuicStreamFrame(data_stream_id, /* fin = */ false, data);
   if (version_.HasIetfQuicFrames()) {
@@ -663,12 +636,11 @@ QuicTestPacketMaker::MakeDataRstAckAndConnectionClosePacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeStopSendingPacket(
     uint64_t num,
-    bool include_version,
     quic::QuicStreamId stream_id,
     quic::QuicRstStreamErrorCode error_code) {
   DCHECK(version_.HasIetfQuicFrames());
 
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
   AddQuicStopSendingFrame(stream_id, error_code);
 
   return BuildPacket();
@@ -677,13 +649,12 @@ QuicTestPacketMaker::MakeStopSendingPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckAndConnectionClosePacket(
     uint64_t num,
-    bool include_version,
     uint64_t largest_received,
     uint64_t smallest_received,
     quic::QuicErrorCode quic_error,
     const std::string& quic_error_details,
     uint64_t frame_type) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
   AddQuicAckFrame(largest_received, smallest_received);
   AddQuicConnectionCloseFrame(quic_error, quic_error_details, frame_type);
   return BuildPacket();
@@ -692,10 +663,9 @@ QuicTestPacketMaker::MakeAckAndConnectionClosePacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeConnectionClosePacket(
     uint64_t num,
-    bool include_version,
     quic::QuicErrorCode quic_error,
     const std::string& quic_error_details) {
-  InitializeHeader(num, include_version);
+  InitializeHeader(num);
   AddQuicConnectionCloseFrame(quic_error, quic_error_details);
   return BuildPacket();
 }
@@ -704,7 +674,7 @@ std::unique_ptr<quic::QuicReceivedPacket> QuicTestPacketMaker::MakeGoAwayPacket(
     uint64_t num,
     quic::QuicErrorCode error_code,
     std::string reason_phrase) {
-  InitializeHeader(num, /*include_version=*/false);
+  InitializeHeader(num);
   AddQuicGoAwayFrame(error_code, reason_phrase);
   return BuildPacket();
 }
@@ -721,7 +691,7 @@ std::unique_ptr<quic::QuicReceivedPacket> QuicTestPacketMaker::MakeAckPacket(
     uint64_t first_received,
     uint64_t largest_received,
     uint64_t smallest_received) {
-  InitializeHeader(packet_number, /*include_version=*/false);
+  InitializeHeader(packet_number);
   AddQuicAckFrame(first_received, largest_received, smallest_received);
   return BuildPacket();
 }
@@ -729,23 +699,21 @@ std::unique_ptr<quic::QuicReceivedPacket> QuicTestPacketMaker::MakeAckPacket(
 std::unique_ptr<quic::QuicReceivedPacket> QuicTestPacketMaker::MakeDataPacket(
     uint64_t packet_number,
     quic::QuicStreamId stream_id,
-    bool should_include_version,
     bool fin,
     absl::string_view data) {
-  InitializeHeader(packet_number, should_include_version);
+  InitializeHeader(packet_number);
   AddQuicStreamFrame(stream_id, fin, data);
   return BuildPacket();
 }
 
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckAndDataPacket(uint64_t packet_number,
-                                          bool include_version,
                                           quic::QuicStreamId stream_id,
                                           uint64_t largest_received,
                                           uint64_t smallest_received,
                                           bool fin,
                                           absl::string_view data) {
-  InitializeHeader(packet_number, include_version);
+  InitializeHeader(packet_number);
 
   AddQuicAckFrame(largest_received, smallest_received);
   AddQuicStreamFrame(stream_id, fin, data);
@@ -756,14 +724,13 @@ QuicTestPacketMaker::MakeAckAndDataPacket(uint64_t packet_number,
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckRetransmissionAndDataPacket(
     uint64_t packet_number,
-    bool include_version,
     const std::vector<uint64_t>& original_packet_numbers,
     quic::QuicStreamId stream_id,
     uint64_t largest_received,
     uint64_t smallest_received,
     bool fin,
     absl::string_view data) {
-  InitializeHeader(packet_number, include_version);
+  InitializeHeader(packet_number);
 
   AddQuicAckFrame(largest_received, smallest_received);
   for (auto it : original_packet_numbers) {
@@ -782,13 +749,12 @@ std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeRequestHeadersAndMultipleDataFramesPacket(
     uint64_t packet_number,
     quic::QuicStreamId stream_id,
-    bool should_include_version,
     bool fin,
     spdy::SpdyPriority spdy_priority,
     spdy::Http2HeaderBlock headers,
     size_t* spdy_headers_frame_length,
     const std::vector<std::string>& data_writes) {
-  InitializeHeader(packet_number, should_include_version);
+  InitializeHeader(packet_number);
 
   MaybeAddHttp3SettingsFrames();
 
@@ -812,13 +778,12 @@ std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeRequestHeadersPacket(
     uint64_t packet_number,
     quic::QuicStreamId stream_id,
-    bool should_include_version,
     bool fin,
     spdy::SpdyPriority spdy_priority,
     spdy::Http2HeaderBlock headers,
     size_t* spdy_headers_frame_length,
     bool should_include_priority_frame) {
-  InitializeHeader(packet_number, should_include_version);
+  InitializeHeader(packet_number);
 
   MaybeAddHttp3SettingsFrames();
 
@@ -842,13 +807,12 @@ QuicTestPacketMaker::MakeRetransmissionAndRequestHeadersPacket(
     const std::vector<uint64_t>& original_packet_numbers,
     uint64_t packet_number,
     quic::QuicStreamId stream_id,
-    bool should_include_version,
     bool fin,
     spdy::SpdyPriority spdy_priority,
     spdy::Http2HeaderBlock headers,
     size_t* spdy_headers_frame_length) {
   DCHECK(save_packet_frames_);
-  InitializeHeader(packet_number, should_include_version);
+  InitializeHeader(packet_number);
 
   for (auto it : original_packet_numbers) {
     for (auto frame : saved_frames_[quic::QuicPacketNumber(it)]) {
@@ -877,13 +841,12 @@ std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeRequestHeadersAndRstPacket(
     uint64_t packet_number,
     quic::QuicStreamId stream_id,
-    bool should_include_version,
     bool fin,
     spdy::SpdyPriority spdy_priority,
     spdy::Http2HeaderBlock headers,
     size_t* spdy_headers_frame_length,
     quic::QuicRstStreamErrorCode error_code) {
-  InitializeHeader(packet_number, should_include_version);
+  InitializeHeader(packet_number);
 
   MaybeAddHttp3SettingsFrames();
 
@@ -906,11 +869,10 @@ std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeResponseHeadersPacket(
     uint64_t packet_number,
     quic::QuicStreamId stream_id,
-    bool should_include_version,
     bool fin,
     spdy::Http2HeaderBlock headers,
     size_t* spdy_headers_frame_length) {
-  InitializeHeader(packet_number, should_include_version);
+  InitializeHeader(packet_number);
 
   std::string data = QpackEncodeHeaders(stream_id, std::move(headers),
                                         spdy_headers_frame_length);
@@ -922,17 +884,16 @@ QuicTestPacketMaker::MakeResponseHeadersPacket(
 
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeInitialSettingsPacket(uint64_t packet_number) {
-  InitializeHeader(packet_number, /*should_include_version*/ true);
+  InitializeHeader(packet_number);
   MaybeAddHttp3SettingsFrames();
   return BuildPacket();
 }
 
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakePriorityPacket(uint64_t packet_number,
-                                        bool should_include_version,
                                         quic::QuicStreamId id,
                                         spdy::SpdyPriority spdy_priority) {
-  InitializeHeader(packet_number, should_include_version);
+  InitializeHeader(packet_number);
 
   std::string priority_data = GenerateHttp3PriorityData(spdy_priority, id);
   if (!priority_data.empty()) {
@@ -945,12 +906,11 @@ QuicTestPacketMaker::MakePriorityPacket(uint64_t packet_number,
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckAndPriorityPacket(
     uint64_t packet_number,
-    bool should_include_version,
     uint64_t largest_received,
     uint64_t smallest_received,
     quic::QuicStreamId id,
     spdy::SpdyPriority spdy_priority) {
-  InitializeHeader(packet_number, should_include_version);
+  InitializeHeader(packet_number);
 
   AddQuicAckFrame(largest_received, smallest_received);
 
@@ -965,12 +925,11 @@ QuicTestPacketMaker::MakeAckAndPriorityPacket(
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckAndPriorityUpdatePacket(
     uint64_t packet_number,
-    bool should_include_version,
     uint64_t largest_received,
     uint64_t smallest_received,
     quic::QuicStreamId id,
     spdy::SpdyPriority spdy_priority) {
-  InitializeHeader(packet_number, should_include_version);
+  InitializeHeader(packet_number);
 
   AddQuicAckFrame(largest_received, smallest_received);
 
@@ -984,10 +943,9 @@ QuicTestPacketMaker::MakeAckAndPriorityUpdatePacket(
 
 std::unique_ptr<quic::QuicReceivedPacket>
 QuicTestPacketMaker::MakeRetransmissionPacket(uint64_t original_packet_number,
-                                              uint64_t new_packet_number,
-                                              bool should_include_version) {
+                                              uint64_t new_packet_number) {
   DCHECK(save_packet_frames_);
-  InitializeHeader(new_packet_number, should_include_version);
+  InitializeHeader(new_packet_number);
   return BuildPacketImpl(
       saved_frames_[quic::QuicPacketNumber(original_packet_number)], nullptr);
 }
@@ -1088,13 +1046,6 @@ std::string QuicTestPacketMaker::QpackEncodeHeaders(
   const std::string headers_frame_header =
       quic::HttpEncoder::SerializeHeadersFrameHeader(encoded_headers.size());
 
-  // Possible add a PUSH stream type.
-  if (!quic::QuicUtils::IsBidirectionalStreamId(stream_id, version_) &&
-      stream_offsets_[stream_id] == 0) {
-    // Push stream type header
-    data += "\x01";
-  }
-
   // Add the HEADERS frame header.
   data += headers_frame_header;
   // Add the HEADERS frame payload.
@@ -1107,18 +1058,15 @@ std::string QuicTestPacketMaker::QpackEncodeHeaders(
   return data;
 }
 
-void QuicTestPacketMaker::InitializeHeader(uint64_t packet_number,
-                                           bool should_include_version) {
+void QuicTestPacketMaker::InitializeHeader(uint64_t packet_number) {
   header_.destination_connection_id = DestinationConnectionId();
   header_.destination_connection_id_included = HasDestinationConnectionId();
   header_.source_connection_id = SourceConnectionId();
   header_.source_connection_id_included = HasSourceConnectionId();
   header_.reset_flag = false;
-  header_.version_flag = ShouldIncludeVersion(should_include_version);
-  if (quic::VersionHasIetfInvariantHeader(version_.transport_version)) {
-    header_.form = header_.version_flag ? quic::IETF_QUIC_LONG_HEADER_PACKET
-                                        : quic::IETF_QUIC_SHORT_HEADER_PACKET;
-  }
+  header_.version_flag = ShouldIncludeVersion();
+  header_.form = header_.version_flag ? quic::IETF_QUIC_LONG_HEADER_PACKET
+                                      : quic::IETF_QUIC_SHORT_HEADER_PACKET;
   header_.long_packet_type = long_header_type_;
   header_.packet_number_length = GetPacketNumberLength();
   header_.packet_number = quic::QuicPacketNumber(packet_number);
@@ -1377,17 +1325,13 @@ std::unique_ptr<quic::QuicReceivedPacket> QuicTestPacketMaker::BuildPacketImpl(
   return encrypted.Clone();
 }
 
-bool QuicTestPacketMaker::ShouldIncludeVersion(bool include_version) const {
-  if (version_.HasIetfInvariantHeader()) {
-    return encryption_level_ < quic::ENCRYPTION_FORWARD_SECURE;
-  }
-  return include_version;
+bool QuicTestPacketMaker::ShouldIncludeVersion() const {
+  return encryption_level_ < quic::ENCRYPTION_FORWARD_SECURE;
 }
 
 quic::QuicPacketNumberLength QuicTestPacketMaker::GetPacketNumberLength()
     const {
-  if (version_.HasIetfInvariantHeader() &&
-      encryption_level_ < quic::ENCRYPTION_FORWARD_SECURE &&
+  if (ShouldIncludeVersion() &&
       !version_.SendsVariableLengthPacketNumberInLongHeader()) {
     return quic::PACKET_4BYTE_PACKET_NUMBER;
   }
@@ -1456,8 +1400,7 @@ std::string QuicTestPacketMaker::GenerateHttp3PriorityData(
   quic::PriorityUpdateFrame priority_update;
   quic::HttpStreamPriority priority{
       spdy_priority, quic::HttpStreamPriority::kDefaultIncremental};
-  if (client_priority_uses_incremental_ &&
-      base::FeatureList::IsEnabled(features::kPriorityIncremental)) {
+  if (client_priority_uses_incremental_) {
     priority.incremental = kDefaultPriorityIncremental;
   }
 
