@@ -38,6 +38,9 @@ class StatsCounter;
   V(isolate_root, "Isolate::isolate_root()")                                   \
   V(allocation_sites_list_address, "Heap::allocation_sites_list_address()")    \
   V(address_of_jslimit, "StackGuard::address_of_jslimit()")                    \
+  V(address_of_no_heap_write_interrupt_request,                                \
+    "StackGuard::address_of_interrupt_request(StackGuard::InterruptLevel::"    \
+    "kNoHeapWrites)")                                                          \
   V(address_of_real_jslimit, "StackGuard::address_of_real_jslimit()")          \
   V(heap_is_marking_flag_address, "heap_is_marking_flag_address")              \
   V(heap_is_minor_marking_flag_address, "heap_is_minor_marking_flag_address")  \
@@ -71,6 +74,8 @@ class StatsCounter;
   V(fast_c_call_caller_pc_address,                                             \
     "IsolateData::fast_c_call_caller_pc_address")                              \
   V(fast_api_call_target_address, "IsolateData::fast_api_call_target_address") \
+  V(api_callback_thunk_argument_address,                                       \
+    "IsolateData::api_callback_thunk_argument_address")                        \
   V(stack_is_iterable_address, "IsolateData::stack_is_iterable_address")       \
   V(address_of_regexp_stack_limit_address,                                     \
     "RegExpStack::limit_address_address()")                                    \
@@ -132,6 +137,8 @@ class StatsCounter;
   V(copy_typed_array_elements_to_typed_array,                                  \
     "copy_typed_array_elements_to_typed_array")                                \
   V(cpu_features, "cpu_features")                                              \
+  V(debug_break_at_entry_function, "DebugBreakAtEntry")                        \
+  V(debug_get_coverage_info_function, "DebugGetCoverageInfo")                  \
   V(delete_handle_scope_extensions, "HandleScope::DeleteExtensions")           \
   V(ephemeron_key_write_barrier_function,                                      \
     "Heap::EphemeronKeyWriteBarrierFromCode")                                  \
@@ -166,7 +173,8 @@ class StatsCounter;
   V(invalidate_prototype_chains_function,                                      \
     "JSObject::InvalidatePrototypeChains()")                                   \
   V(invoke_accessor_getter_callback, "InvokeAccessorGetterCallback")           \
-  V(invoke_function_callback, "InvokeFunctionCallback")                        \
+  V(invoke_function_callback_generic, "InvokeFunctionCallbackGeneric")         \
+  V(invoke_function_callback_optimized, "InvokeFunctionCallbackOptimized")     \
   V(jsarray_array_join_concat_to_sequential_string,                            \
     "jsarray_array_join_concat_to_sequential_string")                          \
   V(jsreceiver_create_identity_hash, "jsreceiver_create_identity_hash")        \
@@ -247,8 +255,11 @@ class StatsCounter;
     "name_to_index_hashtable_lookup_forwarded_string")                         \
   V(name_to_index_hashtable_find_insertion_entry_forwarded_string,             \
     "name_to_index_hashtable_find_insertion_entry_forwarded_string")           \
-  IF_WASM(V, wasm_call_trap_callback_for_testing,                              \
-          "wasm::call_trap_callback_for_testing")                              \
+  IF_WASM(V, wasm_sync_stack_limit, "wasm_sync_stack_limit")                   \
+  IF_WASM(V, wasm_switch_to_the_central_stack,                                 \
+          "wasm::switch_to_the_central_stack")                                 \
+  IF_WASM(V, wasm_switch_from_the_central_stack,                               \
+          "wasm::switch_from_the_central_stack")                               \
   IF_WASM(V, wasm_f32_ceil, "wasm::f32_ceil_wrapper")                          \
   IF_WASM(V, wasm_f32_floor, "wasm::f32_floor_wrapper")                        \
   IF_WASM(V, wasm_f32_nearest_int, "wasm::f32_nearest_int_wrapper")            \
@@ -297,6 +308,7 @@ class StatsCounter;
   IF_WASM(V, wasm_memory_fill, "wasm::memory_fill")                            \
   IF_WASM(V, wasm_array_copy, "wasm::array_copy")                              \
   IF_WASM(V, wasm_array_fill, "wasm::array_fill")                              \
+  IF_WASM(V, wasm_string_to_f64, "wasm_string_to_f64")                         \
   V(address_of_wasm_i8x16_swizzle_mask, "wasm_i8x16_swizzle_mask")             \
   V(address_of_wasm_i8x16_popcnt_mask, "wasm_i8x16_popcnt_mask")               \
   V(address_of_wasm_i8x16_splat_0x01, "wasm_i8x16_splat_0x01")                 \
@@ -313,6 +325,8 @@ class StatsCounter;
   V(address_of_wasm_int32_overflow_as_float, "wasm_int32_overflow_as_float")   \
   V(supports_cetss_address, "CpuFeatures::supports_cetss_address")             \
   V(write_barrier_marking_from_code_function, "WriteBarrier::MarkingFromCode") \
+  V(write_barrier_indirect_pointer_marking_from_code_function,                 \
+    "WriteBarrier::IndirectPointerMarkingFromCode")                            \
   V(write_barrier_shared_marking_from_code_function,                           \
     "WriteBarrier::SharedMarkingFromCode")                                     \
   V(shared_barrier_from_code_function, "WriteBarrier::SharedFromCode")         \
@@ -381,10 +395,11 @@ class StatsCounter;
 #endif  // V8_INTL_SUPPORT
 
 #ifdef V8_ENABLE_SANDBOX
-#define EXTERNAL_REFERENCE_LIST_SANDBOX(V)   \
-  V(sandbox_base_address, "Sandbox::base()") \
-  V(sandbox_end_address, "Sandbox::end()")   \
-  V(empty_backing_store_buffer, "EmptyBackingStoreBuffer()")
+#define EXTERNAL_REFERENCE_LIST_SANDBOX(V)                   \
+  V(sandbox_base_address, "Sandbox::base()")                 \
+  V(sandbox_end_address, "Sandbox::end()")                   \
+  V(empty_backing_store_buffer, "EmptyBackingStoreBuffer()") \
+  V(code_pointer_table_address, "GetProcessWideCodePointerTable()")
 #else
 #define EXTERNAL_REFERENCE_LIST_SANDBOX(V)
 #endif  // V8_ENABLE_SANDBOX
@@ -426,23 +441,18 @@ class ExternalReference {
     // double f(double, int).
     BUILTIN_FP_INT_CALL,
 
+    // Builtin call that returns floating point.
+    // double f(Address tagged_ptr).
+    BUILTIN_FP_POINTER_CALL,
+
     // Direct call to API function callback.
     // void f(v8::FunctionCallbackInfo&)
     DIRECT_API_CALL,
 
-    // Call to function callback via InvokeFunctionCallback.
-    // void f(v8::FunctionCallbackInfo&, v8::FunctionCallback)
-    PROFILING_API_CALL,
-
     // Direct call to accessor getter callback.
     // void f(Local<Name> property, PropertyCallbackInfo& info)
     DIRECT_GETTER_CALL,
 
-    // Call to accessor getter callback via InvokeAccessorGetterCallback.
-    // void f(Local<Name> property, PropertyCallbackInfo& info,
-    //     AccessorNameGetterCallback callback)
-    PROFILING_GETTER_CALL,
-
     // C call, either representing a fast API call or used in tests.
     // Can have arbitrary signature from the types supported by the fast API.
     FAST_C_CALL
@@ -501,6 +511,8 @@ class ExternalReference {
   static V8_EXPORT_PRIVATE ExternalReference
   address_of_store_to_stack_count(const char* function_name);
 
+  static ExternalReference invoke_function_callback(CallApiCallbackMode mode);
+
   Address address() const { return address_; }
 
   // Creates a redirection trampoline for given C function and signature for
