@@ -15,6 +15,7 @@
 #include "src/compiler/linkage.h"
 #include "src/compiler/type-cache.h"
 // For TNode types.
+#include "src/deoptimizer/deoptimize-reason.h"
 #include "src/objects/elements-kind.h"
 #include "src/objects/heap-number.h"
 #include "src/objects/instance-type.h"
@@ -172,6 +173,15 @@ PURE_ASSEMBLER_MACH_BINOP_LIST(PURE_BINOP_DEF, PURE_BINOP_DEF_TNODE)
 #undef PURE_BINOP_DEF
 #undef PURE_BINOP_DEF_TNODE
 
+TNode<BoolT> GraphAssembler::UintPtrLessThan(TNode<UintPtrT> left,
+                                             TNode<UintPtrT> right) {
+  return kSystemPointerSize == 8
+             ? Uint64LessThan(TNode<Uint64T>::UncheckedCast(left),
+                              TNode<Uint64T>::UncheckedCast(right))
+             : Uint32LessThan(TNode<Uint32T>::UncheckedCast(left),
+                              TNode<Uint32T>::UncheckedCast(right));
+}
+
 TNode<BoolT> GraphAssembler::UintPtrLessThanOrEqual(TNode<UintPtrT> left,
                                                     TNode<UintPtrT> right) {
   return kSystemPointerSize == 8
@@ -260,9 +270,9 @@ Node* GraphAssembler::TruncateFloat64ToInt64(Node* value, TruncateKind kind) {
       graph()->NewNode(machine()->TruncateFloat64ToInt64(kind), value));
 }
 
-Node* GraphAssembler::Projection(int index, Node* value) {
-  return AddNode(
-      graph()->NewNode(common()->Projection(index), value, control()));
+Node* GraphAssembler::Projection(int index, Node* value, Node* ctrl) {
+  return AddNode(graph()->NewNode(common()->Projection(index), value,
+                                  ctrl ? ctrl : control()));
 }
 
 Node* JSGraphAssembler::Allocate(AllocationType allocation, Node* size) {
@@ -433,9 +443,10 @@ TNode<Boolean> JSGraphAssembler::ObjectIsUndetectable(TNode<Object> value) {
       graph()->NewNode(simplified()->ObjectIsUndetectable(), value));
 }
 
-Node* JSGraphAssembler::CheckIf(Node* cond, DeoptimizeReason reason) {
-  return AddNode(graph()->NewNode(simplified()->CheckIf(reason), cond, effect(),
-                                  control()));
+Node* JSGraphAssembler::CheckIf(Node* cond, DeoptimizeReason reason,
+                                const FeedbackSource& feedback) {
+  return AddNode(graph()->NewNode(simplified()->CheckIf(reason, feedback), cond,
+                                  effect(), control()));
 }
 
 TNode<Boolean> JSGraphAssembler::NumberIsFloat64Hole(TNode<Number> value) {
@@ -481,6 +492,11 @@ Node* JSGraphAssembler::StringCharCodeAt(TNode<String> string,
                                   position, effect(), control()));
 }
 
+TNode<String> JSGraphAssembler::StringFromSingleCharCode(TNode<Number> code) {
+  return AddNode<String>(
+      graph()->NewNode(simplified()->StringFromSingleCharCode(), code));
+}
+
 class ArrayBufferViewAccessBuilder {
  public:
   explicit ArrayBufferViewAccessBuilder(JSGraphAssembler* assembler,
@@ -767,6 +783,64 @@ class ArrayBufferViewAccessBuilder {
         .Value();
   }
 
+  TNode<Word32T> BuildDetachedCheck(TNode<JSArrayBufferView> view) {
+    auto& a = *assembler_;
+
+    // Load the underlying buffer and its bitfield.
+    TNode<HeapObject> buffer = a.LoadField<HeapObject>(
+        AccessBuilder::ForJSArrayBufferViewBuffer(), view);
+    TNode<Word32T> buffer_bit_field =
+        MachineLoadField<Word32T>(AccessBuilder::ForJSArrayBufferBitField(),
+                                  buffer, UseInfo::TruncatingWord32());
+    // Mask the detached bit.
+    TNode<Word32T> detached_bit =
+        a.Word32And(buffer_bit_field,
+                    a.Uint32Constant(JSArrayBuffer::WasDetachedBit::kMask));
+
+    // If we statically know we cannot have rab/gsab backed, we are done here.
+    if (!maybe_rab_gsab()) {
+      return detached_bit;
+    }
+
+    // Otherwise, we need to generate the checks for the view's bitfield.
+    TNode<Word32T> bitfield = a.EnterMachineGraph<Word32T>(
+        a.LoadField<Word32T>(AccessBuilder::ForJSArrayBufferViewBitField(),
+                             view),
+        UseInfo::TruncatingWord32());
+    TNode<Word32T> length_tracking_bit = a.Word32And(
+        bitfield, a.Uint32Constant(JSArrayBufferView::kIsLengthTracking));
+    TNode<Word32T> backed_by_rab_bit = a.Word32And(
+        bitfield, a.Uint32Constant(JSArrayBufferView::kIsBackedByRab));
+
+    auto RabFixed = [&]() {
+      TNode<UintPtrT> unchecked_byte_length = MachineLoadField<UintPtrT>(
+          AccessBuilder::ForJSArrayBufferViewByteLength(), view,
+          UseInfo::Word());
+      TNode<UintPtrT> byte_offset = MachineLoadField<UintPtrT>(
+          AccessBuilder::ForJSArrayBufferViewByteOffset(), view,
+          UseInfo::Word());
+
+      TNode<UintPtrT> underlying_byte_length = MachineLoadField<UintPtrT>(
+          AccessBuilder::ForJSArrayBufferByteLength(), buffer, UseInfo::Word());
+
+      return a.Word32Or(
+          detached_bit,
+          a.UintPtrLessThan(underlying_byte_length,
+                            a.UintPtrAdd(byte_offset, unchecked_byte_length)));
+    };
+
+    // Dispatch depending on rab/gsab and length tracking.
+    return a.MachineSelectIf<Word32T>(length_tracking_bit)
+        .Then([&]() { return detached_bit; })
+        .Else([&]() {
+          return a.MachineSelectIf<Word32T>(backed_by_rab_bit)
+              .Then(RabFixed)
+              .Else([&]() { return detached_bit; })
+              .Value();
+        })
+        .Value();
+  }
+
  private:
   template <typename T>
   TNode<T> MachineLoadField(FieldAccess const& access, TNode<HeapObject> object,
@@ -795,12 +869,26 @@ TNode<Number> JSGraphAssembler::TypedArrayLength(
     TNode<JSTypedArray> typed_array,
     std::set<ElementsKind> elements_kinds_candidates, TNode<Context> context) {
   ArrayBufferViewAccessBuilder builder(this, JS_TYPED_ARRAY_TYPE,
-                                       elements_kinds_candidates);
+                                       std::move(elements_kinds_candidates));
   return ExitMachineGraph<Number>(builder.BuildLength(typed_array, context),
                                   MachineType::PointerRepresentation(),
                                   TypeCache::Get()->kJSTypedArrayLengthType);
 }
 
+void JSGraphAssembler::CheckIfTypedArrayWasDetached(
+    TNode<JSTypedArray> typed_array,
+    std::set<ElementsKind> elements_kinds_candidates,
+    const FeedbackSource& feedback) {
+  ArrayBufferViewAccessBuilder builder(this, JS_TYPED_ARRAY_TYPE,
+                                       std::move(elements_kinds_candidates));
+
+  TNode<Word32T> detached_check = builder.BuildDetachedCheck(typed_array);
+  TNode<Boolean> is_not_detached =
+      ExitMachineGraph<Boolean>(Word32Equal(detached_check, Uint32Constant(0)),
+                                MachineRepresentation::kBit, Type::Boolean());
+  CheckIf(is_not_detached, DeoptimizeReason::kArrayBufferWasDetached, feedback);
+}
+
 TNode<Uint32T> JSGraphAssembler::LookupByteShiftForElementsKind(
     TNode<Uint32T> elements_kind) {
   TNode<UintPtrT> index = ChangeUint32ToUintPtr(Int32Sub(
