@@ -72,11 +72,16 @@ enum class OddballType : uint8_t {
   kBoolean,  // True or False.
   kUndefined,
   kNull,
-  kHole,
   kUninitialized,
   kOther  // Oddball, but none of the above.
 };
 
+enum class HoleType : uint8_t {
+  kNone,  // Not a Hole.
+  kGeneric,
+  kPropertyCell,
+};
+
 enum class RefSerializationKind {
   // Skips serialization.
   kNeverSerialized,
@@ -203,12 +208,24 @@ struct ref_traits<Object> {
 template <>
 struct ref_traits<Oddball> : public ref_traits<HeapObject> {};
 template <>
+struct ref_traits<Null> : public ref_traits<HeapObject> {};
+template <>
+struct ref_traits<Undefined> : public ref_traits<HeapObject> {};
+template <>
+struct ref_traits<True> : public ref_traits<HeapObject> {};
+template <>
+struct ref_traits<False> : public ref_traits<HeapObject> {};
+template <>
+struct ref_traits<Hole> : public ref_traits<HeapObject> {};
+template <>
 struct ref_traits<EnumCache> : public ref_traits<HeapObject> {};
 template <>
 struct ref_traits<PropertyArray> : public ref_traits<HeapObject> {};
 template <>
 struct ref_traits<ByteArray> : public ref_traits<HeapObject> {};
 template <>
+struct ref_traits<ExternalPointerArray> : public ref_traits<HeapObject> {};
+template <>
 struct ref_traits<ClosureFeedbackCellArray> : public ref_traits<HeapObject> {};
 template <>
 struct ref_traits<NumberDictionary> : public ref_traits<HeapObject> {};
@@ -346,8 +363,11 @@ class V8_EXPORT_PRIVATE ObjectRef {
 #undef HEAP_AS_METHOD_DECL
 
   bool IsNull() const;
-  bool IsNullOrUndefined(JSHeapBroker* broker) const;
-  bool IsTheHole(JSHeapBroker* broker) const;
+  bool IsUndefined() const;
+  enum HoleType HoleType() const;
+  bool IsTheHole() const;
+  bool IsPropertyCellHole() const;
+  bool IsNullOrUndefined() const;
 
   base::Optional<bool> TryGetBooleanValue(JSHeapBroker* broker) const;
   Maybe<double> OddballToNumber(JSHeapBroker* broker) const;
@@ -413,17 +433,20 @@ class HeapObjectType {
   using Flags = base::Flags<Flag>;
 
   HeapObjectType(InstanceType instance_type, Flags flags,
-                 OddballType oddball_type)
+                 OddballType oddball_type, HoleType hole_type)
       : instance_type_(instance_type),
         oddball_type_(oddball_type),
+        hole_type_(hole_type),
         flags_(flags) {
     DCHECK_EQ(instance_type == ODDBALL_TYPE,
               oddball_type != OddballType::kNone);
   }
 
   OddballType oddball_type() const { return oddball_type_; }
+  HoleType hole_type() const { return hole_type_; }
   // For compatibility with MapRef.
   OddballType oddball_type(JSHeapBroker* broker) const { return oddball_type_; }
+  HoleType hole_type(JSHeapBroker* broker) const { return hole_type_; }
   InstanceType instance_type() const { return instance_type_; }
   Flags flags() const { return flags_; }
 
@@ -433,6 +456,7 @@ class HeapObjectType {
  private:
   InstanceType const instance_type_;
   OddballType const oddball_type_;
+  HoleType const hole_type_;
   Flags const flags_;
 };
 
@@ -512,9 +536,9 @@ class JSObjectRef : public JSReceiverRef {
   // The direct-read implementation of the above, extracted into a helper since
   // it's also called from compilation-dependency validation. This helper is
   // guaranteed to not create new Ref instances.
-  base::Optional<Object> GetOwnConstantElementFromHeap(
-      JSHeapBroker* broker, FixedArrayBase elements, ElementsKind elements_kind,
-      uint32_t index) const;
+  base::Optional<Tagged<Object>> GetOwnConstantElementFromHeap(
+      JSHeapBroker* broker, Tagged<FixedArrayBase> elements,
+      ElementsKind elements_kind, uint32_t index) const;
 
   // Return the value of the property identified by the field {index}
   // if {index} is known to be an own data property of the object.
@@ -745,7 +769,7 @@ class CallHandlerInfoRef : public HeapObjectRef {
 
   Handle<CallHandlerInfo> object() const;
 
-  Address callback() const;
+  Address callback(JSHeapBroker* broker) const;
   ObjectRef data(JSHeapBroker* broker) const;
 };
 
@@ -811,6 +835,7 @@ class V8_EXPORT_PRIVATE MapRef : public HeapObjectRef {
   bool is_undetectable() const;
   bool is_callable() const;
   bool has_indexed_interceptor() const;
+  int construction_counter() const;
   bool is_migration_target() const;
   bool supports_fast_array_iteration(JSHeapBroker* broker) const;
   bool supports_fast_array_resize(JSHeapBroker* broker) const;
@@ -832,7 +857,7 @@ class V8_EXPORT_PRIVATE MapRef : public HeapObjectRef {
 
   HeapObjectRef prototype(JSHeapBroker* broker) const;
 
-  bool HasOnlyStablePrototypesWithFastElements(
+  bool PrototypesElementsDoNotHaveAccessorsOrThrow(
       JSHeapBroker* broker, ZoneVector<MapRef>* prototype_maps);
 
   // Concerning the underlying instance_descriptors:
@@ -1003,7 +1028,6 @@ class ScopeInfoRef : public HeapObjectRef {
   V(FunctionKind, kind)                                         \
   V(LanguageMode, language_mode)                                \
   V(bool, native)                                               \
-  V(bool, HasBreakInfo)                                         \
   V(bool, HasBuiltinId)                                         \
   V(bool, construct_as_builtin)                                 \
   V(bool, HasBytecodeArray)                                     \
@@ -1025,6 +1049,7 @@ class V8_EXPORT_PRIVATE SharedFunctionInfoRef : public HeapObjectRef {
   int context_header_size() const;
   int context_parameters_start() const;
   BytecodeArrayRef GetBytecodeArray(JSHeapBroker* broker) const;
+  bool HasBreakInfo(JSHeapBroker* broker) const;
   SharedFunctionInfo::Inlineability GetInlineability(
       JSHeapBroker* broker) const;
   OptionalFunctionTemplateInfoRef function_template_info(
@@ -1065,6 +1090,7 @@ class StringRef : public NameRef {
   bool IsExternalString() const;
 
   bool IsContentAccessible() const;
+  bool IsOneByteRepresentation() const;
 
  private:
   // With concurrent inlining on, we currently support reading directly
