@@ -42,8 +42,8 @@ void SetContextId(Local<Context> context, int id) {
 int GetContextId(Local<Context> context) {
   auto v8_context = Utils::OpenHandle(*context);
   DCHECK_NO_SCRIPT_NO_EXCEPTION(v8_context->GetIsolate());
-  i::Object value = v8_context->debug_context_id();
-  return (value.IsSmi()) ? i::Smi::ToInt(value) : 0;
+  i::Tagged<i::Object> value = v8_context->debug_context_id();
+  return (IsSmi(value)) ? i::Smi::ToInt(value) : 0;
 }
 
 void SetInspector(Isolate* isolate, v8_inspector::V8Inspector* inspector) {
@@ -127,9 +127,9 @@ Local<String> GetDateDescription(Local<Date> date) {
   i::Handle<i::JSDate> jsdate = i::Handle<i::JSDate>::cast(receiver);
   i::Isolate* i_isolate = jsdate->GetIsolate();
   ENTER_V8_NO_SCRIPT_NO_EXCEPTION(i_isolate);
-  auto buffer =
-      i::ToDateString(jsdate->value().Number(), i_isolate->date_cache(),
-                      i::ToDateStringMode::kLocalDateAndTime);
+  auto buffer = i::ToDateString(i::Object::Number(jsdate->value()),
+                                i_isolate->date_cache(),
+                                i::ToDateStringMode::kLocalDateAndTime);
   return Utils::ToLocal(i_isolate->factory()
                             ->NewStringFromUtf8(base::VectorOf(buffer))
                             .ToHandleChecked());
@@ -139,19 +139,20 @@ Local<String> GetFunctionDescription(Local<Function> function) {
   auto receiver = Utils::OpenHandle(*function);
   auto i_isolate = receiver->GetIsolate();
   ENTER_V8_NO_SCRIPT_NO_EXCEPTION(i_isolate);
-  if (receiver->IsJSBoundFunction()) {
+  if (IsJSBoundFunction(*receiver)) {
     return Utils::ToLocal(i::JSBoundFunction::ToString(
         i::Handle<i::JSBoundFunction>::cast(receiver)));
   }
-  if (receiver->IsJSFunction()) {
+  if (IsJSFunction(*receiver)) {
     auto js_function = i::Handle<i::JSFunction>::cast(receiver);
 #if V8_ENABLE_WEBASSEMBLY
-    if (js_function->shared().HasWasmExportedFunctionData()) {
+    if (js_function->shared()->HasWasmExportedFunctionData()) {
       auto i_isolate = js_function->GetIsolate();
-      auto func_index =
-          js_function->shared().wasm_exported_function_data().function_index();
+      auto func_index = js_function->shared()
+                            ->wasm_exported_function_data()
+                            ->function_index();
       auto instance = i::handle(
-          js_function->shared().wasm_exported_function_data().instance(),
+          js_function->shared()->wasm_exported_function_data()->instance(),
           i_isolate);
       if (instance->module()->origin == i::wasm::kWasmOrigin) {
         // For asm.js functions, we can still print the source
@@ -295,7 +296,7 @@ bool GetPrivateMembers(Local<Context> context, Local<Object> object, int filter,
   // Estimate number of static private methods/accessors for classes.
   bool has_static_private_methods_or_accessors = false;
   if (include_methods_or_accessors) {
-    if (receiver->IsJSFunction()) {
+    if (IsJSFunction(*receiver)) {
       i::Handle<i::JSFunction> func(i::JSFunction::cast(*receiver), isolate);
       i::Handle<i::SharedFunctionInfo> shared(func->shared(), isolate);
       if (shared->is_class_constructor() &&
@@ -316,15 +317,15 @@ bool GetPrivateMembers(Local<Context> context, Local<Object> object, int filter,
   auto add_private_entry = [&](i::VariableMode mode, i::Handle<i::String> name,
                                i::Handle<i::Object> value) {
     DCHECK_IMPLIES(mode == i::VariableMode::kPrivateMethod,
-                   value->IsJSFunction());
+                   IsJSFunction(*value));
     DCHECK_IMPLIES(mode != i::VariableMode::kPrivateMethod,
-                   value->IsAccessorPair());
+                   IsAccessorPair(*value));
     names_out->push_back(Utils::ToLocal(name));
     values_out->push_back(Utils::ToLocal(value));
   };
   if (has_static_private_methods_or_accessors) {
     i::Handle<i::Context> recevier_context(
-        i::JSFunction::cast(*receiver).context(), isolate);
+        i::JSFunction::cast(*receiver)->context(), isolate);
     ForEachContextLocal(isolate, recevier_context, var_mode_filter,
                         static_filter, add_private_entry);
   }
@@ -338,14 +339,14 @@ bool GetPrivateMembers(Local<Context> context, Local<Object> object, int filter,
         isolate, value, i::Object::GetProperty(isolate, receiver, key), false);
     if (key->is_private_brand()) {
       if (include_methods_or_accessors) {
-        DCHECK(value->IsContext());
+        DCHECK(IsContext(*value));
         i::Handle<i::Context> value_context(i::Context::cast(*value), isolate);
         ForEachContextLocal(isolate, value_context, var_mode_filter,
                             instance_filter, add_private_entry);
       }
     } else if (include_fields) {  // Private fields
       i::Handle<i::String> name(
-          i::String::cast(i::Symbol::cast(*key).description()), isolate);
+          i::String::cast(i::Symbol::cast(*key)->description()), isolate);
       names_out->push_back(Utils::ToLocal(name));
       values_out->push_back(Utils::ToLocal(value));
     }
@@ -358,7 +359,7 @@ bool GetPrivateMembers(Local<Context> context, Local<Object> object, int filter,
 
 MaybeLocal<Context> GetCreationContext(Local<Object> value) {
   i::Handle<i::Object> val = Utils::OpenHandle(*value);
-  if (val->IsJSGlobalProxy()) {
+  if (IsJSGlobalProxy(*val)) {
     return MaybeLocal<Context>();
   }
   return value->GetCreationContext();
@@ -434,7 +435,7 @@ bool CanBreakProgram(Isolate* v8_isolate) {
 
 size_t ScriptSource::Length() const {
   i::Handle<i::HeapObject> source = Utils::OpenHandle(this);
-  if (source->IsString()) return i::Handle<i::String>::cast(source)->length();
+  if (IsString(*source)) return i::Handle<i::String>::cast(source)->length();
   return Size();
 }
 
@@ -446,23 +447,23 @@ size_t ScriptSource::Size() const {
   }
 #endif  // V8_ENABLE_WEBASSEMBLY
   i::Handle<i::HeapObject> source = Utils::OpenHandle(this);
-  if (!source->IsString()) return 0;
+  if (!IsString(*source)) return 0;
   i::Handle<i::String> string = i::Handle<i::String>::cast(source);
   return string->length() * (string->IsTwoByteRepresentation() ? 2 : 1);
 }
 
 MaybeLocal<String> ScriptSource::JavaScriptCode() const {
   i::Handle<i::HeapObject> source = Utils::OpenHandle(this);
-  if (!source->IsString()) return MaybeLocal<String>();
+  if (!IsString(*source)) return MaybeLocal<String>();
   return Utils::ToLocal(i::Handle<i::String>::cast(source));
 }
 
 #if V8_ENABLE_WEBASSEMBLY
 Maybe<MemorySpan<const uint8_t>> ScriptSource::WasmBytecode() const {
   i::Handle<i::HeapObject> source = Utils::OpenHandle(this);
-  if (!source->IsForeign()) return Nothing<MemorySpan<const uint8_t>>();
+  if (!IsForeign(*source)) return Nothing<MemorySpan<const uint8_t>>();
   base::Vector<const uint8_t> wire_bytes =
-      i::Managed<i::wasm::NativeModule>::cast(*source).raw()->wire_bytes();
+      i::Managed<i::wasm::NativeModule>::cast(*source)->raw()->wire_bytes();
   return Just(MemorySpan<const uint8_t>{wire_bytes.begin(), wire_bytes.size()});
 }
 #endif  // V8_ENABLE_WEBASSEMBLY
@@ -499,14 +500,14 @@ int Script::EndLine() const {
 #if V8_ENABLE_WEBASSEMBLY
   if (script->type() == i::Script::Type::kWasm) return 0;
 #endif  // V8_ENABLE_WEBASSEMBLY
-  if (!script->source().IsString()) {
+  if (!IsString(script->source())) {
     return script->line_offset();
   }
   i::Isolate* isolate = script->GetIsolate();
   i::HandleScope scope(isolate);
   i::Script::PositionInfo info;
-  i::Script::GetPositionInfo(script, i::String::cast(script->source()).length(),
-                             &info);
+  i::Script::GetPositionInfo(
+      script, i::String::cast(script->source())->length(), &info);
   return info.line;
 }
 
@@ -517,14 +518,14 @@ int Script::EndColumn() const {
     return script->wasm_native_module()->wire_bytes().length();
   }
 #endif  // V8_ENABLE_WEBASSEMBLY
-  if (!script->source().IsString()) {
+  if (!IsString(script->source())) {
     return script->column_offset();
   }
   i::Isolate* isolate = script->GetIsolate();
   i::HandleScope scope(isolate);
   i::Script::PositionInfo info;
-  i::Script::GetPositionInfo(script, i::String::cast(script->source()).length(),
-                             &info);
+  i::Script::GetPositionInfo(
+      script, i::String::cast(script->source())->length(), &info);
   return info.column;
 }
 
@@ -532,7 +533,7 @@ MaybeLocal<String> Script::Name() const {
   i::Handle<i::Script> script = Utils::OpenHandle(this);
   i::Isolate* isolate = script->GetIsolate();
   i::Handle<i::Object> value(script->name(), isolate);
-  if (!value->IsString()) return MaybeLocal<String>();
+  if (!IsString(*value)) return MaybeLocal<String>();
   return Utils::ToLocal(i::Handle<i::String>::cast(value));
 }
 
@@ -540,7 +541,7 @@ MaybeLocal<String> Script::SourceURL() const {
   i::Handle<i::Script> script = Utils::OpenHandle(this);
   i::Isolate* isolate = script->GetIsolate();
   i::Handle<i::PrimitiveHeapObject> value(script->source_url(), isolate);
-  if (!value->IsString()) return MaybeLocal<String>();
+  if (!IsString(*value)) return MaybeLocal<String>();
   return Utils::ToLocal(i::Handle<i::String>::cast(value));
 }
 
@@ -548,7 +549,7 @@ MaybeLocal<String> Script::SourceMappingURL() const {
   i::Handle<i::Script> script = Utils::OpenHandle(this);
   i::Isolate* isolate = script->GetIsolate();
   i::Handle<i::Object> value(script->source_mapping_url(), isolate);
-  if (!value->IsString()) return MaybeLocal<String>();
+  if (!IsString(*value)) return MaybeLocal<String>();
   return Utils::ToLocal(i::Handle<i::String>::cast(value));
 }
 
@@ -562,8 +563,8 @@ MaybeLocal<String> Script::GetSha256Hash() const {
 
 Maybe<int> Script::ContextId() const {
   i::Handle<i::Script> script = Utils::OpenHandle(this);
-  i::Object value = script->context_data();
-  if (value.IsSmi()) return Just(i::Smi::ToInt(value));
+  i::Tagged<i::Object> value = script->context_data();
+  if (IsSmi(value)) return Just(i::Smi::ToInt(value));
   return Nothing<int>();
 }
 
@@ -758,8 +759,9 @@ bool Script::SetInstrumentationBreakpoint(BreakpointId* id) const {
   }
 #endif  // V8_ENABLE_WEBASSEMBLY
   i::SharedFunctionInfo::ScriptIterator it(isolate, *script);
-  for (i::SharedFunctionInfo sfi = it.Next(); !sfi.is_null(); sfi = it.Next()) {
-    if (sfi.is_toplevel()) {
+  for (i::Tagged<i::SharedFunctionInfo> sfi = it.Next(); !sfi.is_null();
+       sfi = it.Next()) {
+    if (sfi->is_toplevel()) {
       return isolate->debug()->SetBreakpointForFunction(
           handle(sfi, isolate), isolate->factory()->empty_string(), id,
           internal::Debug::kInstrumentation);
@@ -885,6 +887,12 @@ void WasmScript::Disassemble(DisassemblyCollector* collector,
                        collector, function_body_offsets);
 }
 
+void Disassemble(base::Vector<const uint8_t> wire_bytes,
+                 DisassemblyCollector* collector,
+                 std::vector<int>* function_body_offsets) {
+  i::wasm::Disassemble(wire_bytes, collector, function_body_offsets);
+}
+
 uint32_t WasmScript::GetFunctionHash(int function_index) {
   i::DisallowGarbageCollection no_gc;
   i::Handle<i::Script> script = Utils::OpenHandle(this);
@@ -895,7 +903,7 @@ uint32_t WasmScript::GetFunctionHash(int function_index) {
   DCHECK_GT(module->functions.size(), function_index);
   const i::wasm::WasmFunction& func = module->functions[function_index];
   i::wasm::ModuleWireBytes wire_bytes(native_module->wire_bytes());
-  base::Vector<const i::byte> function_bytes =
+  base::Vector<const uint8_t> function_bytes =
       wire_bytes.GetFunctionBytes(&func);
   // TODO(herhut): Maybe also take module, name and signature into account.
   return i::StringHasher::HashSequentialString(function_bytes.begin(),
@@ -945,17 +953,17 @@ void GetLoadedScripts(Isolate* v8_isolate,
   {
     i::DisallowGarbageCollection no_gc;
     i::Script::Iterator iterator(isolate);
-    for (i::Script script = iterator.Next(); !script.is_null();
+    for (i::Tagged<i::Script> script = iterator.Next(); !script.is_null();
          script = iterator.Next()) {
 #if V8_ENABLE_WEBASSEMBLY
-      if (script.type() != i::Script::Type::kNormal &&
-          script.type() != i::Script::Type::kWasm) {
+      if (script->type() != i::Script::Type::kNormal &&
+          script->type() != i::Script::Type::kWasm) {
         continue;
       }
 #else
-      if (script.type() != i::Script::Type::kNormal) continue;
+      if (script->type() != i::Script::Type::kNormal) continue;
 #endif  // V8_ENABLE_WEBASSEMBLY
-      if (!script.HasValidSource()) continue;
+      if (!script->HasValidSource()) continue;
       i::HandleScope handle_scope(isolate);
       i::Handle<i::Script> script_handle(script, isolate);
       scripts.emplace_back(v8_isolate, ToApiHandle<Script>(script_handle));
@@ -1011,10 +1019,10 @@ void ResetBlackboxedStateCache(Isolate* v8_isolate, Local<Script> script) {
   i::DisallowGarbageCollection no_gc;
   i::SharedFunctionInfo::ScriptIterator iter(isolate,
                                              *Utils::OpenHandle(*script));
-  for (i::SharedFunctionInfo info = iter.Next(); !info.is_null();
+  for (i::Tagged<i::SharedFunctionInfo> info = iter.Next(); !info.is_null();
        info = iter.Next()) {
-    if (info.HasDebugInfo()) {
-      info.GetDebugInfo().set_computed_debug_is_blackboxed(false);
+    if (auto debug_info = isolate->debug()->TryGetDebugInfo(info)) {
+      debug_info->set_computed_debug_is_blackboxed(false);
     }
   }
 }
@@ -1023,27 +1031,28 @@ int EstimatedValueSize(Isolate* v8_isolate, Local<Value> value) {
   i::Isolate* isolate = reinterpret_cast<i::Isolate*>(v8_isolate);
   ENTER_V8_NO_SCRIPT_NO_EXCEPTION(isolate);
   i::Handle<i::Object> object = Utils::OpenHandle(*value);
-  if (object->IsSmi()) return i::kTaggedSize;
-  CHECK(object->IsHeapObject());
+  if (IsSmi(*object)) return i::kTaggedSize;
+  CHECK(IsHeapObject(*object));
   return i::Handle<i::HeapObject>::cast(object)->Size();
 }
 
 void AccessorPair::CheckCast(Value* that) {
   i::Handle<i::Object> obj = Utils::OpenHandle(that);
-  Utils::ApiCheck(obj->IsAccessorPair(), "v8::debug::AccessorPair::Cast",
+  Utils::ApiCheck(i::IsAccessorPair(*obj), "v8::debug::AccessorPair::Cast",
                   "Value is not a v8::debug::AccessorPair");
 }
 
 #if V8_ENABLE_WEBASSEMBLY
 void WasmValueObject::CheckCast(Value* that) {
   i::Handle<i::Object> obj = Utils::OpenHandle(that);
-  Utils::ApiCheck(obj->IsWasmValueObject(), "v8::debug::WasmValueObject::Cast",
+  Utils::ApiCheck(i::IsWasmValueObject(*obj),
+                  "v8::debug::WasmValueObject::Cast",
                   "Value is not a v8::debug::WasmValueObject");
 }
 
 bool WasmValueObject::IsWasmValueObject(Local<Value> that) {
   i::Handle<i::Object> obj = Utils::OpenHandle(*that);
-  return obj->IsWasmValueObject();
+  return i::IsWasmValueObject(*obj);
 }
 
 Local<String> WasmValueObject::type() const {
@@ -1074,8 +1083,8 @@ Local<Function> GetBuiltin(Isolate* v8_isolate, Builtin requested_builtin) {
           .set_map(isolate->strict_function_without_prototype_map())
           .Build();
 
-  fun->shared().set_internal_formal_parameter_count(i::JSParameterCount(0));
-  fun->shared().set_length(0);
+  fun->shared()->set_internal_formal_parameter_count(i::JSParameterCount(0));
+  fun->shared()->set_length(0);
   return Utils::ToLocal(handle_scope.CloseAndEscape(fun));
 }
 
@@ -1113,8 +1122,8 @@ v8::Local<v8::Message> CreateMessageFromException(
 
 MaybeLocal<Script> GeneratorObject::Script() {
   i::Handle<i::JSGeneratorObject> obj = Utils::OpenHandle(this);
-  i::Object maybe_script = obj->function().shared().script();
-  if (!maybe_script.IsScript()) return {};
+  i::Tagged<i::Object> maybe_script = obj->function()->shared()->script();
+  if (!IsScript(maybe_script)) return {};
   i::Handle<i::Script> script(i::Script::cast(maybe_script), obj->GetIsolate());
   return ToApiHandle<v8::debug::Script>(script);
 }
@@ -1127,13 +1136,13 @@ Local<Function> GeneratorObject::Function() {
 Location GeneratorObject::SuspendedLocation() {
   i::Handle<i::JSGeneratorObject> obj = Utils::OpenHandle(this);
   CHECK(obj->is_suspended());
-  i::Object maybe_script = obj->function().shared().script();
-  if (!maybe_script.IsScript()) return Location();
+  i::Tagged<i::Object> maybe_script = obj->function()->shared()->script();
+  if (!IsScript(maybe_script)) return Location();
   i::Isolate* isolate = obj->GetIsolate();
   i::Handle<i::Script> script(i::Script::cast(maybe_script), isolate);
   i::Script::PositionInfo info;
   i::SharedFunctionInfo::EnsureSourcePositionsAvailable(
-      isolate, i::handle(obj->function().shared(), isolate));
+      isolate, i::handle(obj->function()->shared(), isolate));
   i::Script::GetPositionInfo(script, obj->source_position(), &info);
   return Location(info.line, info.column);
 }
@@ -1216,8 +1225,7 @@ void GlobalLexicalScopeNames(v8::Local<v8::Context> v8_context,
   i::Handle<i::Context> context = Utils::OpenHandle(*v8_context);
   i::Isolate* isolate = context->GetIsolate();
   i::Handle<i::ScriptContextTable> table(
-      context->global_object().native_context().script_context_table(),
-      isolate);
+      context->native_context()->script_context_table(), isolate);
   for (int i = 0; i < table->used(kAcquireLoad); i++) {
     i::Handle<i::Context> script_context =
         i::ScriptContextTable::GetContext(isolate, table, i);
@@ -1244,7 +1252,7 @@ int64_t GetNextRandomInt64(v8::Isolate* v8_isolate) {
 
 int GetDebuggingId(v8::Local<v8::Function> function) {
   i::Handle<i::JSReceiver> callable = v8::Utils::OpenHandle(*function);
-  if (!callable->IsJSFunction()) return i::DebugInfo::kNoDebuggingId;
+  if (!IsJSFunction(*callable)) return i::DebugInfo::kNoDebuggingId;
   i::Handle<i::JSFunction> func = i::Handle<i::JSFunction>::cast(callable);
   int id = func->GetIsolate()->debug()->GetFunctionDebuggingId(func);
   DCHECK_NE(i::DebugInfo::kNoDebuggingId, id);
@@ -1254,7 +1262,7 @@ int GetDebuggingId(v8::Local<v8::Function> function) {
 bool SetFunctionBreakpoint(v8::Local<v8::Function> function,
                            v8::Local<v8::String> condition, BreakpointId* id) {
   i::Handle<i::JSReceiver> callable = Utils::OpenHandle(*function);
-  if (!callable->IsJSFunction()) return false;
+  if (!IsJSFunction(*callable)) return false;
   i::Handle<i::JSFunction> jsfunction =
       i::Handle<i::JSFunction>::cast(callable);
   i::Isolate* isolate = jsfunction->GetIsolate();
@@ -1347,11 +1355,11 @@ MaybeLocal<v8::Value> EphemeronTable::Get(v8::Isolate* isolate,
   i::Isolate* i_isolate = reinterpret_cast<i::Isolate*>(isolate);
   auto self = i::Handle<i::EphemeronHashTable>::cast(Utils::OpenHandle(this));
   i::Handle<i::Object> internal_key = Utils::OpenHandle(*key);
-  DCHECK(internal_key->IsJSReceiver());
+  DCHECK(IsJSReceiver(*internal_key));
 
   i::Handle<i::Object> value(self->Lookup(internal_key), i_isolate);
 
-  if (value->IsTheHole()) return {};
+  if (IsTheHole(*value)) return {};
   return Utils::ToLocal(value);
 }
 
@@ -1361,7 +1369,7 @@ Local<EphemeronTable> EphemeronTable::Set(v8::Isolate* isolate,
   auto self = i::Handle<i::EphemeronHashTable>::cast(Utils::OpenHandle(this));
   i::Handle<i::Object> internal_key = Utils::OpenHandle(*key);
   i::Handle<i::Object> internal_value = Utils::OpenHandle(*value);
-  DCHECK(internal_key->IsJSReceiver());
+  DCHECK(IsJSReceiver(*internal_key));
 
   i::Handle<i::EphemeronHashTable> result(
       i::EphemeronHashTable::Put(self, internal_key, internal_value));
@@ -1397,7 +1405,7 @@ Local<Value> AccessorPair::setter() {
 
 bool AccessorPair::IsAccessorPair(Local<Value> that) {
   i::Handle<i::Object> obj = Utils::OpenHandle(*that);
-  return obj->IsAccessorPair();
+  return i::IsAccessorPair(*obj);
 }
 
 MaybeLocal<Message> GetMessageFromPromise(Local<Promise> p) {
@@ -1408,7 +1416,7 @@ MaybeLocal<Message> GetMessageFromPromise(Local<Promise> p) {
   i::Handle<i::Object> maybeMessage =
       i::JSReceiver::GetDataProperty(isolate, promise, key);
 
-  if (!maybeMessage->IsJSMessageObject(isolate)) return MaybeLocal<Message>();
+  if (!IsJSMessageObject(*maybeMessage, isolate)) return MaybeLocal<Message>();
   return ToApiHandle<Message>(
       i::Handle<i::JSMessageObject>::cast(maybeMessage));
 }
@@ -1450,7 +1458,7 @@ Maybe<bool> DebugPropertyIterator::Advance() {
     return Nothing<bool>();
   }
   Local<v8::Context> context =
-      Utils::ToLocal(handle(isolate_->context(), isolate_));
+      Utils::ToLocal(handle(isolate_->context()->native_context(), isolate_));
   CallDepthScope<false> call_depth_scope(isolate_, context);
 
   if (!AdvanceInternal()) {
