@@ -42,8 +42,8 @@ class V8NameConverter : public disasm::NameConverter {
  public:
   explicit V8NameConverter(Isolate* isolate, CodeReference code = {})
       : isolate_(isolate), code_(code) {}
-  const char* NameOfAddress(byte* pc) const override;
-  const char* NameInCode(byte* addr) const override;
+  const char* NameOfAddress(uint8_t* pc) const override;
+  const char* NameInCode(uint8_t* addr) const override;
   const char* RootRelativeName(int offset) const override;
 
   const CodeReference& code() const { return code_; }
@@ -82,7 +82,7 @@ void V8NameConverter::InitExternalRefsCache() const {
   }
 }
 
-const char* V8NameConverter::NameOfAddress(byte* pc) const {
+const char* V8NameConverter::NameOfAddress(uint8_t* pc) const {
   if (!code_.is_null()) {
     const char* name =
         isolate_ ? isolate_->builtins()->Lookup(reinterpret_cast<Address>(pc))
@@ -115,7 +115,7 @@ const char* V8NameConverter::NameOfAddress(byte* pc) const {
   return disasm::NameConverter::NameOfAddress(pc);
 }
 
-const char* V8NameConverter::NameInCode(byte* addr) const {
+const char* V8NameConverter::NameInCode(uint8_t* addr) const {
   // The V8NameConverter is used for well known code, so we can "safely"
   // dereference pointers in generated code.
   return code_.is_null() ? "" : reinterpret_cast<const char*>(addr);
@@ -243,7 +243,7 @@ static void PrintRelocInfo(std::ostringstream& out, Isolate* isolate,
   } else if (RelocInfo::IsEmbeddedObjectMode(rmode)) {
     HeapStringAllocator allocator;
     StringStream accumulator(&allocator);
-    relocinfo->target_object(isolate).ShortPrint(&accumulator);
+    ShortPrint(relocinfo->target_object(isolate), &accumulator);
     std::unique_ptr<char[]> obj_name = accumulator.ToCString();
     const bool is_compressed = RelocInfo::IsCompressedEmbeddedObject(rmode);
     out << "    ;; " << (is_compressed ? "(compressed) " : "")
@@ -257,11 +257,11 @@ static void PrintRelocInfo(std::ostringstream& out, Isolate* isolate,
     out << "    ;; external reference (" << reference_name << ")";
   } else if (RelocInfo::IsCodeTargetMode(rmode)) {
     out << "    ;; code:";
-    Code code =
+    Tagged<Code> code =
         isolate->heap()->FindCodeForInnerPointer(relocinfo->target_address());
-    CodeKind kind = code.kind();
-    if (code.is_builtin()) {
-      out << " Builtin::" << Builtins::name(code.builtin_id());
+    CodeKind kind = code->kind();
+    if (code->is_builtin()) {
+      out << " Builtin::" << Builtins::name(code->builtin_id());
     } else {
       out << " " << CodeKindToString(kind);
     }
@@ -280,12 +280,12 @@ static void PrintRelocInfo(std::ostringstream& out, Isolate* isolate,
 
 static int DecodeIt(Isolate* isolate, ExternalReferenceEncoder* ref_encoder,
                     std::ostream& os, CodeReference code,
-                    const V8NameConverter& converter, byte* begin, byte* end,
-                    Address current_pc) {
+                    const V8NameConverter& converter, uint8_t* begin,
+                    uint8_t* end, Address current_pc, size_t range_limit) {
   CHECK(!code.is_null());
   v8::base::EmbeddedVector<char, 128> decode_buffer;
   std::ostringstream out;
-  byte* pc = begin;
+  uint8_t* pc = begin;
   disasm::Disassembler d(converter,
                          disasm::Disassembler::kContinueOnUnimplementedOpcode);
   RelocIterator rit(code);
@@ -294,7 +294,7 @@ static int DecodeIt(Isolate* isolate, ExternalReferenceEncoder* ref_encoder,
 
   while (pc < end) {
     // First decode instruction so that we know its length.
-    byte* prev_pc = pc;
+    uint8_t* prev_pc = pc;
     bool decoding_constant_pool = constants > 0;
     if (decoding_constant_pool) {
       SNPrintF(
@@ -315,18 +315,37 @@ static int DecodeIt(Isolate* isolate, ExternalReferenceEncoder* ref_encoder,
                  rit.rinfo()->pc() == reinterpret_cast<Address>(pc) &&
                  rit.rinfo()->rmode() == RelocInfo::INTERNAL_REFERENCE) {
         // A raw pointer embedded in code stream.
-        byte* ptr =
-            base::ReadUnalignedValue<byte*>(reinterpret_cast<Address>(pc));
+        uint8_t* ptr =
+            base::ReadUnalignedValue<uint8_t*>(reinterpret_cast<Address>(pc));
         SNPrintF(decode_buffer, "%08" V8PRIxPTR "       jump table entry %4zu",
                  reinterpret_cast<intptr_t>(ptr),
                  static_cast<size_t>(ptr - begin));
         pc += sizeof(ptr);
+#ifdef V8_TARGET_ARCH_X64
+      } else if (!rit.done() &&
+                 rit.rinfo()->pc() == reinterpret_cast<Address>(pc) &&
+                 rit.rinfo()->rmode() ==
+                     RelocInfo::RELATIVE_SWITCH_TABLE_ENTRY) {
+        int target_pc_offset = static_cast<int>(rit.rinfo()->data());
+        uint8_t* ptr = begin + target_pc_offset;
+        SNPrintF(decode_buffer, "%08" V8PRIxPTR "       jump table entry %4zx",
+                 reinterpret_cast<intptr_t>(ptr),
+                 static_cast<size_t>(target_pc_offset));
+        // We use emitl (4 bytes) for the value in the table.
+        pc += 4;
+#endif  // V8_TARGET_ARCH_X64
       } else {
         decode_buffer[0] = '\0';
         pc += d.InstructionDecode(decode_buffer, pc);
       }
     }
 
+    Address pc_address = reinterpret_cast<Address>(pc);
+    if (range_limit != 0) {
+      if (pc_address > current_pc + range_limit) break;
+      if (pc_address <= current_pc - range_limit) continue;
+    }
+
     // Collect RelocInfo for this instruction (prev_pc .. pc-1)
     std::vector<const char*> comments;
     std::vector<Address> pcs;
@@ -339,9 +358,13 @@ static int DecodeIt(Isolate* isolate, ExternalReferenceEncoder* ref_encoder,
       datas.push_back(rit.rinfo()->data());
       rit.next();
     }
-    while (cit.HasCurrent() &&
-           cit.GetPCOffset() < static_cast<Address>(pc - begin)) {
-      comments.push_back(cit.GetComment());
+    while (cit.HasCurrent()) {
+      Address cur = cit.GetPCOffset();
+      if (cur >= static_cast<Address>(pc - begin)) break;
+      if (range_limit == 0 ||
+          cur + range_limit > current_pc - reinterpret_cast<Address>(begin)) {
+        comments.push_back(cit.GetComment());
+      }
       cit.Next();
     }
 
@@ -417,16 +440,21 @@ static int DecodeIt(Isolate* isolate, ExternalReferenceEncoder* ref_encoder,
 
   // Emit comments following the last instruction (if any).
   while (cit.HasCurrent()) {
-    out << "                  " << cit.GetComment();
-    DumpBuffer(os, out);
+    Address cur = cit.GetPCOffset();
+    if (range_limit == 0 ||
+        cur + range_limit == current_pc - reinterpret_cast<Address>(begin)) {
+      out << "                  " << cit.GetComment();
+      DumpBuffer(os, out);
+    }
     cit.Next();
   }
 
   return static_cast<int>(pc - begin);
 }
 
-int Disassembler::Decode(Isolate* isolate, std::ostream& os, byte* begin,
-                         byte* end, CodeReference code, Address current_pc) {
+int Disassembler::Decode(Isolate* isolate, std::ostream& os, uint8_t* begin,
+                         uint8_t* end, CodeReference code, Address current_pc,
+                         size_t range_limit) {
   DCHECK_WITH_MSG(v8_flags.text_is_readable,
                   "Builtins disassembly requires a readable .text section");
   V8NameConverter v8NameConverter(isolate, code);
@@ -437,19 +465,20 @@ int Disassembler::Decode(Isolate* isolate, std::ostream& os, byte* begin,
     DisallowGarbageCollection no_alloc;
     ExternalReferenceEncoder ref_encoder(isolate);
     return DecodeIt(isolate, &ref_encoder, os, code, v8NameConverter, begin,
-                    end, current_pc);
+                    end, current_pc, range_limit);
   } else {
     // No isolate => isolate-independent code. Only V8 External references
     // available.
     return DecodeIt(nullptr, nullptr, os, code, v8NameConverter, begin, end,
-                    current_pc);
+                    current_pc, range_limit);
   }
 }
 
 #else  // ENABLE_DISASSEMBLER
 
-int Disassembler::Decode(Isolate* isolate, std::ostream& os, byte* begin,
-                         byte* end, CodeReference code, Address current_pc) {
+int Disassembler::Decode(Isolate* isolate, std::ostream& os, uint8_t* begin,
+                         uint8_t* end, CodeReference code, Address current_pc,
+                         size_t range_limit) {
   return 0;
 }
 
