@@ -4,6 +4,7 @@
 
 #include "src/objects/js-function.h"
 
+#include "src/base/optional.h"
 #include "src/baseline/baseline-batch-compiler.h"
 #include "src/codegen/compiler.h"
 #include "src/common/globals.h"
@@ -23,10 +24,10 @@ namespace v8 {
 namespace internal {
 
 CodeKinds JSFunction::GetAttachedCodeKinds() const {
-  const CodeKind kind = code().kind();
+  const CodeKind kind = code()->kind();
   if (!CodeKindIsJSFunction(kind)) return {};
   if (CodeKindIsOptimizedJSFunction(kind) &&
-      code().marked_for_deoptimization()) {
+      code()->marked_for_deoptimization()) {
     return {};
   }
   return CodeKindToCodeKindFlag(kind);
@@ -37,24 +38,24 @@ CodeKinds JSFunction::GetAvailableCodeKinds() const {
 
   if ((result & CodeKindFlag::INTERPRETED_FUNCTION) == 0) {
     // The SharedFunctionInfo could have attached bytecode.
-    if (shared().HasBytecodeArray()) {
+    if (shared()->HasBytecodeArray()) {
       result |= CodeKindFlag::INTERPRETED_FUNCTION;
     }
   }
 
   if ((result & CodeKindFlag::BASELINE) == 0) {
     // The SharedFunctionInfo could have attached baseline code.
-    if (shared().HasBaselineCode()) {
+    if (shared()->HasBaselineCode()) {
       result |= CodeKindFlag::BASELINE;
     }
   }
 
   // Check the optimized code cache.
-  if (has_feedback_vector() && feedback_vector().has_optimized_code() &&
-      !feedback_vector().optimized_code().marked_for_deoptimization()) {
-    Code code = feedback_vector().optimized_code();
-    DCHECK(CodeKindIsOptimizedJSFunction(code.kind()));
-    result |= CodeKindToCodeKindFlag(code.kind());
+  if (has_feedback_vector() && feedback_vector()->has_optimized_code() &&
+      !feedback_vector()->optimized_code()->marked_for_deoptimization()) {
+    Tagged<Code> code = feedback_vector()->optimized_code();
+    DCHECK(CodeKindIsOptimizedJSFunction(code->kind()));
+    result |= CodeKindToCodeKindFlag(code->kind());
   }
 
   DCHECK_EQ((result & ~kJSFunctionCodeKindsMask), 0);
@@ -121,8 +122,8 @@ base::Optional<CodeKind> JSFunction::GetActiveTier() const {
   // Asm/Wasm functions are currently not supported. For simplicity, this
   // includes invalid asm.js functions whose code hasn't yet been updated to
   // CompileLazy but is still the InstantiateAsmJs builtin.
-  if (shared().HasAsmWasmData() ||
-      code().builtin_id() == Builtin::kInstantiateAsmJs) {
+  if (shared()->HasAsmWasmData() ||
+      code()->builtin_id() == Builtin::kInstantiateAsmJs) {
     return {};
   }
 #endif  // V8_ENABLE_WEBASSEMBLY
@@ -137,11 +138,11 @@ base::Optional<CodeKind> JSFunction::GetActiveTier() const {
         highest_tier == CodeKind::INTERPRETED_FUNCTION);
 
   if (highest_tier == CodeKind::INTERPRETED_FUNCTION) {
-    CHECK(code().is_interpreter_trampoline_builtin() ||
-          (CodeKindIsOptimizedJSFunction(code().kind()) &&
-           code().marked_for_deoptimization()) ||
-          (code().builtin_id() == Builtin::kCompileLazy &&
-           shared().HasBytecodeArray() && !shared().HasBaselineCode()));
+    CHECK(code()->is_interpreter_trampoline_builtin() ||
+          (CodeKindIsOptimizedJSFunction(code()->kind()) &&
+           code()->marked_for_deoptimization()) ||
+          (code()->builtin_id() == Builtin::kCompileLazy &&
+           shared()->HasBytecodeArray() && !shared()->HasBaselineCode()));
   }
 #endif  // DEBUG
 
@@ -174,7 +175,7 @@ bool JSFunction::CanDiscardCompiled() const {
   //
   // Note that when the function has not yet been compiled we also return
   // false; that's fine, since nothing must be discarded in that case.
-  if (CodeKindIsOptimizedJSFunction(code().kind())) return true;
+  if (CodeKindIsOptimizedJSFunction(code()->kind())) return true;
   CodeKinds result = GetAvailableCodeKinds();
   return (result & kJSFunctionCodeKindsMask) != 0;
 }
@@ -205,22 +206,22 @@ void JSFunction::MarkForOptimization(Isolate* isolate, CodeKind target_kind,
   DCHECK(!is_compiled() || ActiveTierIsIgnition() || ActiveTierIsBaseline() ||
          ActiveTierIsMaglev());
   DCHECK(!ActiveTierIsTurbofan());
-  DCHECK(shared().HasBytecodeArray());
-  DCHECK(shared().allows_lazy_compilation() ||
-         !shared().optimization_disabled());
+  DCHECK(shared()->HasBytecodeArray());
+  DCHECK(shared()->allows_lazy_compilation() ||
+         !shared()->optimization_disabled());
 
   if (IsConcurrent(mode)) {
     if (IsInProgress(tiering_state())) {
       if (v8_flags.trace_concurrent_recompilation) {
         PrintF("  ** Not marking ");
-        ShortPrint();
+        ShortPrint(*this);
         PrintF(" -- already in optimization queue.\n");
       }
       return;
     }
     if (v8_flags.trace_concurrent_recompilation) {
       PrintF("  ** Marking ");
-      ShortPrint();
+      ShortPrint(*this);
       PrintF(" for concurrent %s recompilation.\n",
              CodeKindToString(target_kind));
     }
@@ -229,9 +230,10 @@ void JSFunction::MarkForOptimization(Isolate* isolate, CodeKind target_kind,
   set_tiering_state(TieringStateFor(target_kind, mode));
 }
 
-void JSFunction::SetInterruptBudget(Isolate* isolate) {
-  raw_feedback_cell().set_interrupt_budget(
-      TieringManager::InterruptBudgetFor(isolate, *this));
+void JSFunction::SetInterruptBudget(
+    Isolate* isolate, base::Optional<CodeKind> override_active_tier) {
+  raw_feedback_cell()->set_interrupt_budget(
+      TieringManager::InterruptBudgetFor(isolate, *this, override_active_tier));
 }
 
 // static
@@ -249,7 +251,7 @@ Maybe<bool> JSFunctionOrBoundFunctionOrWrappedFunction::CopyNameAndLength(
   LookupIterator length_lookup(isolate, target,
                                isolate->factory()->length_string(), target,
                                LookupIterator::OWN);
-  if (!target->IsJSFunction() ||
+  if (!IsJSFunction(*target) ||
       length_lookup.state() != LookupIterator::ACCESSOR ||
       !length_lookup.GetAccessors().is_identical_to(function_length_accessor)) {
     Handle<Object> length(Smi::zero(), isolate);
@@ -261,9 +263,9 @@ Maybe<bool> JSFunctionOrBoundFunctionOrWrappedFunction::CopyNameAndLength(
       ASSIGN_RETURN_ON_EXCEPTION_VALUE(isolate, target_length,
                                        Object::GetProperty(&length_lookup),
                                        Nothing<bool>());
-      if (target_length->IsNumber()) {
+      if (IsNumber(*target_length)) {
         length = isolate->factory()->NewNumber(std::max(
-            0.0, DoubleToInteger(target_length->Number()) - arg_count));
+            0.0, DoubleToInteger(Object::Number(*target_length)) - arg_count));
       }
     }
     LookupIterator it(isolate, function, isolate->factory()->length_string(),
@@ -284,7 +286,7 @@ Maybe<bool> JSFunctionOrBoundFunctionOrWrappedFunction::CopyNameAndLength(
       isolate->factory()->function_name_accessor();
   LookupIterator name_lookup(isolate, target, isolate->factory()->name_string(),
                              target);
-  if (!target->IsJSFunction() ||
+  if (!IsJSFunction(*target) ||
       name_lookup.state() != LookupIterator::ACCESSOR ||
       !name_lookup.GetAccessors().is_identical_to(function_name_accessor) ||
       (name_lookup.IsFound() && !name_lookup.HolderIsReceiver())) {
@@ -293,7 +295,7 @@ Maybe<bool> JSFunctionOrBoundFunctionOrWrappedFunction::CopyNameAndLength(
                                      Object::GetProperty(&name_lookup),
                                      Nothing<bool>());
     Handle<String> name;
-    if (target_name->IsString()) {
+    if (IsString(*target_name)) {
       ASSIGN_RETURN_ON_EXCEPTION_VALUE(
           isolate, name,
           Name::ToFunctionName(isolate, Handle<String>::cast(target_name)),
@@ -326,14 +328,14 @@ MaybeHandle<String> JSBoundFunction::GetName(Isolate* isolate,
   Handle<String> target_name = prefix;
   Factory* factory = isolate->factory();
   // Concatenate the "bound " up to the last non-bound target.
-  while (function->bound_target_function().IsJSBoundFunction()) {
+  while (IsJSBoundFunction(function->bound_target_function())) {
     ASSIGN_RETURN_ON_EXCEPTION(isolate, target_name,
                                factory->NewConsString(prefix, target_name),
                                String);
     function = handle(JSBoundFunction::cast(function->bound_target_function()),
                       isolate);
   }
-  if (function->bound_target_function().IsJSWrappedFunction()) {
+  if (IsJSWrappedFunction(function->bound_target_function())) {
     Handle<JSWrappedFunction> target(
         JSWrappedFunction::cast(function->bound_target_function()), isolate);
     Handle<String> name;
@@ -341,7 +343,7 @@ MaybeHandle<String> JSBoundFunction::GetName(Isolate* isolate,
         isolate, name, JSWrappedFunction::GetName(isolate, target), String);
     return factory->NewConsString(target_name, name);
   }
-  if (function->bound_target_function().IsJSFunction()) {
+  if (IsJSFunction(function->bound_target_function())) {
     Handle<JSFunction> target(
         JSFunction::cast(function->bound_target_function()), isolate);
     Handle<String> name = JSFunction::GetName(isolate, target);
@@ -354,21 +356,21 @@ MaybeHandle<String> JSBoundFunction::GetName(Isolate* isolate,
 // static
 Maybe<int> JSBoundFunction::GetLength(Isolate* isolate,
                                       Handle<JSBoundFunction> function) {
-  int nof_bound_arguments = function->bound_arguments().length();
-  while (function->bound_target_function().IsJSBoundFunction()) {
+  int nof_bound_arguments = function->bound_arguments()->length();
+  while (IsJSBoundFunction(function->bound_target_function())) {
     function = handle(JSBoundFunction::cast(function->bound_target_function()),
                       isolate);
     // Make sure we never overflow {nof_bound_arguments}, the number of
     // arguments of a function is strictly limited by the max length of an
     // JSAarray, Smi::kMaxValue is thus a reasonably good overestimate.
-    int length = function->bound_arguments().length();
+    int length = function->bound_arguments()->length();
     if (V8_LIKELY(Smi::kMaxValue - nof_bound_arguments > length)) {
       nof_bound_arguments += length;
     } else {
       nof_bound_arguments = Smi::kMaxValue;
     }
   }
-  if (function->bound_target_function().IsJSWrappedFunction()) {
+  if (IsJSWrappedFunction(function->bound_target_function())) {
     Handle<JSWrappedFunction> target(
         JSWrappedFunction::cast(function->bound_target_function()), isolate);
     int target_length = 0;
@@ -402,12 +404,12 @@ MaybeHandle<String> JSWrappedFunction::GetName(
   Handle<String> target_name = factory->empty_string();
   Handle<JSReceiver> target =
       handle(function->wrapped_target_function(), isolate);
-  if (target->IsJSBoundFunction()) {
+  if (IsJSBoundFunction(*target)) {
     return JSBoundFunction::GetName(
         isolate,
         handle(JSBoundFunction::cast(function->wrapped_target_function()),
                isolate));
-  } else if (target->IsJSFunction()) {
+  } else if (IsJSFunction(*target)) {
     return JSFunction::GetName(
         isolate,
         handle(JSFunction::cast(function->wrapped_target_function()), isolate));
@@ -422,7 +424,7 @@ Maybe<int> JSWrappedFunction::GetLength(Isolate* isolate,
   STACK_CHECK(isolate, Nothing<int>());
   Handle<JSReceiver> target =
       handle(function->wrapped_target_function(), isolate);
-  if (target->IsJSBoundFunction()) {
+  if (IsJSBoundFunction(*target)) {
     return JSBoundFunction::GetLength(
         isolate,
         handle(JSBoundFunction::cast(function->wrapped_target_function()),
@@ -444,11 +446,11 @@ MaybeHandle<Object> JSWrappedFunction::Create(
     Isolate* isolate, Handle<NativeContext> creation_context,
     Handle<JSReceiver> value) {
   // The value must be a callable according to the specification.
-  DCHECK(value->IsCallable());
+  DCHECK(IsCallable(*value));
   // The intermediate wrapped functions are not user-visible. And calling a
   // wrapped function won't cause a side effect in the creation realm.
   // Unwrap here to avoid nested unwrapping at the call site.
-  if (value->IsJSWrappedFunction()) {
+  if (IsJSWrappedFunction(*value)) {
     Handle<JSWrappedFunction> target_wrapped =
         Handle<JSWrappedFunction>::cast(value);
     value =
@@ -498,24 +500,24 @@ MaybeHandle<Object> JSWrappedFunction::Create(
 // static
 Handle<String> JSFunction::GetName(Isolate* isolate,
                                    Handle<JSFunction> function) {
-  if (function->shared().name_should_print_as_anonymous()) {
+  if (function->shared()->name_should_print_as_anonymous()) {
     return isolate->factory()->anonymous_string();
   }
-  return handle(function->shared().Name(), isolate);
+  return handle(function->shared()->Name(), isolate);
 }
 
 // static
 void JSFunction::EnsureClosureFeedbackCellArray(
     Handle<JSFunction> function, bool reset_budget_for_feedback_allocation) {
   Isolate* const isolate = function->GetIsolate();
-  DCHECK(function->shared().is_compiled());
-  DCHECK(function->shared().HasFeedbackMetadata());
+  DCHECK(function->shared()->is_compiled());
+  DCHECK(function->shared()->HasFeedbackMetadata());
 #if V8_ENABLE_WEBASSEMBLY
-  if (function->shared().HasAsmWasmData()) return;
+  if (function->shared()->HasAsmWasmData()) return;
 #endif  // V8_ENABLE_WEBASSEMBLY
 
   Handle<SharedFunctionInfo> shared(function->shared(), isolate);
-  DCHECK(function->shared().HasBytecodeArray());
+  DCHECK(function->shared()->HasBytecodeArray());
 
   const bool has_closure_feedback_cell_array =
       (function->has_closure_feedback_cell_array() ||
@@ -548,8 +550,8 @@ void JSFunction::EnsureClosureFeedbackCellArray(
     function->set_raw_feedback_cell(*feedback_cell, kReleaseStore);
     function->SetInterruptBudget(isolate);
   } else {
-    function->raw_feedback_cell().set_value(*feedback_cell_array,
-                                            kReleaseStore);
+    function->raw_feedback_cell()->set_value(*feedback_cell_array,
+                                             kReleaseStore);
   }
 }
 
@@ -558,10 +560,10 @@ void JSFunction::EnsureFeedbackVector(Isolate* isolate,
                                       Handle<JSFunction> function,
                                       IsCompiledScope* compiled_scope) {
   DCHECK(compiled_scope->is_compiled());
-  DCHECK(function->shared().HasFeedbackMetadata());
+  DCHECK(function->shared()->HasFeedbackMetadata());
   if (function->has_feedback_vector()) return;
 #if V8_ENABLE_WEBASSEMBLY
-  if (function->shared().HasAsmWasmData()) return;
+  if (function->shared()->HasAsmWasmData()) return;
 #endif  // V8_ENABLE_WEBASSEMBLY
 
   CreateAndAttachFeedbackVector(isolate, function, compiled_scope);
@@ -572,14 +574,14 @@ void JSFunction::CreateAndAttachFeedbackVector(
     Isolate* isolate, Handle<JSFunction> function,
     IsCompiledScope* compiled_scope) {
   DCHECK(compiled_scope->is_compiled());
-  DCHECK(function->shared().HasFeedbackMetadata());
+  DCHECK(function->shared()->HasFeedbackMetadata());
   DCHECK(!function->has_feedback_vector());
 #if V8_ENABLE_WEBASSEMBLY
-  DCHECK(!function->shared().HasAsmWasmData());
+  DCHECK(!function->shared()->HasAsmWasmData());
 #endif  // V8_ENABLE_WEBASSEMBLY
 
   Handle<SharedFunctionInfo> shared(function->shared(), isolate);
-  DCHECK(function->shared().HasBytecodeArray());
+  DCHECK(function->shared()->HasBytecodeArray());
 
   EnsureClosureFeedbackCellArray(function, false);
   Handle<ClosureFeedbackCellArray> closure_feedback_cell_array =
@@ -593,7 +595,7 @@ void JSFunction::CreateAndAttachFeedbackVector(
   // for more details.
   DCHECK(function->raw_feedback_cell() !=
          isolate->heap()->many_closures_cell());
-  DCHECK_EQ(function->raw_feedback_cell().value(), *feedback_vector);
+  DCHECK_EQ(function->raw_feedback_cell()->value(), *feedback_vector);
   function->SetInterruptBudget(isolate);
 
   DCHECK_EQ(v8_flags.log_function_events,
@@ -611,19 +613,19 @@ void JSFunction::InitializeFeedbackCell(
   // feedback vectors, so a mismatch between the metadata and feedback vector is
   // harmless. The checks could fail for functions that has has_asm_wasm_broken
   // set at runtime (for ex: failed instantiation).
-  if (function->shared().HasAsmWasmData()) return;
+  if (function->shared()->HasAsmWasmData()) return;
 #endif  // V8_ENABLE_WEBASSEMBLY
 
   if (function->has_feedback_vector()) {
-    CHECK_EQ(function->feedback_vector().length(),
-             function->feedback_vector().metadata().slot_count());
+    CHECK_EQ(function->feedback_vector()->length(),
+             function->feedback_vector()->metadata()->slot_count());
     return;
   }
 
   if (function->has_closure_feedback_cell_array()) {
     CHECK_EQ(
-        function->closure_feedback_cell_array().length(),
-        function->shared().feedback_metadata().create_closure_slot_count());
+        function->closure_feedback_cell_array()->length(),
+        function->shared()->feedback_metadata()->create_closure_slot_count());
   }
 
   const bool needs_feedback_vector =
@@ -632,7 +634,7 @@ void JSFunction::InitializeFeedbackCell(
       // profile and more precise code coverage.
       v8_flags.log_function_events ||
       !isolate->is_best_effort_code_coverage() ||
-      function->shared().sparkplug_compiled();
+      function->shared()->sparkplug_compiled();
 
   if (needs_feedback_vector) {
     CreateAndAttachFeedbackVector(isolate, function, is_compiled_scope);
@@ -641,14 +643,14 @@ void JSFunction::InitializeFeedbackCell(
                                    reset_budget_for_feedback_allocation);
   }
   // TODO(jgruber): Unduplicate these conditions from tiering-manager.cc.
-  if (function->shared().sparkplug_compiled() &&
+  if (function->shared()->sparkplug_compiled() &&
       CanCompileWithBaseline(isolate, function->shared()) &&
       function->ActiveTierIsIgnition()) {
     if (v8_flags.baseline_batch_compilation) {
       isolate->baseline_batch_compiler()->EnqueueFunction(function);
     } else {
       IsCompiledScope is_compiled_scope(
-          function->shared().is_compiled_scope(isolate));
+          function->shared()->is_compiled_scope(isolate));
       Compiler::CompileBaseline(isolate, function, Compiler::CLEAR_EXCEPTION,
                                 &is_compiled_scope);
     }
@@ -676,7 +678,7 @@ void SetInstancePrototype(Isolate* isolate, Handle<JSFunction> function,
       // At that point, a new initial map is created and the prototype is put
       // into the initial map where it belongs.
       function->set_prototype_or_initial_map(*value, kReleaseStore);
-      if (value->IsJSObjectThatCanBeTrackedAsPrototype()) {
+      if (IsJSObjectThatCanBeTrackedAsPrototype(*value)) {
         // Optimize as prototype to detach it from its transition tree.
         JSObject::OptimizeAsPrototype(Handle<JSObject>::cast(value));
       }
@@ -685,7 +687,7 @@ void SetInstancePrototype(Isolate* isolate, Handle<JSFunction> function,
           Map::Copy(isolate, initial_map, "SetInstancePrototype");
       JSFunction::SetInitialMap(isolate, function, new_map, value);
       DCHECK_IMPLIES(!isolate->bootstrapper()->IsActive(),
-                     *function != function->native_context().array_function());
+                     *function != function->native_context()->array_function());
     }
 
     // Deoptimize all code that embeds the previous initial map.
@@ -696,7 +698,7 @@ void SetInstancePrototype(Isolate* isolate, Handle<JSFunction> function,
     // needed.  At that point, a new initial map is created and the
     // prototype is put into the initial map where it belongs.
     function->set_prototype_or_initial_map(*value, kReleaseStore);
-    if (value->IsJSObjectThatCanBeTrackedAsPrototype()) {
+    if (IsJSObjectThatCanBeTrackedAsPrototype(*value)) {
       // Optimize as prototype to detach it from its transition tree.
       JSObject::OptimizeAsPrototype(Handle<JSObject>::cast(value));
     }
@@ -707,8 +709,8 @@ void SetInstancePrototype(Isolate* isolate, Handle<JSFunction> function,
 
 void JSFunction::SetPrototype(Handle<JSFunction> function,
                               Handle<Object> value) {
-  DCHECK(function->IsConstructor() ||
-         IsGeneratorFunction(function->shared().kind()));
+  DCHECK(IsConstructor(*function) ||
+         IsGeneratorFunction(function->shared()->kind()));
   Isolate* isolate = function->GetIsolate();
   Handle<JSReceiver> construct_prototype;
 
@@ -716,18 +718,25 @@ void JSFunction::SetPrototype(Handle<JSFunction> function,
   // constructor field so it can be accessed.  Also, set the prototype
   // used for constructing objects to the original object prototype.
   // See ECMA-262 13.2.2.
-  if (!value->IsJSReceiver()) {
+  if (!IsJSReceiver(*value)) {
     // Copy the map so this does not affect unrelated functions.
     // Remove map transitions because they point to maps with a
     // different prototype.
     Handle<Map> new_map =
         Map::Copy(isolate, handle(function->map(), isolate), "SetPrototype");
 
-    new_map->SetConstructor(*value);
+    // Create a new {constructor, non-instance_prototype} tuple and store it
+    // in Map::constructor field.
+    Handle<Object> constructor(new_map->GetConstructor(), isolate);
+    Handle<Tuple2> non_instance_prototype_constructor_tuple =
+        isolate->factory()->NewTuple2(constructor, value, AllocationType::kOld);
+
     new_map->set_has_non_instance_prototype(true);
+    new_map->SetConstructor(*non_instance_prototype_constructor_tuple);
+
     JSObject::MigrateToMap(isolate, function, new_map);
 
-    FunctionKind kind = function->shared().kind();
+    FunctionKind kind = function->shared()->kind();
     Handle<Context> native_context(function->native_context(), isolate);
 
     construct_prototype = Handle<JSReceiver>(
@@ -739,7 +748,7 @@ void JSFunction::SetPrototype(Handle<JSFunction> function,
         isolate);
   } else {
     construct_prototype = Handle<JSReceiver>::cast(value);
-    function->map().set_has_non_instance_prototype(false);
+    function->map()->set_has_non_instance_prototype(false);
   }
 
   SetInstancePrototype(isolate, function, construct_prototype);
@@ -752,11 +761,10 @@ void JSFunction::SetInitialMap(Isolate* isolate, Handle<JSFunction> function,
 
 void JSFunction::SetInitialMap(Isolate* isolate, Handle<JSFunction> function,
                                Handle<Map> map, Handle<HeapObject> prototype,
-                               Handle<HeapObject> constructor) {
+                               Handle<JSFunction> constructor) {
   if (map->prototype() != *prototype) {
     Map::SetPrototype(isolate, map, prototype);
   }
-  DCHECK_IMPLIES(!constructor->IsJSFunction(), map->InSharedHeap());
   map->SetConstructor(*constructor);
   function->set_prototype_or_initial_map(*map, kReleaseStore);
   if (v8_flags.log_maps) {
@@ -768,8 +776,8 @@ void JSFunction::SetInitialMap(Isolate* isolate, Handle<JSFunction> function,
 
 void JSFunction::EnsureHasInitialMap(Handle<JSFunction> function) {
   DCHECK(function->has_prototype_slot());
-  DCHECK(function->IsConstructor() ||
-         IsResumableFunction(function->shared().kind()));
+  DCHECK(IsConstructor(*function) ||
+         IsResumableFunction(function->shared()->kind()));
   if (function->has_initial_map()) return;
   Isolate* isolate = function->GetIsolate();
 
@@ -784,8 +792,8 @@ void JSFunction::EnsureHasInitialMap(Handle<JSFunction> function) {
   // Create a new map with the size and number of in-object properties suggested
   // by the function.
   InstanceType instance_type;
-  if (IsResumableFunction(function->shared().kind())) {
-    instance_type = IsAsyncGeneratorFunction(function->shared().kind())
+  if (IsResumableFunction(function->shared()->kind())) {
+    instance_type = IsAsyncGeneratorFunction(function->shared()->kind())
                         ? JS_ASYNC_GENERATOR_OBJECT_TYPE
                         : JS_GENERATOR_OBJECT_TYPE;
   } else {
@@ -813,7 +821,7 @@ void JSFunction::EnsureHasInitialMap(Handle<JSFunction> function) {
   DCHECK(map->has_fast_object_elements());
 
   // Finally link initial map and constructor function.
-  DCHECK(prototype->IsJSReceiver());
+  DCHECK(IsJSReceiver(*prototype));
   JSFunction::SetInitialMap(isolate, function, map, prototype);
   map->StartInobjectSlackTracking();
 }
@@ -962,8 +970,8 @@ bool FastInitializeDerivedMap(Isolate* isolate, Handle<JSFunction> new_target,
   // Check that |function|'s initial map still in sync with the |constructor|,
   // otherwise we must create a new initial map for |function|.
   if (new_target->has_initial_map() &&
-      new_target->initial_map().GetConstructor() == *constructor) {
-    DCHECK(new_target->instance_prototype().IsJSReceiver());
+      new_target->initial_map()->GetConstructor() == *constructor) {
+    DCHECK(IsJSReceiver(new_target->instance_prototype()));
     return true;
   }
   InstanceType instance_type = constructor_initial_map->instance_type();
@@ -973,7 +981,7 @@ bool FastInitializeDerivedMap(Isolate* isolate, Handle<JSFunction> new_target,
 
   // Link initial map and constructor function if the new.target is actually a
   // subclass constructor.
-  if (!IsDerivedConstructor(new_target->shared().kind())) return false;
+  if (!IsDerivedConstructor(new_target->shared()->kind())) return false;
 
   int instance_size;
   int in_object_properties;
@@ -986,7 +994,7 @@ bool FastInitializeDerivedMap(Isolate* isolate, Handle<JSFunction> new_target,
   // failure occur during prototype chain iteration.
   // So we take the maximum of two values.
   int expected_nof_properties = std::max(
-      static_cast<int>(constructor->shared().expected_nof_properties()),
+      static_cast<int>(constructor->shared()->expected_nof_properties()),
       JSFunction::CalculateExpectedNofProperties(isolate, new_target));
   JSFunction::CalculateInstanceSizeHelper(
       instance_type, constructor_initial_map->has_prototype_slot(),
@@ -1003,7 +1011,7 @@ bool FastInitializeDerivedMap(Isolate* isolate, Handle<JSFunction> new_target,
   map->set_new_target_is_base(false);
   Handle<HeapObject> prototype(new_target->instance_prototype(), isolate);
   JSFunction::SetInitialMap(isolate, new_target, map, prototype, constructor);
-  DCHECK(new_target->instance_prototype().IsJSReceiver());
+  DCHECK(IsJSReceiver(new_target->instance_prototype()));
   map->set_construction_counter(Map::kNoSlackTracking);
   map->StartInobjectSlackTracking();
   return true;
@@ -1024,7 +1032,7 @@ MaybeHandle<Map> JSFunction::GetDerivedMap(Isolate* isolate,
   // Fast case, new.target is a subclass of constructor. The map is cacheable
   // (and may already have been cached). new.target.prototype is guaranteed to
   // be a JSReceiver.
-  if (new_target->IsJSFunction()) {
+  if (IsJSFunction(*new_target)) {
     Handle<JSFunction> function = Handle<JSFunction>::cast(new_target);
     if (FastInitializeDerivedMap(isolate, function, constructor,
                                  constructor_initial_map)) {
@@ -1036,7 +1044,7 @@ MaybeHandle<Map> JSFunction::GetDerivedMap(Isolate* isolate,
   // new.target.prototype is not guaranteed to be a JSReceiver, and may need to
   // fall back to the intrinsicDefaultProto.
   Handle<Object> prototype;
-  if (new_target->IsJSFunction()) {
+  if (IsJSFunction(*new_target)) {
     Handle<JSFunction> function = Handle<JSFunction>::cast(new_target);
     if (function->has_prototype_slot()) {
       // Make sure the new.target.prototype is cached.
@@ -1061,16 +1069,16 @@ MaybeHandle<Map> JSFunction::GetDerivedMap(Isolate* isolate,
   // correct realm. Rather than directly fetching the .prototype, we fetch the
   // constructor that points to the .prototype. This relies on
   // constructor.prototype being FROZEN for those constructors.
-  if (!prototype->IsJSReceiver()) {
+  if (!IsJSReceiver(*prototype)) {
     Handle<Context> context;
     ASSIGN_RETURN_ON_EXCEPTION(isolate, context,
                                JSReceiver::GetFunctionRealm(new_target), Map);
-    DCHECK(context->IsNativeContext());
+    DCHECK(IsNativeContext(*context));
     Handle<Object> maybe_index = JSReceiver::GetDataProperty(
         isolate, constructor,
         isolate->factory()->native_context_index_symbol());
-    int index = maybe_index->IsSmi() ? Smi::ToInt(*maybe_index)
-                                     : Context::OBJECT_FUNCTION_INDEX;
+    int index = IsSmi(*maybe_index) ? Smi::ToInt(*maybe_index)
+                                    : Context::OBJECT_FUNCTION_INDEX;
     Handle<JSFunction> realm_constructor(JSFunction::cast(context->get(index)),
                                          isolate);
     prototype = handle(realm_constructor->prototype(), isolate);
@@ -1078,7 +1086,7 @@ MaybeHandle<Map> JSFunction::GetDerivedMap(Isolate* isolate,
 
   Handle<Map> map = Map::CopyInitialMap(isolate, constructor_initial_map);
   map->set_new_target_is_base(false);
-  CHECK(prototype->IsJSReceiver());
+  CHECK(IsJSReceiver(*prototype));
   if (map->prototype() != *prototype)
     Map::SetPrototype(isolate, map, Handle<HeapObject>::cast(prototype));
   map->SetConstructor(*constructor);
@@ -1124,13 +1132,13 @@ MaybeHandle<Map> JSFunction::GetDerivedRabGsabTypedArrayMap(
   }
   {
     DisallowHeapAllocation no_alloc;
-    NativeContext context = isolate->context().native_context();
+    Tagged<NativeContext> context = isolate->context()->native_context();
     int ctor_index =
         TypedArrayElementsKindToConstructorIndex(map->elements_kind());
-    if (*new_target == context.get(ctor_index)) {
+    if (*new_target == context->get(ctor_index)) {
       ctor_index =
           TypedArrayElementsKindToRabGsabCtorIndex(map->elements_kind());
-      return handle(Map::cast(context.get(ctor_index)), isolate);
+      return handle(Map::cast(context->get(ctor_index)), isolate);
     }
   }
 
@@ -1146,7 +1154,7 @@ MaybeHandle<Map> JSFunction::GetDerivedRabGsabTypedArrayMap(
 MaybeHandle<Map> JSFunction::GetDerivedRabGsabDataViewMap(
     Isolate* isolate, Handle<JSReceiver> new_target) {
   Handle<Context> context =
-      handle(isolate->context().native_context(), isolate);
+      handle(isolate->context()->native_context(), isolate);
   Handle<JSFunction> constructor = handle(context->data_view_fun(), isolate);
   MaybeHandle<Map> maybe_map = GetDerivedMap(isolate, constructor, new_target);
   Handle<Map> map;
@@ -1167,15 +1175,15 @@ MaybeHandle<Map> JSFunction::GetDerivedRabGsabDataViewMap(
 
 int JSFunction::ComputeInstanceSizeWithMinSlack(Isolate* isolate) {
   CHECK(has_initial_map());
-  if (initial_map().IsInobjectSlackTrackingInProgress()) {
-    int slack = initial_map().ComputeMinObjectSlack(isolate);
-    return initial_map().InstanceSizeFromSlack(slack);
+  if (initial_map()->IsInobjectSlackTrackingInProgress()) {
+    int slack = initial_map()->ComputeMinObjectSlack(isolate);
+    return initial_map()->InstanceSizeFromSlack(slack);
   }
-  return initial_map().instance_size();
+  return initial_map()->instance_size();
 }
 
 std::unique_ptr<char[]> JSFunction::DebugNameCStr() {
-  return shared().DebugNameCStr();
+  return shared()->DebugNameCStr();
 }
 
 void JSFunction::PrintName(FILE* out) {
@@ -1185,22 +1193,22 @@ void JSFunction::PrintName(FILE* out) {
 namespace {
 
 bool UseFastFunctionNameLookup(Isolate* isolate, Map map) {
-  DCHECK(map.IsJSFunctionMap());
-  if (map.NumberOfOwnDescriptors() <
+  DCHECK(IsJSFunctionMap(map));
+  if (map->NumberOfOwnDescriptors() <
       JSFunction::kMinDescriptorsForFastBindAndWrap) {
     return false;
   }
-  DCHECK(!map.is_dictionary_map());
-  HeapObject value;
+  DCHECK(!map->is_dictionary_map());
+  Tagged<HeapObject> value;
   ReadOnlyRoots roots(isolate);
-  auto descriptors = map.instance_descriptors(isolate);
+  auto descriptors = map->instance_descriptors(isolate);
   InternalIndex kNameIndex{JSFunction::kNameDescriptorIndex};
-  if (descriptors.GetKey(kNameIndex) != roots.name_string() ||
-      !descriptors.GetValue(kNameIndex)
+  if (descriptors->GetKey(kNameIndex) != roots.name_string() ||
+      !descriptors->GetValue(kNameIndex)
            .GetHeapObjectIfStrong(isolate, &value)) {
     return false;
   }
-  return value.IsAccessorInfo();
+  return IsAccessorInfo(value);
 }
 
 }  // namespace
@@ -1223,7 +1231,7 @@ Handle<String> JSFunction::GetDebugName(Handle<JSFunction> function) {
     // in case of the fast-path.
     Handle<Object> name =
         GetDataProperty(isolate, function, isolate->factory()->name_string());
-    if (name->IsString()) return Handle<String>::cast(name);
+    if (IsString(*name)) return Handle<String>::cast(name);
   }
   return SharedFunctionInfo::DebugName(isolate,
                                        handle(function->shared(), isolate));
@@ -1279,13 +1287,13 @@ Handle<String> JSFunction::ToString(Handle<JSFunction> function) {
     // Check if we should print {function} as a class.
     Handle<Object> maybe_class_positions = JSReceiver::GetDataProperty(
         isolate, function, isolate->factory()->class_positions_symbol());
-    if (maybe_class_positions->IsClassPositions()) {
+    if (IsClassPositions(*maybe_class_positions)) {
       ClassPositions class_positions =
           ClassPositions::cast(*maybe_class_positions);
-      int start_position = class_positions.start();
-      int end_position = class_positions.end();
+      int start_position = class_positions->start();
+      int end_position = class_positions->end();
       Handle<String> script_source(
-          String::cast(Script::cast(shared_info->script()).source()), isolate);
+          String::cast(Script::cast(shared_info->script())->source()), isolate);
       return isolate->factory()->NewSubString(script_source, start_position,
                                               end_position);
     }
@@ -1302,13 +1310,13 @@ Handle<String> JSFunction::ToString(Handle<JSFunction> function) {
   if (shared_info->HasWasmExportedFunctionData()) {
     Handle<WasmExportedFunctionData> function_data(
         shared_info->wasm_exported_function_data(), isolate);
-    const wasm::WasmModule* module = function_data->instance().module();
+    const wasm::WasmModule* module = function_data->instance()->module();
     if (is_asmjs_module(module)) {
       std::pair<int, int> offsets =
           module->asm_js_offset_information->GetFunctionOffsets(
               declared_function_index(module, function_data->function_index()));
       Handle<String> source(
-          String::cast(Script::cast(shared_info->script()).source()), isolate);
+          String::cast(Script::cast(shared_info->script())->source()), isolate);
       return isolate->factory()->NewSubString(source, offsets.first,
                                               offsets.second);
     }
@@ -1335,7 +1343,7 @@ int JSFunction::CalculateExpectedNofProperties(Isolate* isolate,
        !iter.IsAtEnd(); iter.Advance()) {
     Handle<JSReceiver> current =
         PrototypeIterator::GetCurrent<JSReceiver>(iter);
-    if (!current->IsJSFunction()) break;
+    if (!IsJSFunction(*current)) break;
     Handle<JSFunction> func = Handle<JSFunction>::cast(current);
     // The super constructor should be compiled for the number of expected
     // properties to be available.
@@ -1405,7 +1413,7 @@ void JSFunction::ClearAllTypeFeedbackInfoForTesting() {
   if (has_feedback_vector()) {
     FeedbackVector vector = feedback_vector();
     Isolate* isolate = GetIsolate();
-    if (vector.ClearAllSlotsForTesting(isolate)) {
+    if (vector->ClearAllSlotsForTesting(isolate)) {
       IC::OnFeedbackChanged(isolate, vector, FeedbackSlot::Invalid(),
                             "ClearAllTypeFeedbackInfoForTesting");
     }
