@@ -31,10 +31,10 @@
 namespace v8 {
 namespace internal {
 
-AlignedCachedData::AlignedCachedData(const byte* data, int length)
+AlignedCachedData::AlignedCachedData(const uint8_t* data, int length)
     : owns_data_(false), rejected_(false), data_(data), length_(length) {
   if (!IsAligned(reinterpret_cast<intptr_t>(data), kPointerAlignment)) {
-    byte* copy = NewArray<byte>(length);
+    uint8_t* copy = NewArray<uint8_t>(length);
     DCHECK(IsAligned(reinterpret_cast<intptr_t>(copy), kPointerAlignment));
     CopyBytes(copy, data, length);
     data_ = copy;
@@ -60,7 +60,7 @@ ScriptCompiler::CachedData* CodeSerializer::Serialize(
   Handle<Script> script(Script::cast(info->script()), isolate);
   if (v8_flags.trace_serializer) {
     PrintF("[Serializing from");
-    script->name().ShortPrint();
+    ShortPrint(script->name());
     PrintF("]\n");
   }
 #if V8_ENABLE_WEBASSEMBLY
@@ -113,20 +113,14 @@ void CodeSerializer::SerializeObjectImpl(Handle<HeapObject> obj,
   InstanceType instance_type;
   {
     DisallowGarbageCollection no_gc;
-    HeapObject raw = *obj;
+    Tagged<HeapObject> raw = *obj;
     if (SerializeHotObject(raw)) return;
     if (SerializeRoot(raw)) return;
     if (SerializeBackReference(raw)) return;
     if (SerializeReadOnlyObjectReference(raw, &sink_)) return;
 
-    instance_type = raw.map().instance_type();
+    instance_type = raw->map()->instance_type();
     CHECK(!InstanceTypeChecker::IsInstructionStream(instance_type));
-
-    if (ElideObject(raw)) {
-      AllowGarbageCollection allow_gc;
-      return SerializeObject(roots.undefined_value_handle(),
-                             SlotType::kAnySlot);
-    }
   }
 
   if (InstanceTypeChecker::IsScript(instance_type)) {
@@ -134,29 +128,29 @@ void CodeSerializer::SerializeObjectImpl(Handle<HeapObject> obj,
     Handle<Object> context_data;
     {
       DisallowGarbageCollection no_gc;
-      Script script_obj = Script::cast(*obj);
-      DCHECK_NE(script_obj.compilation_type(), Script::CompilationType::kEval);
+      Tagged<Script> script_obj = Script::cast(*obj);
+      DCHECK_NE(script_obj->compilation_type(), Script::CompilationType::kEval);
       // We want to differentiate between undefined and uninitialized_symbol for
       // context_data for now. It is hack to allow debugging for scripts that
       // are included as a part of custom snapshot. (see
       // debug::Script::IsEmbedded())
-      Object raw_context_data = script_obj.context_data();
+      Tagged<Object> raw_context_data = script_obj->context_data();
       if (raw_context_data != roots.undefined_value() &&
           raw_context_data != roots.uninitialized_symbol()) {
-        script_obj.set_context_data(roots.undefined_value());
+        script_obj->set_context_data(roots.undefined_value());
       }
       context_data = handle(raw_context_data, isolate());
       // We don't want to serialize host options to avoid serializing
       // unnecessary object graph.
-      host_options = handle(script_obj.host_defined_options(), isolate());
-      script_obj.set_host_defined_options(roots.empty_fixed_array());
+      host_options = handle(script_obj->host_defined_options(), isolate());
+      script_obj->set_host_defined_options(roots.empty_fixed_array());
     }
     SerializeGeneric(obj, slot_type);
     {
       DisallowGarbageCollection no_gc;
-      Script script_obj = Script::cast(*obj);
-      script_obj.set_host_defined_options(*host_options);
-      script_obj.set_context_data(*context_data);
+      Tagged<Script> script_obj = Script::cast(*obj);
+      script_obj->set_host_defined_options(*host_options);
+      script_obj->set_context_data(*context_data);
     }
     return;
   } else if (InstanceTypeChecker::IsSharedFunctionInfo(instance_type)) {
@@ -164,35 +158,28 @@ void CodeSerializer::SerializeObjectImpl(Handle<HeapObject> obj,
     bool restore_bytecode = false;
     {
       DisallowGarbageCollection no_gc;
-      SharedFunctionInfo sfi = SharedFunctionInfo::cast(*obj);
-      DCHECK(!sfi.IsApiFunction());
+      Tagged<SharedFunctionInfo> sfi = SharedFunctionInfo::cast(*obj);
+      DCHECK(!sfi->IsApiFunction());
 #if V8_ENABLE_WEBASSEMBLY
       // TODO(7110): Enable serializing of Asm modules once the AsmWasmData
       // is context independent.
-      DCHECK(!sfi.HasAsmWasmData());
+      DCHECK(!sfi->HasAsmWasmData());
 #endif  // V8_ENABLE_WEBASSEMBLY
 
-      if (sfi.HasDebugInfo()) {
+      if (auto maybe_debug_info = sfi->TryGetDebugInfo(isolate())) {
+        debug_info = handle(maybe_debug_info.value(), isolate());
         // Clear debug info.
-        DebugInfo raw_debug_info = sfi.GetDebugInfo();
-        if (raw_debug_info.HasInstrumentedBytecodeArray()) {
+        if (debug_info->HasInstrumentedBytecodeArray()) {
           restore_bytecode = true;
-          sfi.SetActiveBytecodeArray(raw_debug_info.OriginalBytecodeArray());
+          sfi->SetActiveBytecodeArray(debug_info->OriginalBytecodeArray());
         }
-        sfi.set_script_or_debug_info(raw_debug_info.script(), kReleaseStore);
-        debug_info = handle(raw_debug_info, isolate());
       }
-      DCHECK(!sfi.HasDebugInfo());
     }
     SerializeGeneric(obj, slot_type);
-    // Restore debug info
-    if (!debug_info.is_null()) {
+    if (restore_bytecode) {
       DisallowGarbageCollection no_gc;
-      SharedFunctionInfo sfi = SharedFunctionInfo::cast(*obj);
-      sfi.set_script_or_debug_info(*debug_info, kReleaseStore);
-      if (restore_bytecode) {
-        sfi.SetActiveBytecodeArray(debug_info->DebugBytecodeArray());
-      }
+      Tagged<SharedFunctionInfo> sfi = SharedFunctionInfo::cast(*obj);
+      sfi->SetActiveBytecodeArray(debug_info->DebugBytecodeArray());
     }
     return;
   } else if (InstanceTypeChecker::IsUncompiledDataWithoutPreparseDataWithJob(
@@ -221,8 +208,8 @@ void CodeSerializer::SerializeObjectImpl(Handle<HeapObject> obj,
   // information. On deserialization we'll create our code objects again, if
   // --interpreted-frames-native-stack is on. See v8:9122 for more context
   if (V8_UNLIKELY(v8_flags.interpreted_frames_native_stack) &&
-      obj->IsInterpreterData()) {
-    obj = handle(InterpreterData::cast(*obj).bytecode_array(), isolate());
+      IsInterpreterData(*obj)) {
+    obj = handle(InterpreterData::cast(*obj)->bytecode_array(), isolate());
   }
 
   // Past this point we should not see any (context-specific) maps anymore.
@@ -261,16 +248,16 @@ void CreateInterpreterDataForDeserializedCode(
   Handle<Script> script(Script::cast(result_sfi->script()), isolate);
   if (log_code_creation) Script::InitLineEnds(isolate, script);
 
-  String name = ReadOnlyRoots(isolate).empty_string();
-  if (script->name().IsString()) name = String::cast(script->name());
+  Tagged<String> name = ReadOnlyRoots(isolate).empty_string();
+  if (IsString(script->name())) name = String::cast(script->name());
   Handle<String> name_handle(name, isolate);
 
   SharedFunctionInfo::ScriptIterator iter(isolate, *script);
-  for (SharedFunctionInfo shared_info = iter.Next(); !shared_info.is_null();
-       shared_info = iter.Next()) {
+  for (Tagged<SharedFunctionInfo> shared_info = iter.Next();
+       !shared_info.is_null(); shared_info = iter.Next()) {
     IsCompiledScope is_compiled(shared_info, isolate);
     if (!is_compiled.is_compiled()) continue;
-    DCHECK(shared_info.HasBytecodeArray());
+    DCHECK(shared_info->HasBytecodeArray());
     Handle<SharedFunctionInfo> sfi = handle(shared_info, isolate);
 
     Handle<Code> code =
@@ -284,7 +271,7 @@ void CreateInterpreterDataForDeserializedCode(
     interpreter_data->set_interpreter_trampoline(*code);
     if (sfi->HasBaselineCode()) {
       sfi->baseline_code(kAcquireLoad)
-          .set_bytecode_or_interpreter_data(*interpreter_data);
+          ->set_bytecode_or_interpreter_data(*interpreter_data);
     } else {
       sfi->set_interpreter_data(*interpreter_data);
     }
@@ -337,6 +324,11 @@ class StressOffThreadDeserializeThread final : public base::Thread {
 void FinalizeDeserialization(Isolate* isolate,
                              Handle<SharedFunctionInfo> result,
                              const base::ElapsedTimer& timer) {
+  // Devtools can report time in this function as profiler overhead, since none
+  // of the following tasks would need to happen normally.
+  TRACE_EVENT0(TRACE_DISABLED_BY_DEFAULT("v8.compile"),
+               "V8.FinalizeDeserialization");
+
   const bool log_code_creation = isolate->IsLoggingCodeCreation();
 
   if (V8_UNLIKELY(v8_flags.interpreted_frames_native_stack)) {
@@ -352,8 +344,8 @@ void FinalizeDeserialization(Isolate* isolate,
     Script::InitLineEnds(isolate, script);
   }
 
-  Handle<String> name(script->name().IsString()
-                          ? String::cast(script->name())
+  Handle<String> name(IsString(script->name())
+                          ? Tagged<String>::cast(script->name())
                           : ReadOnlyRoots(isolate).empty_string(),
                       isolate);
 
@@ -365,9 +357,9 @@ void FinalizeDeserialization(Isolate* isolate,
   }
 
   SharedFunctionInfo::ScriptIterator iter(isolate, *script);
-  for (SharedFunctionInfo info = iter.Next(); !info.is_null();
+  for (Tagged<SharedFunctionInfo> info = iter.Next(); !info.is_null();
        info = iter.Next()) {
-    if (!info.is_compiled()) continue;
+    if (!info->is_compiled()) continue;
     Handle<SharedFunctionInfo> shared_info(info, isolate);
     if (needs_source_positions) {
       SharedFunctionInfo::EnsureSourcePositionsAvailable(isolate, shared_info);
@@ -385,21 +377,44 @@ void FinalizeDeserialization(Isolate* isolate,
   }
 }
 
-void BaselineBatchCompileIfSparkplugCompiled(Isolate* isolate, Script script) {
+void BaselineBatchCompileIfSparkplugCompiled(Isolate* isolate,
+                                             Tagged<Script> script) {
   // Here is main thread, we trigger early baseline compilation only in
   // concurrent sparkplug and baseline batch compilation mode which consumes
   // little main thread execution time.
   if (v8_flags.concurrent_sparkplug && v8_flags.baseline_batch_compilation) {
     SharedFunctionInfo::ScriptIterator iter(isolate, script);
-    for (SharedFunctionInfo info = iter.Next(); !info.is_null();
+    for (Tagged<SharedFunctionInfo> info = iter.Next(); !info.is_null();
          info = iter.Next()) {
-      if (info.sparkplug_compiled() && CanCompileWithBaseline(isolate, info)) {
+      if (info->sparkplug_compiled() && CanCompileWithBaseline(isolate, info)) {
         isolate->baseline_batch_compiler()->EnqueueSFI(info);
       }
     }
   }
 }
 
+const char* ToString(SerializedCodeSanityCheckResult result) {
+  switch (result) {
+    case SerializedCodeSanityCheckResult::kSuccess:
+      return "success";
+    case SerializedCodeSanityCheckResult::kMagicNumberMismatch:
+      return "magic number mismatch";
+    case SerializedCodeSanityCheckResult::kVersionMismatch:
+      return "version mismatch";
+    case SerializedCodeSanityCheckResult::kSourceMismatch:
+      return "source mismatch";
+    case SerializedCodeSanityCheckResult::kFlagsMismatch:
+      return "flags mismatch";
+    case SerializedCodeSanityCheckResult::kChecksumMismatch:
+      return "checksum mismatch";
+    case SerializedCodeSanityCheckResult::kInvalidHeader:
+      return "invalid header";
+    case SerializedCodeSanityCheckResult::kLengthMismatch:
+      return "length mismatch";
+    case SerializedCodeSanityCheckResult::kReadOnlySnapshotChecksumMismatch:
+      return "read-only snapshot checksum mismatch";
+  }
+}
 }  // namespace
 
 MaybeHandle<SharedFunctionInfo> CodeSerializer::Deserialize(
@@ -415,19 +430,22 @@ MaybeHandle<SharedFunctionInfo> CodeSerializer::Deserialize(
   }
 
   base::ElapsedTimer timer;
-  if (v8_flags.profile_deserialization || v8_flags.log_function_events)
+  if (v8_flags.profile_deserialization || v8_flags.log_function_events) {
     timer.Start();
+  }
 
   HandleScope scope(isolate);
 
   SerializedCodeSanityCheckResult sanity_check_result =
       SerializedCodeSanityCheckResult::kSuccess;
   const SerializedCodeData scd = SerializedCodeData::FromCachedData(
-      cached_data, SerializedCodeData::SourceHash(source, origin_options),
+      isolate, cached_data,
+      SerializedCodeData::SourceHash(source, origin_options),
       &sanity_check_result);
   if (sanity_check_result != SerializedCodeSanityCheckResult::kSuccess) {
-    if (v8_flags.profile_deserialization)
-      PrintF("[Cached code failed check]\n");
+    if (v8_flags.profile_deserialization) {
+      PrintF("[Cached code failed check: %s]\n", ToString(sanity_check_result));
+    }
     DCHECK(cached_data->rejected());
     isolate->counters()->code_cache_reject_reason()->AddSample(
         static_cast<int>(sanity_check_result));
@@ -501,7 +519,7 @@ CodeSerializer::StartDeserializeOffThread(LocalIsolate* local_isolate,
 
   const SerializedCodeData scd =
       SerializedCodeData::FromCachedDataWithoutSource(
-          cached_data, &result.sanity_check_result);
+          local_isolate, cached_data, &result.sanity_check_result);
   if (result.sanity_check_result != SerializedCodeSanityCheckResult::kSuccess) {
     // Exit early but don't report yet, we'll re-check this when finishing on
     // the main thread
@@ -526,8 +544,9 @@ MaybeHandle<SharedFunctionInfo> CodeSerializer::FinishOffThreadDeserialize(
     ScriptOriginOptions origin_options,
     BackgroundMergeTask* background_merge_task) {
   base::ElapsedTimer timer;
-  if (v8_flags.profile_deserialization || v8_flags.log_function_events)
+  if (v8_flags.profile_deserialization || v8_flags.log_function_events) {
     timer.Start();
+  }
 
   HandleScope scope(isolate);
 
@@ -552,8 +571,9 @@ MaybeHandle<SharedFunctionInfo> CodeSerializer::FinishOffThreadDeserialize(
     DCHECK_IMPLIES(sanity_check_result != data.sanity_check_result,
                    sanity_check_result ==
                        SerializedCodeSanityCheckResult::kSourceMismatch);
-    if (v8_flags.profile_deserialization)
-      PrintF("[Cached code failed check]\n");
+    if (v8_flags.profile_deserialization) {
+      PrintF("[Cached code failed check: %s]\n", ToString(sanity_check_result));
+    }
     DCHECK(cached_data->rejected());
     isolate->counters()->code_cache_reject_reason()->AddSample(
         static_cast<int>(sanity_check_result));
@@ -577,7 +597,8 @@ MaybeHandle<SharedFunctionInfo> CodeSerializer::FinishOffThreadDeserialize(
       background_merge_task->HasPendingForegroundWork()) {
     Handle<Script> script = handle(Script::cast(result->script()), isolate);
     result = background_merge_task->CompleteMergeInForeground(isolate, script);
-    DCHECK(Script::cast(result->script()).source().StrictEquals(*source));
+    DCHECK(Object::StrictEquals(Script::cast(result->script())->source(),
+                                *source));
     DCHECK(isolate->factory()->script_list()->Contains(
         MaybeObject::MakeWeak(MaybeObject::FromObject(result->script()))));
   } else {
@@ -616,7 +637,7 @@ MaybeHandle<SharedFunctionInfo> CodeSerializer::FinishOffThreadDeserialize(
   return scope.CloseAndEscape(result);
 }
 
-SerializedCodeData::SerializedCodeData(const std::vector<byte>* payload,
+SerializedCodeData::SerializedCodeData(const std::vector<uint8_t>* payload,
                                        const CodeSerializer* cs) {
   DisallowGarbageCollection no_gc;
 
@@ -635,6 +656,9 @@ SerializedCodeData::SerializedCodeData(const std::vector<byte>* payload,
   SetHeaderValue(kVersionHashOffset, Version::Hash());
   SetHeaderValue(kSourceHashOffset, cs->source_hash());
   SetHeaderValue(kFlagHashOffset, FlagList::Hash());
+  SetHeaderValue(kReadOnlySnapshotChecksumOffset,
+                 Snapshot::ExtractReadOnlySnapshotChecksum(
+                     cs->isolate()->snapshot_blob()));
   SetHeaderValue(kPayloadLengthOffset, static_cast<uint32_t>(payload->size()));
 
   // Zero out any padding in the header.
@@ -649,8 +673,10 @@ SerializedCodeData::SerializedCodeData(const std::vector<byte>* payload,
 }
 
 SerializedCodeSanityCheckResult SerializedCodeData::SanityCheck(
+    uint32_t expected_ro_snapshot_checksum,
     uint32_t expected_source_hash) const {
-  SerializedCodeSanityCheckResult result = SanityCheckWithoutSource();
+  SerializedCodeSanityCheckResult result =
+      SanityCheckWithoutSource(expected_ro_snapshot_checksum);
   if (result != SerializedCodeSanityCheckResult::kSuccess) return result;
   return SanityCheckJustSource(expected_source_hash);
 }
@@ -664,9 +690,9 @@ SerializedCodeSanityCheckResult SerializedCodeData::SanityCheckJustSource(
   return SerializedCodeSanityCheckResult::kSuccess;
 }
 
-SerializedCodeSanityCheckResult SerializedCodeData::SanityCheckWithoutSource()
-    const {
-  if (this->size_ < kHeaderSize) {
+SerializedCodeSanityCheckResult SerializedCodeData::SanityCheckWithoutSource(
+    uint32_t expected_ro_snapshot_checksum) const {
+  if (size_ < kHeaderSize) {
     return SerializedCodeSanityCheckResult::kInvalidHeader;
   }
   uint32_t magic_number = GetMagicNumber();
@@ -681,8 +707,13 @@ SerializedCodeSanityCheckResult SerializedCodeData::SanityCheckWithoutSource()
   if (flags_hash != FlagList::Hash()) {
     return SerializedCodeSanityCheckResult::kFlagsMismatch;
   }
+  uint32_t ro_snapshot_checksum =
+      GetHeaderValue(kReadOnlySnapshotChecksumOffset);
+  if (ro_snapshot_checksum != expected_ro_snapshot_checksum) {
+    return SerializedCodeSanityCheckResult::kReadOnlySnapshotChecksumMismatch;
+  }
   uint32_t payload_length = GetHeaderValue(kPayloadLengthOffset);
-  uint32_t max_payload_length = this->size_ - kHeaderSize;
+  uint32_t max_payload_length = size_ - kHeaderSize;
   if (payload_length > max_payload_length) {
     return SerializedCodeSanityCheckResult::kLengthMismatch;
   }
@@ -716,23 +747,26 @@ AlignedCachedData* SerializedCodeData::GetScriptData() {
   return result;
 }
 
-base::Vector<const byte> SerializedCodeData::Payload() const {
-  const byte* payload = data_ + kHeaderSize;
+base::Vector<const uint8_t> SerializedCodeData::Payload() const {
+  const uint8_t* payload = data_ + kHeaderSize;
   DCHECK(IsAligned(reinterpret_cast<intptr_t>(payload), kPointerAlignment));
   int length = GetHeaderValue(kPayloadLengthOffset);
   DCHECK_EQ(data_ + size_, payload + length);
-  return base::Vector<const byte>(payload, length);
+  return base::Vector<const uint8_t>(payload, length);
 }
 
 SerializedCodeData::SerializedCodeData(AlignedCachedData* data)
-    : SerializedData(const_cast<byte*>(data->data()), data->length()) {}
+    : SerializedData(const_cast<uint8_t*>(data->data()), data->length()) {}
 
 SerializedCodeData SerializedCodeData::FromCachedData(
-    AlignedCachedData* cached_data, uint32_t expected_source_hash,
+    Isolate* isolate, AlignedCachedData* cached_data,
+    uint32_t expected_source_hash,
     SerializedCodeSanityCheckResult* rejection_result) {
   DisallowGarbageCollection no_gc;
   SerializedCodeData scd(cached_data);
-  *rejection_result = scd.SanityCheck(expected_source_hash);
+  *rejection_result = scd.SanityCheck(
+      Snapshot::ExtractReadOnlySnapshotChecksum(isolate->snapshot_blob()),
+      expected_source_hash);
   if (*rejection_result != SerializedCodeSanityCheckResult::kSuccess) {
     cached_data->Reject();
     return SerializedCodeData(nullptr, 0);
@@ -741,11 +775,13 @@ SerializedCodeData SerializedCodeData::FromCachedData(
 }
 
 SerializedCodeData SerializedCodeData::FromCachedDataWithoutSource(
-    AlignedCachedData* cached_data,
+    LocalIsolate* local_isolate, AlignedCachedData* cached_data,
     SerializedCodeSanityCheckResult* rejection_result) {
   DisallowGarbageCollection no_gc;
   SerializedCodeData scd(cached_data);
-  *rejection_result = scd.SanityCheckWithoutSource();
+  *rejection_result =
+      scd.SanityCheckWithoutSource(Snapshot::ExtractReadOnlySnapshotChecksum(
+          local_isolate->snapshot_blob()));
   if (*rejection_result != SerializedCodeSanityCheckResult::kSuccess) {
     cached_data->Reject();
     return SerializedCodeData(nullptr, 0);
