@@ -54,6 +54,7 @@ namespace internal {
 const auto GetRegConfig = RegisterConfiguration::Default;
 
 class HandleScope;
+class ManualGCScope;
 class Zone;
 
 namespace compiler {
@@ -170,12 +171,6 @@ class CcTest {
 
   static void AddGlobalFunction(v8::Local<v8::Context> env, const char* name,
                                 v8::FunctionCallback callback);
-  static void CollectGarbage(i::AllocationSpace space,
-                             i::Isolate* isolate = nullptr);
-  static void CollectAllGarbage(i::Isolate* isolate = nullptr);
-  static void CollectAllAvailableGarbage(i::Isolate* isolate = nullptr);
-  static void PreciseCollectAllGarbage(i::Isolate* isolate = nullptr);
-  static void CollectSharedGarbage(i::Isolate* isolate = nullptr);
 
   static i::Handle<i::String> MakeString(const char* str);
   static i::Handle<i::String> MakeName(const char* str, int suffix);
@@ -225,7 +220,7 @@ class CcTest {
   TestPlatformFactory* test_platform_factory_;
 
   friend int main(int argc, char** argv);
-  friend class ManualGCScope;
+  friend class v8::internal::ManualGCScope;
 };
 
 // Switches between all the Api tests using the threading support.
@@ -240,6 +235,8 @@ class CcTest {
 // to that thread, suspending the current test.
 class ApiTestFuzzer: public v8::base::Thread {
  public:
+  ~ApiTestFuzzer() override = default;
+
   void CallTest();
 
   // The ApiTestFuzzer is also a Thread, so it has a Run method.
@@ -270,19 +267,22 @@ class ApiTestFuzzer: public v8::base::Thread {
         test_number_(num),
         gate_(0),
         active_(true) {}
-  ~ApiTestFuzzer() override = default;
 
-  static bool fuzzing_;
-  static int tests_being_run_;
-  static int current_;
-  static int active_tests_;
   static bool NextThread();
+  void ContextSwitch();
+  static int GetNextFuzzer();
+
+  static unsigned linear_congruential_generator;
+  static std::vector<std::unique_ptr<ApiTestFuzzer>> fuzzers_;
+  static bool fuzzing_;
+  static v8::base::Semaphore all_tests_done_;
+  static int tests_being_run_;
+  static int active_tests_;
+  static int current_fuzzer_;
+
   int test_number_;
   v8::base::Semaphore gate_;
   bool active_;
-  void ContextSwitch();
-  static int GetNextTestNumber();
-  static v8::base::Semaphore all_tests_done_;
 };
 
 
@@ -295,30 +295,23 @@ class RegisterThreadedTest {
  public:
   explicit RegisterThreadedTest(CcTest::TestFunction* callback,
                                 const char* name)
-      : fuzzer_(nullptr), callback_(callback), name_(name) {
-    prev_ = first_;
-    first_ = this;
-    count_++;
+      : callback_(callback), name_(name) {
+    tests_.push_back(this);
   }
-  static int count() { return count_; }
-  static RegisterThreadedTest* nth(int i) {
-    CHECK(i < count());
-    RegisterThreadedTest* current = first_;
-    while (i > 0) {
-      i--;
-      current = current->prev_;
-    }
-    return current;
+  static int count() { return static_cast<int>(tests_.size()); }
+  static const RegisterThreadedTest* nth(int i) {
+    DCHECK_LE(0, i);
+    DCHECK_LT(i, count());
+    // Added tests used to be prepended to a linked list and therefore the last
+    // one to be added was at index 0. This ensures that we keep this behavior.
+    return tests_[count() - i - 1];
   }
-  CcTest::TestFunction* callback() { return callback_; }
-  ApiTestFuzzer* fuzzer_;
-  const char* name() { return name_; }
+  CcTest::TestFunction* callback() const { return callback_; }
+  const char* name() const { return name_; }
 
  private:
-  static RegisterThreadedTest* first_;
-  static int count_;
+  static std::vector<const RegisterThreadedTest*> tests_;
   CcTest::TestFunction* callback_;
-  RegisterThreadedTest* prev_;
   const char* name_;
 };
 
@@ -367,6 +360,15 @@ static inline uint16_t* AsciiToTwoByteString(const char* source) {
   return converted;
 }
 
+static inline uint16_t* AsciiToTwoByteString(const char16_t* source,
+                                             size_t* length_out = nullptr) {
+  size_t array_length = std::char_traits<char16_t>::length(source) + 1;
+  uint16_t* converted = i::NewArray<uint16_t>(array_length);
+  for (size_t i = 0; i < array_length; i++) converted[i] = source[i];
+  if (length_out != nullptr) *length_out = array_length - 1;
+  return converted;
+}
+
 template <typename T>
 static inline i::Handle<T> GetGlobal(const char* name) {
   i::Isolate* isolate = CcTest::i_isolate();
@@ -574,7 +576,6 @@ static inline void ExpectString(const char* code, const char* expected) {
   CHECK_EQ(0, strcmp(expected, *utf8));
 }
 
-
 static inline void ExpectInt32(const char* code, int expected) {
   v8::Local<v8::Value> result = CompileRun(code);
   CHECK(result->IsInt32());
@@ -685,28 +686,6 @@ class StaticOneByteResource : public v8::String::ExternalOneByteStringResource {
   const char* data_;
 };
 
-// ManualGCScope allows for disabling GC heuristics. This is useful for tests
-// that want to check specific corner cases around GC.
-//
-// The scope will finalize any ongoing GC on the provided Isolate. If no Isolate
-// is manually provided, it is assumed that a CcTest setup (e.g.
-// CcTest::InitializeVM()) is used.
-class V8_NODISCARD ManualGCScope {
- public:
-  explicit ManualGCScope(
-      i::Isolate* isolate = reinterpret_cast<i::Isolate*>(CcTest::isolate_));
-  ~ManualGCScope();
-
- private:
-  const bool flag_concurrent_marking_;
-  const bool flag_concurrent_sweeping_;
-  const bool flag_concurrent_minor_mc_marking_;
-  const bool flag_stress_concurrent_allocation_;
-  const bool flag_stress_incremental_marking_;
-  const bool flag_parallel_marking_;
-  const bool flag_detect_ineffective_gcs_near_heap_limit_;
-};
-
 // This is a base class that can be overridden to implement a test platform. It
 // delegates all operations to the default platform.
 class TestPlatform : public v8::Platform {
