@@ -14,6 +14,7 @@
 #include "src/codegen/macro-assembler.h"
 #include "src/codegen/reloc-info-inl.h"
 #include "src/common/globals.h"
+#include "src/handles/global-handles-inl.h"
 #include "src/handles/handles-inl.h"
 #include "src/handles/handles.h"
 #include "src/handles/local-handles-inl.h"
@@ -34,13 +35,13 @@ namespace internal {
 
 namespace {
 void CreateFixedArray(Heap* heap, Address start, int size) {
-  HeapObject object = HeapObject::FromAddress(start);
-  object.set_map_after_allocation(ReadOnlyRoots(heap).fixed_array_map(),
-                                  SKIP_WRITE_BARRIER);
-  FixedArray array = FixedArray::cast(object);
+  Tagged<HeapObject> object = HeapObject::FromAddress(start);
+  object->set_map_after_allocation(ReadOnlyRoots(heap).fixed_array_map(),
+                                   SKIP_WRITE_BARRIER);
+  Tagged<FixedArray> array = FixedArray::cast(object);
   int length = (size - FixedArray::kHeaderSize) / kTaggedSize;
-  array.set_length(length);
-  MemsetTagged(array.data_start(), ReadOnlyRoots(heap).undefined_value(),
+  array->set_length(length);
+  MemsetTagged(array->data_start(), ReadOnlyRoots(heap).undefined_value(),
                length);
 }
 
@@ -50,14 +51,20 @@ const int kMediumObjectSize = 8 * KB;
 
 void AllocateSomeObjects(LocalHeap* local_heap) {
   for (int i = 0; i < kNumIterations; i++) {
-    Address address = local_heap->AllocateRawOrFail(
+    AllocationResult result = local_heap->AllocateRaw(
         kSmallObjectSize, AllocationType::kOld, AllocationOrigin::kRuntime,
         AllocationAlignment::kTaggedAligned);
-    CreateFixedArray(local_heap->heap(), address, kSmallObjectSize);
-    address = local_heap->AllocateRawOrFail(
-        kMediumObjectSize, AllocationType::kOld, AllocationOrigin::kRuntime,
-        AllocationAlignment::kTaggedAligned);
-    CreateFixedArray(local_heap->heap(), address, kMediumObjectSize);
+    if (!result.IsFailure()) {
+      CreateFixedArray(local_heap->heap(), result.ToAddress(),
+                       kSmallObjectSize);
+    }
+    result = local_heap->AllocateRaw(kMediumObjectSize, AllocationType::kOld,
+                                     AllocationOrigin::kRuntime,
+                                     AllocationAlignment::kTaggedAligned);
+    if (!result.IsFailure()) {
+      CreateFixedArray(local_heap->heap(), result.ToAddress(),
+                       kMediumObjectSize);
+    }
     if (i % 10 == 0) {
       local_heap->Safepoint();
     }
@@ -74,7 +81,6 @@ class ConcurrentAllocationThread final : public v8::base::Thread {
         pending_(pending) {}
 
   void Run() override {
-    RwxMemoryWriteScope::SetDefaultPermissionsForNewThread();
     LocalHeap local_heap(heap_, ThreadKind::kBackground);
     UnparkedScope unparked_scope(&local_heap);
     AllocateSomeObjects(&local_heap);
@@ -388,7 +394,7 @@ UNINITIALIZED_TEST(ConcurrentBlackAllocation) {
   CHECK(thread->Start());
 
   sema_white.Wait();
-  heap->StartIncrementalMarking(i::Heap::kNoGCFlags,
+  heap->StartIncrementalMarking(i::GCFlag::kNoFlags,
                                 i::GarbageCollectionReason::kTesting);
   sema_marking_started.Signal();
 
@@ -398,7 +404,7 @@ UNINITIALIZED_TEST(ConcurrentBlackAllocation) {
 
   for (int i = 0; i < kNumIterations * kObjectsAllocatedPerIteration; i++) {
     Address address = objects[i];
-    HeapObject object = HeapObject::FromAddress(address);
+    Tagged<HeapObject> object = HeapObject::FromAddress(address);
 
     if (i < kWhiteIterations * kObjectsAllocatedPerIteration) {
       CHECK(heap->marking_state()->IsUnmarked(object));
@@ -412,8 +418,8 @@ UNINITIALIZED_TEST(ConcurrentBlackAllocation) {
 
 class ConcurrentWriteBarrierThread final : public v8::base::Thread {
  public:
-  explicit ConcurrentWriteBarrierThread(Heap* heap, FixedArray fixed_array,
-                                        HeapObject value)
+  ConcurrentWriteBarrierThread(Heap* heap, Tagged<FixedArray> fixed_array,
+                               Tagged<HeapObject> value)
       : v8::base::Thread(base::Thread::Options("ThreadWithLocalHeap")),
         heap_(heap),
         fixed_array_(fixed_array),
@@ -422,7 +428,7 @@ class ConcurrentWriteBarrierThread final : public v8::base::Thread {
   void Run() override {
     LocalHeap local_heap(heap_, ThreadKind::kBackground);
     UnparkedScope unparked_scope(&local_heap);
-    fixed_array_.set(0, value_);
+    fixed_array_->set(0, value_);
   }
 
   Heap* heap_;
@@ -444,8 +450,8 @@ UNINITIALIZED_TEST(ConcurrentWriteBarrier) {
   Isolate* i_isolate = reinterpret_cast<Isolate*>(isolate);
   Heap* heap = i_isolate->heap();
 
-  FixedArray fixed_array;
-  HeapObject value;
+  Tagged<FixedArray> fixed_array;
+  Tagged<HeapObject> value;
   {
     HandleScope handle_scope(i_isolate);
     Handle<FixedArray> fixed_array_handle(
@@ -455,10 +461,13 @@ UNINITIALIZED_TEST(ConcurrentWriteBarrier) {
     fixed_array = *fixed_array_handle;
     value = *value_handle;
   }
-  heap->StartIncrementalMarking(i::Heap::kNoGCFlags,
+  heap->StartIncrementalMarking(i::GCFlag::kNoFlags,
                                 i::GarbageCollectionReason::kTesting);
   CHECK(heap->marking_state()->IsUnmarked(value));
 
+  // Mark host |fixed_array| to trigger the barrier.
+  heap->marking_state()->TryMarkAndAccountLiveBytes(fixed_array);
+
   auto thread =
       std::make_unique<ConcurrentWriteBarrierThread>(heap, fixed_array, value);
   CHECK(thread->Start());
@@ -466,29 +475,29 @@ UNINITIALIZED_TEST(ConcurrentWriteBarrier) {
   thread->Join();
 
   CHECK(heap->marking_state()->IsMarked(value));
-  heap::InvokeMarkSweep(i_isolate);
+  heap::InvokeMajorGC(heap);
 
   isolate->Dispose();
 }
 
 class ConcurrentRecordRelocSlotThread final : public v8::base::Thread {
  public:
-  explicit ConcurrentRecordRelocSlotThread(Heap* heap, Code code,
-                                           HeapObject value)
+  ConcurrentRecordRelocSlotThread(Heap* heap, Tagged<Code> code,
+                                  Tagged<HeapObject> value)
       : v8::base::Thread(base::Thread::Options("ThreadWithLocalHeap")),
         heap_(heap),
         code_(code),
         value_(value) {}
 
   void Run() override {
-    RwxMemoryWriteScope::SetDefaultPermissionsForNewThread();
     LocalHeap local_heap(heap_, ThreadKind::kBackground);
     UnparkedScope unparked_scope(&local_heap);
     // Modification of InstructionStream object requires write access.
     RwxMemoryWriteScopeForTesting rwx_write_scope;
     DisallowGarbageCollection no_gc;
-    InstructionStream istream = code_.instruction_stream();
+    Tagged<InstructionStream> istream = code_->instruction_stream();
     int mode_mask = RelocInfo::EmbeddedObjectModeMask();
+    CodePageMemoryModificationScope memory_modification_scope(istream);
     for (RelocIterator it(code_, mode_mask); !it.done(); it.next()) {
       DCHECK(RelocInfo::IsEmbeddedObjectMode(it.rinfo()->rmode()));
       it.rinfo()->set_target_object(istream, value_);
@@ -506,8 +515,9 @@ UNINITIALIZED_TEST(ConcurrentRecordRelocSlot) {
     // The test requires concurrent marking barrier.
     return;
   }
-  v8_flags.manual_evacuation_candidates_selection = true;
   ManualGCScope manual_gc_scope;
+  heap::ManualEvacuationCandidatesSelectionScope
+      manual_evacuation_candidate_selection_scope(manual_gc_scope);
 
   v8::Isolate::CreateParams create_params;
   create_params.array_buffer_allocator = CcTest::array_buffer_allocator();
@@ -515,12 +525,11 @@ UNINITIALIZED_TEST(ConcurrentRecordRelocSlot) {
   Isolate* i_isolate = reinterpret_cast<Isolate*>(isolate);
   Heap* heap = i_isolate->heap();
   {
-    Code code;
-    HeapObject value;
-    CodePageCollectionMemoryModificationScopeForTesting code_scope(heap);
+    Tagged<Code> code;
+    Tagged<HeapObject> value;
     {
       HandleScope handle_scope(i_isolate);
-      i::byte buffer[i::Assembler::kDefaultBufferSize];
+      uint8_t buffer[i::Assembler::kDefaultBufferSize];
       MacroAssembler masm(i_isolate, v8::internal::CodeObjectRequired::kYes,
                           ExternalAssemblerBuffer(buffer, sizeof(buffer)));
 #if V8_TARGET_ARCH_ARM64
@@ -536,6 +545,8 @@ UNINITIALIZED_TEST(ConcurrentRecordRelocSlot) {
       masm.GetCode(i_isolate, &desc);
       Handle<Code> code_handle =
           Factory::CodeBuilder(i_isolate, desc, CodeKind::FOR_TESTING).Build();
+      // Globalize the handle for |code| for the incremental marker to mark it.
+      i_isolate->global_handles()->Create(*code_handle.location());
       heap::AbandonCurrentlyFreeMemory(heap->old_space());
       Handle<HeapNumber> value_handle(
           i_isolate->factory()->NewHeapNumber<AllocationType::kOld>(1.1));
@@ -543,15 +554,17 @@ UNINITIALIZED_TEST(ConcurrentRecordRelocSlot) {
       code = *code_handle;
       value = *value_handle;
     }
-    heap->StartIncrementalMarking(i::Heap::kNoGCFlags,
+    heap->StartIncrementalMarking(i::GCFlag::kNoFlags,
                                   i::GarbageCollectionReason::kTesting);
     CHECK(heap->marking_state()->IsUnmarked(value));
 
+    // Advance marking to make sure |code| is marked.
+    heap->incremental_marking()->AdvanceForTesting(v8::base::TimeDelta::Max());
+
+    CHECK(heap->marking_state()->IsMarked(code));
+    CHECK(heap->marking_state()->IsUnmarked(value));
+
     {
-      // TODO(v8:13023): remove ResetPKUPermissionsForThreadSpawning in the
-      // future when RwxMemoryWriteScope::SetDefaultPermissionsForNewThread() is
-      // stable.
-      ResetPKUPermissionsForThreadSpawning thread_scope;
       auto thread =
           std::make_unique<ConcurrentRecordRelocSlotThread>(heap, code, value);
       CHECK(thread->Start());
@@ -560,7 +573,7 @@ UNINITIALIZED_TEST(ConcurrentRecordRelocSlot) {
     }
 
     CHECK(heap->marking_state()->IsMarked(value));
-    heap::InvokeMarkSweep(i_isolate);
+    heap::InvokeMajorGC(heap);
   }
   isolate->Dispose();
 }
