@@ -40,8 +40,8 @@ TEST(WeakReferencesBasic) {
   if (!v8_flags.single_generation) CHECK(Heap::InYoungGeneration(*lh));
 
   MaybeObject code_object = lh->data1();
-  CHECK(code_object->IsSmi());
-  CcTest::CollectAllGarbage();
+  CHECK(IsSmi(code_object));
+  heap::InvokeMajorGC(CcTest::heap());
   CHECK(!Heap::InYoungGeneration(*lh));
   CHECK_EQ(code_object, lh->data1());
 
@@ -55,20 +55,20 @@ TEST(WeakReferencesBasic) {
     assm.GetCode(isolate, &desc);
     Handle<Code> code =
         Factory::CodeBuilder(isolate, desc, CodeKind::FOR_TESTING).Build();
-    CHECK(code->IsCode());
+    CHECK(IsCode(*code));
 
     lh->set_data1(HeapObjectReference::Weak(*code));
-    HeapObject code_heap_object;
-    CHECK(lh->data1()->GetHeapObjectIfWeak(&code_heap_object));
+    Tagged<HeapObject> code_heap_object;
+    CHECK(lh->data1().GetHeapObjectIfWeak(&code_heap_object));
     CHECK_EQ(*code, code_heap_object);
 
-    CcTest::CollectAllGarbage();
+    heap::InvokeMajorGC(CcTest::heap());
 
-    CHECK(lh->data1()->GetHeapObjectIfWeak(&code_heap_object));
+    CHECK(lh->data1().GetHeapObjectIfWeak(&code_heap_object));
     CHECK_EQ(*code, code_heap_object);
   }  // code will go out of scope.
 
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(CcTest::heap());
   CHECK(lh->data1()->IsCleared());
 }
 
@@ -77,7 +77,8 @@ TEST(WeakReferencesOldToOld) {
   // Like WeakReferencesBasic, but the updated weak slot is in the old space,
   // and referring to an old space object.
   ManualGCScope manual_gc_scope;
-  v8_flags.manual_evacuation_candidates_selection = true;
+  heap::ManualEvacuationCandidatesSelectionScope
+      manual_evacuation_candidate_selection_scope(manual_gc_scope);
   CcTest::InitializeVM();
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
@@ -96,11 +97,11 @@ TEST(WeakReferencesOldToOld) {
 
   Page* page_before_gc = Page::FromHeapObject(*fixed_array);
   heap::ForceEvacuationCandidate(page_before_gc);
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
   CHECK(heap->InOldSpace(*fixed_array));
 
-  HeapObject heap_object;
-  CHECK(lh->data1()->GetHeapObjectIfWeak(&heap_object));
+  Tagged<HeapObject> heap_object;
+  CHECK(lh->data1().GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(heap_object, *fixed_array);
 }
 
@@ -123,10 +124,10 @@ TEST(WeakReferencesOldToNew) {
   CHECK(Heap::InYoungGeneration(*fixed_array));
   lh->set_data1(HeapObjectReference::Weak(*fixed_array));
 
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
 
-  HeapObject heap_object;
-  CHECK(lh->data1()->GetHeapObjectIfWeak(&heap_object));
+  Tagged<HeapObject> heap_object;
+  CHECK(lh->data1().GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(heap_object, *fixed_array);
 }
 
@@ -149,10 +150,10 @@ TEST(WeakReferencesOldToNewScavenged) {
   CHECK(Heap::InYoungGeneration(*fixed_array));
   lh->set_data1(HeapObjectReference::Weak(*fixed_array));
 
-  CcTest::CollectGarbage(NEW_SPACE);
+  heap::InvokeMinorGC(heap);
 
-  HeapObject heap_object;
-  CHECK(lh->data1()->GetHeapObjectIfWeak(&heap_object));
+  Tagged<HeapObject> heap_object;
+  CHECK(lh->data1().GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(heap_object, *fixed_array);
 }
 
@@ -160,7 +161,8 @@ TEST(WeakReferencesOldToCleared) {
   // Like WeakReferencesBasic, but the updated weak slot is in the old space,
   // and is cleared.
   ManualGCScope manual_gc_scope;
-  v8_flags.manual_evacuation_candidates_selection = true;
+  heap::ManualEvacuationCandidatesSelectionScope
+      manual_evacuation_candidate_selection_scope(manual_gc_scope);
   CcTest::InitializeVM();
   Isolate* isolate = CcTest::i_isolate();
   Factory* factory = isolate->factory();
@@ -172,7 +174,7 @@ TEST(WeakReferencesOldToCleared) {
   CHECK(heap->InOldSpace(*lh));
   lh->set_data1(HeapObjectReference::ClearedValue(isolate));
 
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
   CHECK(lh->data1()->IsCleared());
 }
 
@@ -191,7 +193,7 @@ TEST(ObjectMovesBeforeClearingWeakField) {
   HandleScope outer_scope(isolate);
   Handle<LoadHandler> lh = CreateLoadHandlerForTest(factory);
   CHECK(InCorrectGeneration(*lh));
-  LoadHandler lh_location = *lh;
+  Tagged<LoadHandler> lh_location = *lh;
   {
     HandleScope inner_scope(isolate);
     // Create a new FixedArray which the LoadHandler will point to.
@@ -207,13 +209,13 @@ TEST(ObjectMovesBeforeClearingWeakField) {
   SimulateIncrementalMarking(heap, true);
 
   // Scavenger will move *lh.
-  CcTest::CollectGarbage(NEW_SPACE);
-  LoadHandler new_lh_location = *lh;
+  heap::InvokeMinorGC(heap);
+  Tagged<LoadHandler> new_lh_location = *lh;
   CHECK_NE(lh_location, new_lh_location);
   CHECK(lh->data1()->IsWeak());
 
   // Now we try to clear *lh.
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
   CHECK(lh->data1()->IsCleared());
 }
 
@@ -247,10 +249,10 @@ TEST(ObjectWithWeakFieldDies) {
   }  // outer_scope goes out of scope
 
   // lh will die
-  CcTest::CollectGarbage(NEW_SPACE);
+  heap::InvokeMinorGC(heap);
 
   // This used to crash when processing the dead weak reference.
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
 }
 
 TEST(ObjectWithWeakReferencePromoted) {
@@ -269,12 +271,12 @@ TEST(ObjectWithWeakReferencePromoted) {
   CHECK(Heap::InYoungGeneration(*fixed_array));
   lh->set_data1(HeapObjectReference::Weak(*fixed_array));
 
-  CcTest::CollectGarbage(OLD_SPACE);
+  heap::EmptyNewSpaceUsingGC(heap);
   CHECK(heap->InOldSpace(*lh));
   CHECK(heap->InOldSpace(*fixed_array));
 
-  HeapObject heap_object;
-  CHECK(lh->data1()->GetHeapObjectIfWeak(&heap_object));
+  Tagged<HeapObject> heap_object;
+  CHECK(lh->data1().GetHeapObjectIfWeak(&heap_object));
   CHECK_EQ(heap_object, *fixed_array);
 }
 
@@ -291,11 +293,11 @@ TEST(ObjectWithClearedWeakReferencePromoted) {
 
   lh->set_data1(HeapObjectReference::ClearedValue(isolate));
 
-  CcTest::CollectGarbage(OLD_SPACE);
+  heap::EmptyNewSpaceUsingGC(heap);
   CHECK(heap->InOldSpace(*lh));
   CHECK(lh->data1()->IsCleared());
 
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
   CHECK(lh->data1()->IsCleared());
 }
 
@@ -332,7 +334,7 @@ TEST(WeakReferenceWriteBarrier) {
     lh->set_data1(HeapObjectReference::Weak(*fixed_array2));
   }
 
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
 
   // Check that the write barrier treated the weak reference as strong.
   CHECK(lh->data1()->IsWeak());
@@ -345,8 +347,8 @@ TEST(EmptyWeakArray) {
   HandleScope outer_scope(isolate);
 
   Handle<WeakFixedArray> array = factory->empty_weak_fixed_array();
-  CHECK(array->IsWeakFixedArray());
-  CHECK(!array->IsFixedArray());
+  CHECK(IsWeakFixedArray(*array));
+  CHECK(!IsFixedArray(*array));
   CHECK_EQ(array->length(), 0);
 }
 
@@ -363,15 +365,15 @@ TEST(WeakArraysBasic) {
 
   const int length = 4;
   Handle<WeakFixedArray> array = factory->NewWeakFixedArray(length);
-  CHECK(array->IsWeakFixedArray());
-  CHECK(!array->IsFixedArray());
+  CHECK(IsWeakFixedArray(*array));
+  CHECK(!IsFixedArray(*array));
   CHECK_EQ(array->length(), length);
 
   CHECK(Heap::InYoungGeneration(*array));
 
   for (int i = 0; i < length; ++i) {
-    HeapObject heap_object;
-    CHECK(array->Get(i)->GetHeapObjectIfStrong(&heap_object));
+    Tagged<HeapObject> heap_object;
+    CHECK(array->Get(i).GetHeapObjectIfStrong(&heap_object));
     CHECK_EQ(heap_object, ReadOnlyRoots(heap).undefined_value());
   }
 
@@ -401,24 +403,24 @@ TEST(WeakArraysBasic) {
 
   // TODO(marja): update this when/if we do handle weak references in the new
   // space.
-  CcTest::CollectGarbage(NEW_SPACE);
-  HeapObject heap_object;
-  CHECK(array->Get(0)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2016);
-  CHECK(array->Get(1)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2017);
-  CHECK(array->Get(2)->GetHeapObjectIfStrong(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2018);
-  CHECK(array->Get(3)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2019);
+  heap::InvokeMinorGC(heap);
+  Tagged<HeapObject> heap_object;
+  CHECK(array->Get(0).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2016);
+  CHECK(array->Get(1).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2017);
+  CHECK(array->Get(2).GetHeapObjectIfStrong(&heap_object));
+  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2018);
+  CHECK(array->Get(3).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2019);
 
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
   CHECK(heap->InOldSpace(*array));
   CHECK(array->Get(0)->IsCleared());
-  CHECK(array->Get(1)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2017);
-  CHECK(array->Get(2)->GetHeapObjectIfStrong(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2018);
+  CHECK(array->Get(1).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2017);
+  CHECK(array->Get(2).GetHeapObjectIfStrong(&heap_object));
+  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2018);
   CHECK(array->Get(3)->IsCleared());
 }
 
@@ -435,9 +437,9 @@ TEST(WeakArrayListBasic) {
 
   Handle<WeakArrayList> array(ReadOnlyRoots(heap).empty_weak_array_list(),
                               isolate);
-  CHECK(array->IsWeakArrayList());
-  CHECK(!array->IsFixedArray());
-  CHECK(!array->IsWeakFixedArray());
+  CHECK(IsWeakArrayList(*array));
+  CHECK(!IsFixedArray(*array));
+  CHECK(!IsWeakFixedArray(*array));
   CHECK_EQ(array->length(), 0);
 
   Handle<FixedArray> index2 = factory->NewFixedArray(1);
@@ -498,33 +500,33 @@ TEST(WeakArrayListBasic) {
 
   // TODO(marja): update this when/if we do handle weak references in the new
   // space.
-  CcTest::CollectGarbage(NEW_SPACE);
-  HeapObject heap_object;
+  heap::InvokeMinorGC(heap);
+  Tagged<HeapObject> heap_object;
   CHECK_EQ(array->length(), 8);
-  CHECK(array->Get(0)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2016);
+  CHECK(array->Get(0).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2016);
   CHECK_EQ(array->Get(1).ToSmi().value(), 1);
 
-  CHECK(array->Get(2)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2017);
+  CHECK(array->Get(2).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2017);
   CHECK_EQ(array->Get(3).ToSmi().value(), 3);
 
-  CHECK(array->Get(4)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2018);
+  CHECK(array->Get(4).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2018);
   CHECK_EQ(array->Get(5).ToSmi().value(), 5);
 
-  CHECK(array->Get(6)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2019);
+  CHECK(array->Get(6).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2019);
   CHECK_EQ(array->Get(7).ToSmi().value(), 7);
 
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
   CHECK(heap->InOldSpace(*array));
   CHECK_EQ(array->length(), 8);
   CHECK(array->Get(0)->IsCleared());
   CHECK_EQ(array->Get(1).ToSmi().value(), 1);
 
-  CHECK(array->Get(2)->GetHeapObjectIfWeak(&heap_object));
-  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object).get(0)).value(), 2017);
+  CHECK(array->Get(2).GetHeapObjectIfWeak(&heap_object));
+  CHECK_EQ(Smi::cast(FixedArray::cast(heap_object)->get(0)).value(), 2017);
   CHECK_EQ(array->Get(3).ToSmi().value(), 3);
 
   CHECK(array->Get(4)->IsCleared());
@@ -617,14 +619,14 @@ TEST(Regress7768) {
   CompileRun("%DeoptimizeFunction(myfunc);");
 
   // The object pointed to by the weak reference won't be scavenged.
-  CcTest::CollectGarbage(NEW_SPACE);
+  heap::InvokeMinorGC(heap);
 
   // Make sure the memory where it's stored is invalidated, so that we'll crash
   // if we try to access it.
   HeapTester::UncommitUnusedMemory(heap);
 
   // This used to crash when processing the dead weak reference.
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
 }
 
 TEST(PrototypeUsersBasic) {
@@ -709,9 +711,9 @@ TEST(PrototypeUsersBasic) {
 
 namespace {
 
-HeapObject saved_heap_object;
+Tagged<HeapObject> saved_heap_object;
 
-static void TestCompactCallback(HeapObject value, int old_index,
+static void TestCompactCallback(Tagged<HeapObject> value, int old_index,
                                 int new_index) {
   saved_heap_object = value;
   CHECK_EQ(old_index, 2);
@@ -753,13 +755,13 @@ TEST(PrototypeUsersCompacted) {
   }
 
   PrototypeUsers::MarkSlotEmpty(*array, 1);
-  CcTest::CollectAllGarbage();
+  heap::InvokeMajorGC(heap);
   CHECK(array->Get(3)->IsCleared());
 
   CHECK_EQ(array->length(), 3 + PrototypeUsers::kFirstIndex);
-  WeakArrayList new_array =
+  Tagged<WeakArrayList> new_array =
       PrototypeUsers::Compact(array, heap, TestCompactCallback);
-  CHECK_EQ(new_array.length(), 1 + PrototypeUsers::kFirstIndex);
+  CHECK_EQ(new_array->length(), 1 + PrototypeUsers::kFirstIndex);
   CHECK_EQ(saved_heap_object, *live_map);
 }
 
