@@ -72,31 +72,32 @@ static inline Handle<T> CompileRunI(const char* script) {
   return OpenHandle<T>(CompileRun(script));
 }
 
-static Object GetFieldValue(JSObject obj, int property_index) {
-  FieldIndex index = FieldIndex::ForPropertyIndex(obj.map(), property_index);
-  return obj.RawFastPropertyAt(index);
+static Tagged<Object> GetFieldValue(Tagged<JSObject> obj, int property_index) {
+  FieldIndex index = FieldIndex::ForPropertyIndex(obj->map(), property_index);
+  return obj->RawFastPropertyAt(index);
 }
 
-static double GetDoubleFieldValue(JSObject obj, FieldIndex field_index) {
-  Object value = obj.RawFastPropertyAt(field_index);
-  if (value.IsHeapNumber()) {
-    return HeapNumber::cast(value).value();
+static double GetDoubleFieldValue(Tagged<JSObject> obj,
+                                  FieldIndex field_index) {
+  Tagged<Object> value = obj->RawFastPropertyAt(field_index);
+  if (IsHeapNumber(value)) {
+    return HeapNumber::cast(value)->value();
   } else {
-    return value.Number();
+    return Object::Number(value);
   }
 }
 
-static double GetDoubleFieldValue(JSObject obj, int property_index) {
-  FieldIndex index = FieldIndex::ForPropertyIndex(obj.map(), property_index);
+static double GetDoubleFieldValue(Tagged<JSObject> obj, int property_index) {
+  FieldIndex index = FieldIndex::ForPropertyIndex(obj->map(), property_index);
   return GetDoubleFieldValue(obj, index);
 }
 
-bool IsObjectShrinkable(JSObject obj) {
+bool IsObjectShrinkable(Tagged<JSObject> obj) {
   Handle<Map> filler_map =
       CcTest::i_isolate()->factory()->one_pointer_filler_map();
 
-  int inobject_properties = obj.map().GetInObjectProperties();
-  int unused = obj.map().UnusedPropertyFields();
+  int inobject_properties = obj->map()->GetInObjectProperties();
+  int unused = obj->map()->UnusedPropertyFields();
   if (unused == 0) return false;
 
   Address packed_filler = MapWord::FromMap(*filler_map).ptr();
@@ -139,7 +140,7 @@ TEST(JSObjectBasic) {
   CHECK(initial_map->IsInobjectSlackTrackingInProgress());
 
   // There must be at least some slack.
-  CHECK_LT(5, obj->map().GetInObjectProperties());
+  CHECK_LT(5, obj->map()->GetInObjectProperties());
   CHECK_EQ(Smi::FromInt(42), GetFieldValue(*obj, 0));
   CHECK_EQ(4.2, GetDoubleFieldValue(*obj, 1));
   CHECK_EQ(*obj, GetFieldValue(*obj, 2));
@@ -156,7 +157,7 @@ TEST(JSObjectBasic) {
   CHECK(!IsObjectShrinkable(*obj));
 
   // No slack left.
-  CHECK_EQ(3, obj->map().GetInObjectProperties());
+  CHECK_EQ(3, obj->map()->GetInObjectProperties());
 }
 
 
@@ -200,7 +201,7 @@ TEST(JSObjectComplex) {
   CHECK(initial_map->IsInobjectSlackTrackingInProgress());
 
   // There must be at least some slack.
-  CHECK_LT(5, obj3->map().GetInObjectProperties());
+  CHECK_LT(5, obj3->map()->GetInObjectProperties());
   CHECK_EQ(Smi::FromInt(42), GetFieldValue(*obj3, 0));
   CHECK_EQ(4.2, GetDoubleFieldValue(*obj3, 1));
   CHECK_EQ(*obj3, GetFieldValue(*obj3, 2));
@@ -220,14 +221,14 @@ TEST(JSObjectComplex) {
   CHECK(IsObjectShrinkable(*obj3));
   CHECK(!IsObjectShrinkable(*obj5));
 
-  CHECK_EQ(5, obj1->map().GetInObjectProperties());
-  CHECK_EQ(4, obj1->map().UnusedPropertyFields());
+  CHECK_EQ(5, obj1->map()->GetInObjectProperties());
+  CHECK_EQ(4, obj1->map()->UnusedPropertyFields());
 
-  CHECK_EQ(5, obj3->map().GetInObjectProperties());
-  CHECK_EQ(2, obj3->map().UnusedPropertyFields());
+  CHECK_EQ(5, obj3->map()->GetInObjectProperties());
+  CHECK_EQ(2, obj3->map()->UnusedPropertyFields());
 
-  CHECK_EQ(5, obj5->map().GetInObjectProperties());
-  CHECK_EQ(0, obj5->map().UnusedPropertyFields());
+  CHECK_EQ(5, obj5->map()->GetInObjectProperties());
+  CHECK_EQ(0, obj5->map()->UnusedPropertyFields());
 
   // Since slack tracking is complete, the new objects should not be shrinkable.
   obj1 = CompileRunI<JSObject>("new A(1);");
@@ -285,7 +286,7 @@ TEST(JSGeneratorObjectBasic) {
   CHECK(initial_map->IsInobjectSlackTrackingInProgress());
 
   // There must be at least some slack.
-  CHECK_LT(5, obj->map().GetInObjectProperties());
+  CHECK_LT(5, obj->map()->GetInObjectProperties());
   CHECK_EQ(Smi::FromInt(42), GetFieldValue(*obj, 0));
   CHECK_EQ(4.2, GetDoubleFieldValue(*obj, 1));
   CHECK_EQ(*obj, GetFieldValue(*obj, 2));
@@ -302,7 +303,7 @@ TEST(JSGeneratorObjectBasic) {
   CHECK(!IsObjectShrinkable(*obj));
 
   // No slack left.
-  CHECK_EQ(3, obj->map().GetInObjectProperties());
+  CHECK_EQ(3, obj->map()->GetInObjectProperties());
 }
 
 
@@ -369,7 +370,7 @@ TEST(SubclassBasicNoBaseClassInstances) {
   CHECK(b_initial_map->IsInobjectSlackTrackingInProgress());
 
   // There must be at least some slack.
-  CHECK_LT(10, obj->map().GetInObjectProperties());
+  CHECK_LT(10, obj->map()->GetInObjectProperties());
   CHECK_EQ(Smi::FromInt(42), GetFieldValue(*obj, 0));
   CHECK_EQ(4.2, GetDoubleFieldValue(*obj, 1));
   CHECK_EQ(*obj, GetFieldValue(*obj, 2));
@@ -394,7 +395,7 @@ TEST(SubclassBasicNoBaseClassInstances) {
   CHECK(a_initial_map->IsInobjectSlackTrackingInProgress());
 
   // No slack left.
-  CHECK_EQ(6, obj->map().GetInObjectProperties());
+  CHECK_EQ(6, obj->map()->GetInObjectProperties());
 }
 
 
@@ -474,10 +475,10 @@ TEST(SubclassBasic) {
   CHECK(!IsObjectShrinkable(*a_obj));
 
   // No slack left.
-  CHECK_EQ(3, a_obj->map().GetInObjectProperties());
+  CHECK_EQ(3, a_obj->map()->GetInObjectProperties());
 
   // There must be at least some slack.
-  CHECK_LT(10, b_obj->map().GetInObjectProperties());
+  CHECK_LT(10, b_obj->map()->GetInObjectProperties());
   CHECK_EQ(Smi::FromInt(42), GetFieldValue(*b_obj, 0));
   CHECK_EQ(4.2, GetDoubleFieldValue(*b_obj, 1));
   CHECK_EQ(*b_obj, GetFieldValue(*b_obj, 2));
@@ -497,7 +498,7 @@ TEST(SubclassBasic) {
   CHECK(!IsObjectShrinkable(*b_obj));
 
   // No slack left.
-  CHECK_EQ(6, b_obj->map().GetInObjectProperties());
+  CHECK_EQ(6, b_obj->map()->GetInObjectProperties());
 }
 
 
@@ -574,10 +575,10 @@ static void TestClassHierarchy(const std::vector<int>& hierarchy_desc, int n) {
     Handle<Map> initial_map(func->initial_map(), func->GetIsolate());
 
     // If the object is slow-mode already, bail out.
-    if (obj->map().is_dictionary_map()) continue;
+    if (obj->map()->is_dictionary_map()) continue;
 
     // There must be at least some slack.
-    CHECK_LT(fields_count, obj->map().GetInObjectProperties());
+    CHECK_LT(fields_count, obj->map()->GetInObjectProperties());
 
     // One instance was created.
     CHECK_EQ(Map::kSlackTrackingCounterStart - 1,
@@ -601,7 +602,7 @@ static void TestClassHierarchy(const std::vector<int>& hierarchy_desc, int n) {
     CHECK(!IsObjectShrinkable(*obj));
 
     // No slack left.
-    CHECK_EQ(fields_count, obj->map().GetInObjectProperties());
+    CHECK_EQ(fields_count, obj->map()->GetInObjectProperties());
   }
 }
 
@@ -684,8 +685,8 @@ TEST(InobjectPropetiesCountOverflowInSubclass) {
     Handle<Map> initial_map(func->initial_map(), func->GetIsolate());
 
     // There must be no slack left.
-    CHECK_EQ(JSObject::kMaxInstanceSize, obj->map().instance_size());
-    CHECK_EQ(kMaxInobjectProperties, obj->map().GetInObjectProperties());
+    CHECK_EQ(JSObject::kMaxInstanceSize, obj->map()->instance_size());
+    CHECK_EQ(kMaxInobjectProperties, obj->map()->GetInObjectProperties());
 
     // One instance was created.
     CHECK_EQ(Map::kSlackTrackingCounterStart - 1,
@@ -702,7 +703,7 @@ TEST(InobjectPropetiesCountOverflowInSubclass) {
     CHECK(!IsObjectShrinkable(*obj));
 
     // No slack left.
-    CHECK_EQ(kMaxInobjectProperties, obj->map().GetInObjectProperties());
+    CHECK_EQ(kMaxInobjectProperties, obj->map()->GetInObjectProperties());
   }
 
   // The other classes in the hierarchy are not affected.
@@ -712,7 +713,7 @@ TEST(InobjectPropetiesCountOverflowInSubclass) {
 static void CheckExpectedProperties(int expected, std::ostringstream& os) {
   Handle<HeapObject> obj = Handle<HeapObject>::cast(
       v8::Utils::OpenHandle(*CompileRun(os.str().c_str())));
-  CHECK_EQ(expected, obj->map().GetInObjectProperties());
+  CHECK_EQ(expected, obj->map()->GetInObjectProperties());
 }
 
 TEST(ObjectLiteralPropertyBackingStoreSize) {
@@ -870,8 +871,8 @@ TEST(SlowModeSubclass) {
     Handle<Map> initial_map(func->initial_map(), func->GetIsolate());
 
     // Object should go dictionary mode.
-    CHECK_EQ(JSObject::kHeaderSize, obj->map().instance_size());
-    CHECK(obj->map().is_dictionary_map());
+    CHECK_EQ(JSObject::kHeaderSize, obj->map()->instance_size());
+    CHECK(obj->map()->is_dictionary_map());
 
     // One instance was created.
     CHECK_EQ(Map::kSlackTrackingCounterStart - 1,
@@ -888,8 +889,8 @@ TEST(SlowModeSubclass) {
     CHECK(!IsObjectShrinkable(*obj));
 
     // Object should stay in dictionary mode.
-    CHECK_EQ(JSObject::kHeaderSize, obj->map().instance_size());
-    CHECK(obj->map().is_dictionary_map());
+    CHECK_EQ(JSObject::kHeaderSize, obj->map()->instance_size());
+    CHECK(obj->map()->is_dictionary_map());
   }
 
   // The other classes in the hierarchy are not affected.
@@ -952,7 +953,7 @@ static void TestSubclassBuiltin(const char* subclass_name,
   CHECK(initial_map->IsInobjectSlackTrackingInProgress());
 
   // There must be at least some slack.
-  CHECK_LT(builtin_properties_count + 5, obj->map().GetInObjectProperties());
+  CHECK_LT(builtin_properties_count + 5, obj->map()->GetInObjectProperties());
   CHECK_EQ(Smi::FromInt(42), GetFieldValue(*obj, builtin_properties_count + 0));
   CHECK_EQ(4.2, GetDoubleFieldValue(*obj, builtin_properties_count + 1));
   CHECK_EQ(*obj, GetFieldValue(*obj, builtin_properties_count + 2));
@@ -969,9 +970,9 @@ static void TestSubclassBuiltin(const char* subclass_name,
   CHECK(!IsObjectShrinkable(*obj));
 
   // No slack left.
-  CHECK_EQ(builtin_properties_count + 3, obj->map().GetInObjectProperties());
+  CHECK_EQ(builtin_properties_count + 3, obj->map()->GetInObjectProperties());
 
-  CHECK_EQ(instance_type, obj->map().instance_type());
+  CHECK_EQ(instance_type, obj->map()->instance_type());
 }
 
 
@@ -1283,8 +1284,8 @@ TEST(SubclassTranspiledClassHierarchy) {
   CHECK(!IsObjectShrinkable(*obj));
 
   // No slack left.
-  CHECK_EQ(21, obj->map().GetInObjectProperties());
-  CHECK_EQ(JS_OBJECT_TYPE, obj->map().instance_type());
+  CHECK_EQ(21, obj->map()->GetInObjectProperties());
+  CHECK_EQ(JS_OBJECT_TYPE, obj->map()->instance_type());
 }
 
 TEST(Regress8853_ClassConstructor) {
@@ -1294,9 +1295,9 @@ TEST(Regress8853_ClassConstructor) {
   // For classes without any this.prop assignments in their
   // constructors we start out with 10 inobject properties.
   Handle<JSObject> obj = CompileRunI<JSObject>("new (class {});\n");
-  CHECK(obj->map().IsInobjectSlackTrackingInProgress());
+  CHECK(obj->map()->IsInobjectSlackTrackingInProgress());
   CHECK(IsObjectShrinkable(*obj));
-  CHECK_EQ(10, obj->map().GetInObjectProperties());
+  CHECK_EQ(10, obj->map()->GetInObjectProperties());
 
   // For classes with N explicit this.prop assignments in their
   // constructors we start out with N+8 inobject properties.
@@ -1308,9 +1309,9 @@ TEST(Regress8853_ClassConstructor) {
       "    this.z = 3;\n"
       "  }\n"
       "});\n");
-  CHECK(obj->map().IsInobjectSlackTrackingInProgress());
+  CHECK(obj->map()->IsInobjectSlackTrackingInProgress());
   CHECK(IsObjectShrinkable(*obj));
-  CHECK_EQ(3 + 8, obj->map().GetInObjectProperties());
+  CHECK_EQ(3 + 8, obj->map()->GetInObjectProperties());
 }
 
 TEST(Regress8853_ClassHierarchy) {
@@ -1324,9 +1325,9 @@ TEST(Regress8853_ClassHierarchy) {
   for (int i = 1; i < 10; ++i) {
     std::string script = "new " + base + ";\n";
     Handle<JSObject> obj = CompileRunI<JSObject>(script.c_str());
-    CHECK(obj->map().IsInobjectSlackTrackingInProgress());
+    CHECK(obj->map()->IsInobjectSlackTrackingInProgress());
     CHECK(IsObjectShrinkable(*obj));
-    CHECK_EQ(8 + 2 * i, obj->map().GetInObjectProperties());
+    CHECK_EQ(8 + 2 * i, obj->map()->GetInObjectProperties());
     base = "(class extends " + base + " {})";
   }
 }
@@ -1338,9 +1339,9 @@ TEST(Regress8853_FunctionConstructor) {
   // For constructor functions without any this.prop assignments in
   // them we start out with 10 inobject properties.
   Handle<JSObject> obj = CompileRunI<JSObject>("new (function() {});\n");
-  CHECK(obj->map().IsInobjectSlackTrackingInProgress());
+  CHECK(obj->map()->IsInobjectSlackTrackingInProgress());
   CHECK(IsObjectShrinkable(*obj));
-  CHECK_EQ(10, obj->map().GetInObjectProperties());
+  CHECK_EQ(10, obj->map()->GetInObjectProperties());
 
   // For constructor functions with N explicit this.prop assignments
   // in them we start out with N+8 inobject properties.
@@ -1353,9 +1354,9 @@ TEST(Regress8853_FunctionConstructor) {
       "  this.c = 3;\n"
       "  this.f = 3;\n"
       "});\n");
-  CHECK(obj->map().IsInobjectSlackTrackingInProgress());
+  CHECK(obj->map()->IsInobjectSlackTrackingInProgress());
   CHECK(IsObjectShrinkable(*obj));
-  CHECK_EQ(6 + 8, obj->map().GetInObjectProperties());
+  CHECK_EQ(6 + 8, obj->map()->GetInObjectProperties());
 }
 
 TEST(InstanceFieldsArePropertiesDefaultConstructorLazy) {
@@ -1376,7 +1377,7 @@ TEST(InstanceFieldsArePropertiesDefaultConstructorLazy) {
       "  x09 = null;\n"
       "  x10 = null;\n"
       "});\n");
-  CHECK_EQ(11 + 8, obj->map().GetInObjectProperties());
+  CHECK_EQ(11 + 8, obj->map()->GetInObjectProperties());
 }
 
 TEST(InstanceFieldsArePropertiesFieldsAndConstructorLazy) {
@@ -1409,7 +1410,7 @@ TEST(InstanceFieldsArePropertiesFieldsAndConstructorLazy) {
       "    this.x20 = null;\n"
       "  }\n"
       "});\n");
-  CHECK_EQ(21 + 8, obj->map().GetInObjectProperties());
+  CHECK_EQ(21 + 8, obj->map()->GetInObjectProperties());
 }
 
 TEST(InstanceFieldsArePropertiesDefaultConstructorEager) {
@@ -1431,7 +1432,7 @@ TEST(InstanceFieldsArePropertiesDefaultConstructorEager) {
       "  x09 = null;\n"
       "  x10 = null;\n"
       "});\n");
-  CHECK_EQ(11 + 8, obj->map().GetInObjectProperties());
+  CHECK_EQ(11 + 8, obj->map()->GetInObjectProperties());
 }
 
 TEST(InstanceFieldsArePropertiesFieldsAndConstructorEager) {
@@ -1465,7 +1466,7 @@ TEST(InstanceFieldsArePropertiesFieldsAndConstructorEager) {
       "    this.x20 = null;\n"
       "  }\n"
       "});\n");
-  CHECK_EQ(21 + 8, obj->map().GetInObjectProperties());
+  CHECK_EQ(21 + 8, obj->map()->GetInObjectProperties());
 }
 
 }  // namespace test_inobject_slack_tracking
