@@ -168,11 +168,11 @@ TEST(Unwind_BuiltinPCInMiddle_Success_CodePagesAPI) {
   register_state.fp = stack;
 
   // Put the current PC inside of a valid builtin.
-  Code builtin = *BUILTIN_CODE(i_isolate, StringEqual);
+  Tagged<Code> builtin = *BUILTIN_CODE(i_isolate, StringEqual);
   const uintptr_t offset = 40;
-  CHECK_LT(offset, builtin.instruction_size());
+  CHECK_LT(offset, builtin->instruction_size());
   register_state.pc =
-      reinterpret_cast<void*>(builtin.instruction_start() + offset);
+      reinterpret_cast<void*>(builtin->instruction_start() + offset);
 
   bool unwound = v8::Unwinder::TryUnwindV8Frames(
       entry_stubs, pages_length, code_pages, &register_state, stack_base);
@@ -225,8 +225,8 @@ TEST(Unwind_BuiltinPCAtStart_Success_CodePagesAPI) {
 
   // Put the current PC at the start of a valid builtin, so that we are setting
   // up the frame.
-  Code builtin = *BUILTIN_CODE(i_isolate, StringEqual);
-  register_state.pc = reinterpret_cast<void*>(builtin.instruction_start());
+  Tagged<Code> builtin = *BUILTIN_CODE(i_isolate, StringEqual);
+  register_state.pc = reinterpret_cast<void*>(builtin->instruction_start());
 
   bool unwound = v8::Unwinder::TryUnwindV8Frames(
       entry_stubs, pages_length, code_pages, &register_state, stack_base);
@@ -253,10 +253,10 @@ const char* foo_source = R"(
 
 bool PagesContainsAddress(size_t length, MemoryRange* pages,
                           Address search_address) {
-  byte* addr = reinterpret_cast<byte*>(search_address);
+  uint8_t* addr = reinterpret_cast<uint8_t*>(search_address);
   auto it = std::find_if(pages, pages + length, [addr](const MemoryRange& r) {
-    const byte* page_start = reinterpret_cast<const byte*>(r.start);
-    const byte* page_end = page_start + r.length_in_bytes;
+    const uint8_t* page_start = reinterpret_cast<const uint8_t*>(r.start);
+    const uint8_t* page_end = page_start + r.length_in_bytes;
     return addr >= page_start && addr < page_end;
   });
   return it != pages + length;
@@ -296,17 +296,17 @@ TEST(Unwind_CodeObjectPCInMiddle_Success_CodePagesAPI) {
       Handle<JSFunction>::cast(v8::Utils::OpenHandle(*local_foo));
 
   // Put the current PC inside of the created code object.
-  Code code = foo->code();
+  Tagged<Code> code = foo->code();
   // We don't produce optimized code when run with --no-turbofan and
   // --no-maglev.
-  if (!code.is_optimized_code()) return;
+  if (!code->is_optimized_code()) return;
 
   // We don't want the offset too early or it could be the `push rbp`
   // instruction (which is not at the start of generated code, because the lazy
   // deopt check happens before frame setup).
-  const uintptr_t offset = code.instruction_size() - 20;
-  CHECK_LT(offset, code.instruction_size());
-  Address pc = code.instruction_start() + offset;
+  const uintptr_t offset = code->instruction_size() - 20;
+  CHECK_LT(offset, code->instruction_size());
+  Address pc = code->instruction_start() + offset;
   register_state.pc = reinterpret_cast<void*>(pc);
 
   // Get code pages from the API now that the code obejct exists and check that
@@ -455,8 +455,8 @@ TEST(Unwind_JSEntry_Fail_CodePagesAPI) {
   CHECK_LE(pages_length, arraysize(code_pages));
   RegisterState register_state;
 
-  Code js_entry = *BUILTIN_CODE(i_isolate, JSEntry);
-  byte* start = reinterpret_cast<byte*>(js_entry.instruction_start());
+  Tagged<Code> js_entry = *BUILTIN_CODE(i_isolate, JSEntry);
+  uint8_t* start = reinterpret_cast<uint8_t*>(js_entry->instruction_start());
   register_state.pc = start + 10;
 
   bool unwound = v8::Unwinder::TryUnwindV8Frames(
@@ -593,7 +593,7 @@ TEST(PCIsInV8_ValidStateNullPC_Fail_CodePagesAPI) {
 }
 
 void TestRangeBoundaries(size_t pages_length, MemoryRange* code_pages,
-                         byte* range_start, size_t range_length) {
+                         uint8_t* range_start, size_t range_length) {
   void* pc = range_start - 1;
   CHECK(!v8::Unwinder::PCIsInV8(pages_length, code_pages, pc));
   pc = range_start;
@@ -618,8 +618,8 @@ TEST(PCIsInV8_InAllCodePages_CodePagesAPI) {
   CHECK_LE(pages_length, arraysize(code_pages));
 
   for (size_t i = 0; i < pages_length; i++) {
-    byte* range_start =
-        const_cast<byte*>(reinterpret_cast<const byte*>(code_pages[i].start));
+    uint8_t* range_start = const_cast<uint8_t*>(
+        reinterpret_cast<const uint8_t*>(code_pages[i].start));
     size_t range_length = code_pages[i].length_in_bytes;
     TestRangeBoundaries(pages_length, code_pages, range_start, range_length);
   }
@@ -637,9 +637,9 @@ TEST(PCIsInV8_InJSEntryRange_CodePagesAPI) {
       isolate->CopyCodePages(arraysize(code_pages), code_pages);
   CHECK_LE(pages_length, arraysize(code_pages));
 
-  Code js_entry = *BUILTIN_CODE(i_isolate, JSEntry);
-  byte* start = reinterpret_cast<byte*>(js_entry.instruction_start());
-  size_t length = js_entry.instruction_size();
+  Tagged<Code> js_entry = *BUILTIN_CODE(i_isolate, JSEntry);
+  uint8_t* start = reinterpret_cast<uint8_t*>(js_entry->instruction_start());
+  size_t length = js_entry->instruction_size();
 
   void* pc = start;
   CHECK(v8::Unwinder::PCIsInV8(pages_length, code_pages, pc));
@@ -661,7 +661,7 @@ TEST(PCIsInV8_LargeCodeObject_CodePagesAPI) {
   // Create a big function that ends up in CODE_LO_SPACE.
   const int instruction_size = Page::kPageSize + 1;
   CHECK_GT(instruction_size, MemoryChunkLayout::MaxRegularCodeObjectSize());
-  std::unique_ptr<byte[]> instructions(new byte[instruction_size]);
+  std::unique_ptr<uint8_t[]> instructions(new uint8_t[instruction_size]);
 
   CodeDesc desc;
   desc.buffer = instructions.get();
@@ -677,7 +677,7 @@ TEST(PCIsInV8_LargeCodeObject_CodePagesAPI) {
 
   CHECK(i_isolate->heap()->InSpace(foo_code->instruction_stream(),
                                    CODE_LO_SPACE));
-  byte* start = reinterpret_cast<byte*>(foo_code->instruction_start());
+  uint8_t* start = reinterpret_cast<uint8_t*>(foo_code->instruction_start());
 
   MemoryRange code_pages[v8::Isolate::kMinCodePagesBufferSize];
   size_t pages_length =
