@@ -4,9 +4,13 @@
 
 // Flags: --allow-natives-syntax
 
-function foo(x) { Math.floor(x); }
+var foo = (function(stdlib) {
+  "use asm";
+  var bar = stdlib.Symbol;
+  function foo() { return bar("lala"); }
+  return foo;
+})(this);
+
 %PrepareFunctionForOptimization(foo);
-foo(1);
-foo(2);
 %OptimizeFunctionOnNextCall(foo);
-assertThrows(function() { foo(Symbol()) }, TypeError);
+foo();
