@@ -4,6 +4,7 @@
 
 d8.file.execute("test/mjsunit/wasm/wasm-module-builder.js");
 
+const kMemtypeSize64 = 8;
 const kMemtypeSize32 = 4;
 const kMemtypeSize16 = 2;
 const kMemtypeSize8 = 1;
@@ -36,6 +37,26 @@ function GetAtomicBinOpFunction(wasmExpression, alignment, offset) {
   return instance.exports.main;
 }
 
+function GetI64AtomicBinOpFunction(wasmExpression, alignment, offset) {
+  const kSig_l_il = makeSig([kWasmI32, kWasmI64], [kWasmI64]);
+  let builder = new WasmModuleBuilder();
+  builder.addImportedMemory("m", "imported_mem", 0, maxSize, "shared");
+  builder.addFunction("main", kSig_l_il)
+    .addBody([
+      kExprLocalGet, 0,
+      kExprLocalGet, 1,
+      kAtomicPrefix,
+      wasmExpression, alignment, offset
+    ])
+    .exportAs("main");
+
+  // Instantiate module, get function exports
+  let module = new WebAssembly.Module(builder.toBuffer());
+  let instance = new WebAssembly.Instance(module,
+        {m: {imported_mem: memory}});
+  return instance.exports.main;
+}
+
 function GetAtomicCmpExchangeFunction(wasmExpression, alignment, offset) {
   let builder = new WasmModuleBuilder();
   builder.addImportedMemory("m", "imported_mem", 0, maxSize, "shared");
@@ -90,6 +111,23 @@ function GetAtomicStoreFunction(wasmExpression, alignment, offset) {
   return instance.exports.main;
 }
 
+function VerifyBoundsCheck64(func, memtype_size) {
+  const kPageSize = 65536;
+  // Test out of bounds at boundary
+  for (let i = memory.buffer.byteLength - memtype_size + 1;
+       i < memory.buffer.byteLength + memtype_size + 4; i++) {
+    assertTrapsOneOf(
+      // If an underlying platform uses traps for a bounds check,
+      // kTrapUnalignedAccess will be thrown before kTrapMemOutOfBounds.
+      // Otherwise, kTrapMemOutOfBounds will be first.
+      [kTrapMemOutOfBounds, kTrapUnalignedAccess],
+      () => func(i, 5n, 10n)
+    );
+  }
+  // Test out of bounds at maximum + 1
+  assertTraps(kTrapMemOutOfBounds, () => func((maxSize + 1) * kPageSize, 5n, 1n));
+}
+
 function VerifyBoundsCheck(func, memtype_size) {
   const kPageSize = 65536;
   // Test out of bounds at boundary
@@ -147,6 +185,54 @@ function Test8Op(operation, func) {
   VerifyBoundsCheck(func, kMemtypeSize8, 10);
 }
 
+function Test64Op(operation, func) {
+  let i64 = new BigUint64Array(memory.buffer);
+  for (let i = 0; i < i64.length; i = inc(i)) {
+    let expected = 987659876543210n;
+    let value = 111111111111111n;
+    i64[i] = expected;
+    assertEquals(expected, func(i * kMemtypeSize64, value));
+    assertEquals(operation(expected, value), i64[i]);
+  }
+  VerifyBoundsCheck64(func, kMemtypeSize64);
+}
+
+function Test32Op64(operation, func) {
+  let i32 = new Uint32Array(memory.buffer);
+  for (let i = 0; i < i32.length; i = inc(i)) {
+    let expected = 123456;
+    let value = 111111n;
+    i32[i] = expected;
+    assertEquals(expected, Number(func(i * kMemtypeSize32, value)));
+    assertEquals(operation(expected, Number(value)), i32[i]);
+  }
+  VerifyBoundsCheck64(func, kMemtypeSize32);
+}
+
+function Test16Op64(operation, func) {
+  let i16 = new Uint16Array(memory.buffer);
+  for (let i = 0; i < i16.length; i = inc(i)) {
+    let expected = 0xd00d;
+    let value = 0x1111n;
+    i16[i] = expected;
+    assertEquals(expected, Number(func(i * kMemtypeSize16, value)));
+    assertEquals(operation(expected, Number(value)), i16[i]);
+  }
+  VerifyBoundsCheck64(func, kMemtypeSize16);
+}
+
+function Test8Op64(operation, func) {
+  let i8 = new Uint8Array(memory.buffer);
+  for (let i = 0; i < i8.length; i = inc(i)) {
+    let expected = 0xbe;
+    let value = 0x12n;
+    i8[i] = expected;
+    assertEquals(expected, Number(func(i * kMemtypeSize8, value)));
+    assertEquals(operation(expected, Number(value)), i8[i]);
+  }
+  VerifyBoundsCheck64(func, kMemtypeSize8, 10);
+}
+
 (function TestAtomicAdd() {
   print(arguments.callee.name);
   let wasmAdd = GetAtomicBinOpFunction(kExprI32AtomicAdd, 2, 0);
@@ -165,6 +251,30 @@ function Test8Op(operation, func) {
   Test8Op(Add, wasmAdd);
 })();
 
+(function TestI64AtomicAdd() {
+  print(arguments.callee.name);
+  let wasmAdd = GetI64AtomicBinOpFunction(kExprI64AtomicAdd, 3, 0);
+  Test64Op(Add, wasmAdd);
+})();
+
+(function TestI64AtomicAdd32U() {
+  print(arguments.callee.name);
+  let wasmAdd = GetI64AtomicBinOpFunction(kExprI64AtomicAdd32U, 2, 0);
+  Test32Op64(Add, wasmAdd);
+})();
+
+(function TestI64AtomicAdd16U() {
+  print(arguments.callee.name);
+  let wasmAdd = GetI64AtomicBinOpFunction(kExprI64AtomicAdd16U, 1, 0);
+  Test16Op64(Add, wasmAdd);
+})();
+
+(function TestI64AtomicAdd8U() {
+  print(arguments.callee.name);
+  let wasmAdd = GetI64AtomicBinOpFunction(kExprI64AtomicAdd8U, 0, 0);
+  Test8Op64(Add, wasmAdd);
+})();
+
 (function TestAtomicSub() {
   print(arguments.callee.name);
   let wasmSub = GetAtomicBinOpFunction(kExprI32AtomicSub, 2, 0);
@@ -183,6 +293,30 @@ function Test8Op(operation, func) {
   Test8Op(Sub, wasmSub);
 })();
 
+(function TestI64AtomicSub() {
+  print(arguments.callee.name);
+  let wasmSub = GetI64AtomicBinOpFunction(kExprI64AtomicSub, 3, 0);
+  Test64Op(Sub, wasmSub);
+})();
+
+(function TestI64AtomicSub32U() {
+  print(arguments.callee.name);
+  let wasmSub = GetI64AtomicBinOpFunction(kExprI64AtomicSub32U, 2, 0);
+  Test32Op64(Sub, wasmSub);
+})();
+
+(function TestI64AtomicSub16U() {
+  print(arguments.callee.name);
+  let wasmSub = GetI64AtomicBinOpFunction(kExprI64AtomicSub16U, 1, 0);
+  Test16Op64(Sub, wasmSub);
+})();
+
+(function TestI64AtomicSub8U() {
+  print(arguments.callee.name);
+  let wasmSub = GetI64AtomicBinOpFunction(kExprI64AtomicSub8U, 0, 0);
+  Test8Op64(Sub, wasmSub);
+})();
+
 (function TestAtomicAnd() {
   print(arguments.callee.name);
   let wasmAnd = GetAtomicBinOpFunction(kExprI32AtomicAnd, 2, 0);
@@ -201,6 +335,30 @@ function Test8Op(operation, func) {
   Test8Op(And, wasmAnd);
 })();
 
+(function TestI64AtomicAnd() {
+  print(arguments.callee.name);
+  let wasmAnd = GetI64AtomicBinOpFunction(kExprI64AtomicAnd, 3, 0);
+  Test64Op(And, wasmAnd);
+})();
+
+(function TestI64AtomicAnd32U() {
+  print(arguments.callee.name);
+  let wasmAnd = GetI64AtomicBinOpFunction(kExprI64AtomicAnd32U, 2, 0);
+  Test32Op64(And, wasmAnd);
+})();
+
+(function TestI64AtomicAnd16U() {
+  print(arguments.callee.name);
+  let wasmAnd = GetI64AtomicBinOpFunction(kExprI64AtomicAnd16U, 1, 0);
+  Test16Op64(And, wasmAnd);
+})();
+
+(function TestI64AtomicAnd8U() {
+  print(arguments.callee.name);
+  let wasmAnd = GetI64AtomicBinOpFunction(kExprI64AtomicAnd8U, 0, 0);
+  Test8Op64(And, wasmAnd);
+})();
+
 (function TestAtomicOr() {
   print(arguments.callee.name);
   let wasmOr = GetAtomicBinOpFunction(kExprI32AtomicOr, 2, 0);
@@ -219,6 +377,30 @@ function Test8Op(operation, func) {
   Test8Op(Or, wasmOr);
 })();
 
+(function TestI64AtomicOr() {
+  print(arguments.callee.name);
+  let wasmOr = GetI64AtomicBinOpFunction(kExprI64AtomicOr, 3, 0);
+  Test64Op(Or, wasmOr);
+})();
+
+(function TestI64AtomicOr32U() {
+  print(arguments.callee.name);
+  let wasmOr = GetI64AtomicBinOpFunction(kExprI64AtomicOr32U, 2, 0);
+  Test32Op64(Or, wasmOr);
+})();
+
+(function TestI64AtomicOr16U() {
+  print(arguments.callee.name);
+  let wasmOr = GetI64AtomicBinOpFunction(kExprI64AtomicOr16U, 1, 0);
+  Test16Op64(Or, wasmOr);
+})();
+
+(function TestI64AtomicOr8U() {
+  print(arguments.callee.name);
+  let wasmOr = GetI64AtomicBinOpFunction(kExprI64AtomicOr8U, 0, 0);
+  Test8Op64(Or, wasmOr);
+})();
+
 (function TestAtomicXor() {
   print(arguments.callee.name);
   let wasmXor = GetAtomicBinOpFunction(kExprI32AtomicXor, 2, 0);
@@ -237,6 +419,30 @@ function Test8Op(operation, func) {
   Test8Op(Xor, wasmXor);
 })();
 
+(function TestI64AtomicXor() {
+  print(arguments.callee.name);
+  let wasmXor = GetI64AtomicBinOpFunction(kExprI64AtomicXor, 3, 0);
+  Test64Op(Xor, wasmXor);
+})();
+
+(function TestI64AtomicXor32U() {
+  print(arguments.callee.name);
+  let wasmXor = GetI64AtomicBinOpFunction(kExprI64AtomicXor32U, 2, 0);
+  Test32Op64(Xor, wasmXor);
+})();
+
+(function TestI64AtomicXor16U() {
+  print(arguments.callee.name);
+  let wasmXor = GetI64AtomicBinOpFunction(kExprI64AtomicXor16U, 1, 0);
+  Test16Op64(Xor, wasmXor);
+})();
+
+(function TestI64AtomicXor8U() {
+  print(arguments.callee.name);
+  let wasmXor = GetI64AtomicBinOpFunction(kExprI64AtomicXor8U, 0, 0);
+  Test8Op64(Xor, wasmXor);
+})();
+
 (function TestAtomicExchange() {
   print(arguments.callee.name);
   let wasmExchange = GetAtomicBinOpFunction(kExprI32AtomicExchange, 2, 0);
@@ -255,6 +461,31 @@ function Test8Op(operation, func) {
   Test8Op(Exchange, wasmExchange);
 })();
 
+(function TestI64AtomicExchange() {
+  print(arguments.callee.name);
+  let wasmExchange = GetI64AtomicBinOpFunction(kExprI64AtomicExchange, 3, 0);
+  Test64Op(Exchange, wasmExchange);
+})();
+
+(function TestI64AtomicExchange32U() {
+  print(arguments.callee.name);
+  let wasmExchange = GetI64AtomicBinOpFunction(kExprI64AtomicExchange32U, 2, 0);
+  Test32Op64(Exchange, wasmExchange);
+})();
+
+(function TestI64AtomicExchange16U() {
+  print(arguments.callee.name);
+  let wasmExchange = GetI64AtomicBinOpFunction(kExprI64AtomicExchange16U, 1, 0);
+  Test16Op64(Exchange, wasmExchange);
+})();
+
+(function TestI64AtomicExchange8U() {
+  print(arguments.callee.name);
+  let wasmExchange = GetI64AtomicBinOpFunction(kExprI64AtomicExchange8U, 0, 0);
+  Test8Op64(Exchange, wasmExchange);
+})();
+
+
 function TestCmpExchange(func, buffer, params, size, offset = 0) {
   for (let i = 0; i + (offset / size) < buffer.length; i = inc(i)) {
     for (let j = 0; j < params.length; j++) {
