@@ -5,7 +5,7 @@
 d8.file.execute("test/mjsunit/wasm/wasm-module-builder.js");
 
 const builder = new WasmModuleBuilder();
-builder.addMemory(1, undefined, false);
+builder.addMemory(1, undefined);
 builder.addFunction('load', kSig_i_ii)
     .addBody([
         kExprLocalGet, 0,
