@@ -19,7 +19,8 @@ function BasicMemory64Tests(num_pages, use_atomic_ops) {
       use_atomic_ops ? '' : 'non-'}atomic memory`);
 
   let builder = new WasmModuleBuilder();
-  builder.addMemory64(num_pages, num_pages, true);
+  builder.addMemory64(num_pages, num_pages);
+  builder.exportMemoryAs('memory');
 
   // A memory operation with alignment (0) and offset (0).
   let op = (non_atomic, atomic) => use_atomic_ops ?
@@ -194,10 +195,79 @@ function allowOOM(fn) {
   assertEquals(-1n, instance.exports.grow(1n << 32n));
   assertEquals(-1n, instance.exports.grow(1n << 33n));
   assertEquals(-1n, instance.exports.grow(1n << 63n));
-  assertEquals(-1n, instance.exports.grow(7n));  // Above the of 10.
+  assertEquals(-1n, instance.exports.grow(7n));  // Above the maximum of 10.
   assertEquals(4n, instance.exports.grow(6n));   // Just at the maximum of 10.
 })();
 
+(function TestGrow64_ToMemory() {
+  print(arguments.callee.name);
+  let builder = new WasmModuleBuilder();
+  builder.addMemory64(1, 10);
+  builder.exportMemoryAs('memory');
+
+  // Grow memory and store the result in memory for inspection from JS.
+  builder.addFunction('grow', makeSig([kWasmI64], []))
+      .addBody([
+        kExprI64Const, 0,       // i64.const (offset for result)
+        kExprLocalGet, 0,       // local.get 0
+        kExprMemoryGrow, 0,     // memory.grow 0
+        kExprI64StoreMem, 3, 0  // store result to memory
+      ])
+      .exportFunc();
+
+  let instance = builder.instantiate();
+  function grow(arg) {
+    instance.exports.grow(arg);
+    let i64_arr = new BigInt64Array(instance.exports.memory.buffer, 0, 1);
+    return i64_arr[0];
+  }
+
+  assertEquals(1n, grow(2n));
+  assertEquals(3n, grow(1n));
+  assertEquals(-1n, grow(-1n));
+  assertEquals(-1n, grow(1n << 31n));
+  assertEquals(-1n, grow(1n << 32n));
+  assertEquals(-1n, grow(1n << 33n));
+  assertEquals(-1n, grow(1n << 63n));
+  assertEquals(-1n, grow(7n));  // Above the maximum of 10.
+  assertEquals(4n, grow(6n));   // Just at the maximum of 10.
+})();
+
+(function TestGrow64_Above4GB() {
+  print(arguments.callee.name);
+  let builder = new WasmModuleBuilder();
+  let max_pages = 5 * GB / kPageSize;
+  builder.addMemory64(1, max_pages);
+  builder.exportMemoryAs('memory');
+
+  builder.addFunction('grow', makeSig([kWasmI64], [kWasmI64]))
+      .addBody([
+        kExprLocalGet, 0,    // local.get 0
+        kExprMemoryGrow, 0,  // memory.grow 0
+      ])
+      .exportFunc();
+
+  let instance = builder.instantiate();
+
+  // Grow from 1 to 3 pages.
+  assertEquals(1n, instance.exports.grow(2n));
+  // Grow from 3 to {max_pages - 1} pages.
+  // This step can fail. We have to allow this, even though it weakens this test
+  // (we do not know if we failed because of OOM or because of a wrong
+  // engine-internal limit of 4GB).
+  let grow_big_result = instance.exports.grow(BigInt(max_pages) - 4n);
+  if (grow_big_result == -1) return;
+  assertEquals(3n, grow_big_result);
+  // Cannot grow by 2 pages.
+  assertEquals(-1n, instance.exports.grow(2n));
+  // Cannot grow by 2^32 pages.
+  assertEquals(-1n, instance.exports.grow(1n << 32n));
+  // Grow by one more page to the maximum.
+  assertEquals(BigInt(max_pages) - 1n, instance.exports.grow(1n));
+  // Cannot grow further.
+  assertEquals(-1n, instance.exports.grow(1n));
+})();
+
 (function TestBulkMemoryOperations() {
   print(arguments.callee.name);
   let builder = new WasmModuleBuilder();
@@ -307,10 +377,57 @@ function allowOOM(fn) {
   assertTraps(kTrapMemOutOfBounds, () => fill(1n << 63n, 0, 1n));
 })();
 
+(function TestBulkMemoryConstOperations() {
+  print(arguments.callee.name);
+  let builder = new WasmModuleBuilder();
+  const kMemSizeInPages = 10;
+  builder.addMemory64(kMemSizeInPages, kMemSizeInPages);
+  const kSegmentSize = 1024;
+  // Build a data segment with values [0, kSegmentSize-1].
+  const segment = Array.from({length: kSegmentSize}, (_, idx) => idx)
+  builder.addPassiveDataSegment(segment);
+  builder.exportMemoryAs('memory');
+
+  builder.addFunction('fill', makeSig([kWasmI32, kWasmI64], []))
+      .addBody([
+        kExprI64Const, 15,                  // i64.const 15
+        kExprLocalGet, 0,                   // local.get 0 (value)
+        kExprLocalGet, 1,                   // local.get 1 (size)
+        kNumericPrefix, kExprMemoryFill, 0  // memory.fill mem=0
+      ])
+      .exportFunc();
+
+  builder.addFunction('init', makeSig([kWasmI32, kWasmI32], []))
+      .addBody([
+        kExprI64Const, 5,                      // i64.const 5
+        kExprLocalGet, 0,                      // local.get 0 (offset)
+        kExprLocalGet, 1,                      // local.get 1 (size)
+        kNumericPrefix, kExprMemoryInit, 0, 0  // memory.init seg=0 mem=0
+      ])
+      .exportFunc();
+
+  let instance = builder.instantiate();
+  let fill = instance.exports.fill;
+  let init = instance.exports.init;
+  // {memory(offset,size)} extracts the memory at [offset, offset+size)] into an
+  // Array.
+  let memory = (offset, size) => Array.from(new Uint8Array(
+      instance.exports.memory.buffer.slice(offset, offset + size)));
+
+  // Init memory[5..7] with [10..12].
+  init(10, 3);
+  assertEquals([0, 0, 10, 11, 12, 0, 0], memory(3, 7));
+
+  // Fill memory[15..17] with 3s.
+  fill(3, 3n);
+  assertEquals([0, 3, 3, 3, 0], memory(14, 5));
+})();
+
 (function TestMemory64SharedBasic() {
   print(arguments.callee.name);
   let builder = new WasmModuleBuilder();
-  builder.addMemory64(1, 10, true, true);
+  builder.addMemory64(1, 10, true);
+  builder.exportMemoryAs('memory');
   builder.addFunction('load', makeSig([kWasmI64], [kWasmI32]))
       .addBody([
         kExprLocalGet, 0,       // local.get 0
@@ -330,7 +447,8 @@ function allowOOM(fn) {
   // TODO(clemensb): Use the proper API once that's decided.
   let shared_mem64 = (function() {
     let builder = new WasmModuleBuilder();
-    builder.addMemory64(1, 10, true, true);
+    builder.addMemory64(1, 10, true);
+    builder.exportMemoryAs('memory');
     return builder.instantiate().exports.memory;
   })();
 
@@ -375,8 +493,9 @@ function allowOOM(fn) {
       }
 
       function workerAssertEquals(expected, actual, message) {
-        if (expected != actual)
+        if (expected != actual) {
           postMessage(`Check failed (${message}): ${expected} != ${actual}`);
+        }
       }
 
       const kOffset1 = 47n;
@@ -411,3 +530,145 @@ function allowOOM(fn) {
   // This test can fail if 5GB of memory cannot be allocated.
   allowOOM(() => BasicMemory64Tests(num_pages, true));
 })();
+
+(function Test64BitOffsetOn32BitMemory() {
+  print(arguments.callee.name);
+  let builder = new WasmModuleBuilder();
+  builder.addMemory(1, 1);
+
+  builder.addFunction('load', makeSig([kWasmI32], [kWasmI32]))
+      .addBody([
+        // local.get 0
+        kExprLocalGet, 0,
+        // i32.load align=0 offset=2^32+2
+        kExprI32LoadMem, 0, ...wasmSignedLeb64(Math.pow(2, 32) + 2),
+      ])
+      .exportFunc();
+
+  // An offset outside the 32-bit range should not validate.
+  assertFalse(WebAssembly.validate(builder.toBuffer()));
+})();
+
+(function Test64BitOffsetOn64BitMemory() {
+  print(arguments.callee.name);
+  let builder = new WasmModuleBuilder();
+  builder.addMemory64(1, 1);
+
+  builder.addFunction('load', makeSig([kWasmI64], [kWasmI32]))
+      .addBody([
+        // local.get 0
+        kExprLocalGet, 0,
+        // i32.load align=0 offset=2^32+2
+        kExprI32LoadMem, 0, ...wasmSignedLeb64(Math.pow(2, 32) + 2),
+      ])
+      .exportFunc();
+
+  // Instantiation works, this should throw at runtime.
+  let instance = builder.instantiate();
+  let load = instance.exports.load;
+
+  assertTraps(kTrapMemOutOfBounds, () => load(0n));
+})();
+
+(function TestImportMemory64() {
+  print(arguments.callee.name);
+  const builder1 = new WasmModuleBuilder();
+  builder1.addMemory64(1, 1);
+  builder1.exportMemoryAs('mem64');
+  const instance1 = builder1.instantiate();
+  const {mem64} = instance1.exports;
+
+  let builder2 = new WasmModuleBuilder();
+  builder2.addImportedMemory(
+      'imp', 'mem', 1, 1, /* shared */ false, /* memory64 */ true);
+  builder2.instantiate({imp: {mem: mem64}});
+})();
+
+(function TestImportMemory64AsMemory32() {
+  print(arguments.callee.name);
+  const builder1 = new WasmModuleBuilder();
+  builder1.addMemory64(1, 1);
+  builder1.exportMemoryAs('mem64');
+  const instance1 = builder1.instantiate();
+  const {mem64} = instance1.exports;
+
+  let builder2 = new WasmModuleBuilder();
+  builder2.addImportedMemory('imp', 'mem');
+  assertThrows(
+      () => builder2.instantiate({imp: {mem: mem64}}), WebAssembly.LinkError,
+      'WebAssembly.Instance(): cannot import memory64 as memory32');
+})();
+
+(function TestImportMemory32AsMemory64() {
+  print(arguments.callee.name);
+  const builder1 = new WasmModuleBuilder();
+  builder1.addMemory(1, 1);
+  builder1.exportMemoryAs('mem32');
+  const instance1 = builder1.instantiate();
+  const {mem32} = instance1.exports;
+
+  let builder2 = new WasmModuleBuilder();
+  builder2.addImportedMemory(
+      'imp', 'mem', 1, 1, /* shared */ false, /* memory64 */ true);
+  assertThrows(
+      () => builder2.instantiate({imp: {mem: mem32}}), WebAssembly.LinkError,
+      'WebAssembly.Instance(): cannot import memory32 as memory64');
+})();
+
+function InstantiatingWorkerCode() {
+  function workerAssert(condition, message) {
+    if (!condition) postMessage(`Check failed: ${message}`);
+  }
+
+  onmessage = function([mem, module]) {
+    workerAssert(mem instanceof WebAssembly.Memory, 'Wasm memory');
+    workerAssert(mem.buffer instanceof SharedArrayBuffer, 'SAB');
+    try {
+      new WebAssembly.Instance(module, {imp: {mem: mem}});
+      postMessage('Instantiation succeeded');
+    } catch (e) {
+      postMessage(`Exception: ${e}`);
+    }
+  };
+}
+
+(function TestImportMemory64AsMemory32InWorker() {
+  print(arguments.callee.name);
+  const builder1 = new WasmModuleBuilder();
+  builder1.addMemory64(1, 1, /* shared */ true);
+  builder1.exportMemoryAs('mem64');
+  const instance1 = builder1.instantiate();
+  const {mem64} = instance1.exports;
+
+  let builder2 = new WasmModuleBuilder();
+  builder2.addImportedMemory('imp', 'mem');
+  let module2 = builder2.toModule();
+
+  let worker = new Worker(InstantiatingWorkerCode, {type: 'function'});
+  worker.postMessage([mem64, module2]);
+  assertEquals(
+      'Exception: LinkError: WebAssembly.Instance(): ' +
+          'cannot import memory64 as memory32',
+      worker.getMessage());
+})();
+
+(function TestImportMemory32AsMemory64InWorker() {
+  print(arguments.callee.name);
+  const builder1 = new WasmModuleBuilder();
+  builder1.addMemory(1, 1, /* shared */ true);
+  builder1.exportMemoryAs('mem32');
+  const instance1 = builder1.instantiate();
+  const {mem32} = instance1.exports;
+
+  let builder2 = new WasmModuleBuilder();
+  builder2.addImportedMemory(
+      'imp', 'mem', 1, 1, /* shared */ false, /* memory64 */ true);
+  let module2 = builder2.toModule();
+
+  let worker = new Worker(InstantiatingWorkerCode, {type: 'function'});
+  worker.postMessage([mem32, module2]);
+  assertEquals(
+      'Exception: LinkError: WebAssembly.Instance(): ' +
+          'cannot import memory32 as memory64',
+      worker.getMessage());
+})();
