@@ -3,6 +3,8 @@
 // found in the LICENSE file.
 
 // Flags: --experimental-wasm-stringref --experimental-wasm-typed-funcref
+// For {isOneByteString}:
+// Flags: --expose-externalize-string
 
 d8.file.execute("test/mjsunit/wasm/wasm-module-builder.js");
 
@@ -154,7 +156,7 @@ function makeWtf8TestDataSegment() {
 (function TestStringNewWtf8() {
   let builder = new WasmModuleBuilder();
 
-  builder.addMemory(1, undefined, false, false);
+  builder.addMemory(1, undefined);
   let data = makeWtf8TestDataSegment();
   builder.addDataSegment(0, data.data);
 
@@ -222,7 +224,7 @@ function makeWtf8TestDataSegment() {
 (function TestStringNewUtf8TryNullCheck() {
   let builder = new WasmModuleBuilder();
 
-  builder.addMemory(1, undefined, false, false);
+  builder.addMemory(1, undefined);
   let data = makeWtf8TestDataSegment();
   builder.addDataSegment(0, data.data);
 
@@ -274,7 +276,7 @@ function makeWtf16TestDataSegment() {
 (function TestStringNewWtf16() {
   let builder = new WasmModuleBuilder();
 
-  builder.addMemory(1, undefined, false, false);
+  builder.addMemory(1, undefined);
   let data = makeWtf16TestDataSegment();
   builder.addDataSegment(0, data.data);
 
@@ -389,7 +391,8 @@ function makeWtf16TestDataSegment() {
 (function TestStringEncodeWtf8() {
   let builder = new WasmModuleBuilder();
 
-  builder.addMemory(1, undefined, true /* exported */, false);
+  builder.addMemory(1, undefined);
+  builder.exportMemoryAs("memory");
 
   for (let [instr, name] of [[kExprStringEncodeUtf8, "utf8"],
                              [kExprStringEncodeWtf8, "wtf8"],
@@ -487,7 +490,8 @@ function makeWtf16TestDataSegment() {
 (function TestStringEncodeWtf16() {
   let builder = new WasmModuleBuilder();
 
-  builder.addMemory(1, undefined, true /* exported */, false);
+  builder.addMemory(1, undefined);
+  builder.exportMemoryAs("memory");
 
   builder.addFunction("encode_wtf16", kSig_i_wi)
     .exportFunc()
@@ -684,7 +688,8 @@ function makeWtf16TestDataSegment() {
 (function TestStringViewWtf16() {
   let builder = new WasmModuleBuilder();
 
-  builder.addMemory(1, undefined, true /* exported */, false);
+  builder.addMemory(1, undefined);
+  builder.exportMemoryAs("memory");
 
   builder.addFunction("view_from_null", kSig_v_v).exportFunc().addBody([
     kExprRefNull, kStringRefCode,
@@ -842,6 +847,9 @@ function makeWtf16TestDataSegment() {
   assertEquals("oo", instance.exports.slice("foo", 1, 3));
   assertEquals("oo", instance.exports.slice("foo", 1, 100));
   assertEquals("", instance.exports.slice("foo", 1, 0));
+  assertEquals("", instance.exports.slice("foo", 3, 4));
+  assertEquals("foo", instance.exports.slice("foo", 0, -1));
+  assertEquals("", instance.exports.slice("foo", -1, 1));
 
   assertThrows(() => instance.exports.view_from_null(),
                WebAssembly.RuntimeError, 'dereferencing a null pointer');
@@ -855,12 +863,44 @@ function makeWtf16TestDataSegment() {
                WebAssembly.RuntimeError, "dereferencing a null pointer");
   assertThrows(() => instance.exports.slice_null(),
                WebAssembly.RuntimeError, "dereferencing a null pointer");
+
+  // Cover runtime code path for long slices.
+  const prefix = "a".repeat(10);
+  const slice = "x".repeat(40);
+  const suffix = "b".repeat(40);
+  const input = prefix + slice + suffix;
+  const start = prefix.length;
+  const end = start + slice.length;
+  assertEquals(slice, instance.exports.slice(input, start, end));
+
+  // Check that we create one-byte substrings when possible.
+  let onebyte = instance.exports.slice("\u1234abcABCDE", 1, 4);
+  assertEquals("abc", onebyte);
+  assertTrue(isOneByteString(onebyte));
+
+  // Check that the CodeStubAssembler implementation also creates one-byte
+  // substrings.
+  onebyte = instance.exports.slice("\u1234abcA", 1, 4);
+  assertEquals("abc", onebyte);
+  assertTrue(isOneByteString(onebyte));
+  // Cover the code path that checks 8 characters at a time.
+  onebyte = instance.exports.slice("\u1234abcdefgh\u1234", 1, 9);
+  assertEquals("abcdefgh", onebyte);  // Exactly 8 characters.
+  assertTrue(isOneByteString(onebyte));
+  onebyte = instance.exports.slice("\u1234abcdefghijXYZ", 1, 11);
+  assertEquals("abcdefghij", onebyte);  // Longer than 8.
+  assertTrue(isOneByteString(onebyte));
+
+  // Check that the runtime code path also creates one-byte substrings.
+  assertTrue(isOneByteString(
+      instance.exports.slice(input + "\u1234", start, end)));
 })();
 
 (function TestStringViewWtf8() {
   let builder = new WasmModuleBuilder();
 
-  builder.addMemory(1, undefined, true /* exported */, false);
+  builder.addMemory(1, undefined);
+  builder.exportMemoryAs("memory");
 
   builder.addFunction("advance", kSig_i_wii)
     .exportFunc()
