@@ -50,7 +50,7 @@ using DisasmX64Test = TestWithIsolate;
 namespace {
 
 Handle<Code> CreateDummyCode(Isolate* isolate) {
-  i::byte buffer[128];
+  uint8_t buffer[128];
   Assembler assm(AssemblerOptions{},
                  ExternalAssemblerBuffer(buffer, sizeof(buffer)));
   __ nop();
@@ -66,7 +66,7 @@ Handle<Code> CreateDummyCode(Isolate* isolate) {
 
 TEST_F(DisasmX64Test, DisasmX64) {
   HandleScope handle_scope(isolate());
-  v8::internal::byte buffer[8192];
+  uint8_t buffer[8192];
   Assembler assm(AssemblerOptions{},
                  ExternalAssemblerBuffer(buffer, sizeof buffer));
   // Some instructions are tested in DisasmX64CheckOutput.
@@ -304,11 +304,11 @@ TEST_F(DisasmX64Test, DisasmX64) {
   USE(code);
 #ifdef OBJECT_PRINT
   StdoutStream os;
-  code->Print(os);
+  Print(*code, os);
   Address begin = code->instruction_start();
   Address end = code->instruction_start();
-  disasm::Disassembler::Disassemble(stdout, reinterpret_cast<byte*>(begin),
-                                    reinterpret_cast<byte*>(end));
+  disasm::Disassembler::Disassemble(stdout, reinterpret_cast<uint8_t*>(begin),
+                                    reinterpret_cast<uint8_t*>(end));
 #endif
 }
 
@@ -331,7 +331,7 @@ struct DisassemblerTester {
 
   Assembler* assm() { return &assm_; }
 
-  v8::internal::byte buffer_[kAssemblerBufferSize];
+  uint8_t buffer_[kAssemblerBufferSize];
   Assembler assm_;
   disasm::NameConverter converter_;
   disasm::Disassembler disasm;
@@ -749,6 +749,15 @@ TEST_F(DisasmX64Test, DisasmX64CheckOutput) {
           cmovq(less_equal, rax, Operand(rdx, 2)));
   COMPARE("480f4f4203           REX.W cmovgq rax,[rdx+0x3]",
           cmovq(greater, rax, Operand(rdx, 3)));
+  COMPARE("4180f803             cmpb r8l,0x3", cmpb(r8, Immediate(0x3)));
+  COMPARE("6681fa1008           cmpw rdx,0x810", cmpw(rdx, Immediate(0x810)));
+  COMPARE("4180e208             andb r10l,0x8", andb(r10, Immediate(0x8)));
+  COMPARE("4181e1ff3f0000       andl r9,0x3fff", andl(r9, Immediate(0x3fff)));
+  COMPARE("4183e30f             andl r11,0xf", andl(r11, Immediate(0xf)));
+  COMPARE("4883c418             REX.W addq rsp,0x18",
+          addq(rsp, Immediate(0x18)));
+  COMPARE("4881c1cd000000       REX.W addq rcx,0xcd",
+          addq(rcx, Immediate(0xcd)));
 }
 
 // This compares just the disassemble instruction (without the hex).
@@ -1441,6 +1450,8 @@ TEST_F(DisasmX64Test, DisasmX64YMMRegister) {
             vhaddps(ymm0, ymm1, Operand(rbx, rcx, times_4, 10000)));
     COMPARE("c4e27d18bc8b10270000 vbroadcastss ymm7,[rbx+rcx*4+0x2710]",
             vbroadcastss(ymm7, Operand(rbx, rcx, times_4, 10000)));
+    COMPARE("c4e27d19b48b10270000 vbroadcastsd ymm6,[rbx+rcx*4+0x2710]",
+            vbroadcastsd(ymm6, Operand(rbx, rcx, times_4, 10000)));
     COMPARE("c5ff12da             vmovddup ymm3,ymm2", vmovddup(ymm3, ymm2));
     COMPARE("c5ff12a48b10270000   vmovddup ymm4,[rbx+rcx*4+0x2710]",
             vmovddup(ymm4, Operand(rbx, rcx, times_4, 10000)));
@@ -1454,6 +1465,8 @@ TEST_F(DisasmX64Test, DisasmX64YMMRegister) {
             vcvttps2dq(ymm3, ymm2));
     COMPARE("c5fe5b9c8b10270000   vcvttps2dq ymm3,[rbx+rcx*4+0x2710]",
             vcvttps2dq(ymm3, Operand256(rbx, rcx, times_4, 10000)));
+    COMPARE("c4e36d06cb02         vperm2f128 ymm1,ymm2,ymm3,0x2",
+            vperm2f128(ymm1, ymm2, ymm3, 2));
 
     // vcmp
     COMPARE("c5dcc2e900           vcmpps ymm5,ymm4,ymm1, (eq)",
@@ -1472,6 +1485,9 @@ TEST_F(DisasmX64Test, DisasmX64YMMRegister) {
             vcmpnlepd(ymm5, ymm4, Operand(rbx, rcx, times_4, 10000)));
     COMPARE("c5dcc2e90d           vcmpps ymm5,ymm4,ymm1, (ge)",
             vcmpgeps(ymm5, ymm4, ymm1));
+    COMPARE("c4e27d17f9           vptest ymm7,ymm1", vptest(ymm7, ymm1));
+    COMPARE("c4627d17948b10270000 vptest ymm10,[rbx+rcx*4+0x2710]",
+            vptest(ymm10, Operand(rbx, rcx, times_4, 10000)));
 
     // SSE2_UNOP
     COMPARE("c5fd51ca             vsqrtpd ymm1,ymm2", vsqrtpd(ymm1, ymm2));
@@ -1496,6 +1512,8 @@ TEST_F(DisasmX64Test, DisasmX64YMMRegister) {
     // Short immediate instructions
     COMPARE("c4e27d18d1           vbroadcastss ymm2,xmm1",
             vbroadcastss(ymm2, xmm1));
+    COMPARE("c4e27d19f1           vbroadcastsd ymm6,xmm1",
+            vbroadcastsd(ymm6, xmm1));
     COMPARE("c4e27d789c8b10270000 vpbroadcastb ymm3,[rbx+rcx*4+0x2710]",
             vpbroadcastb(ymm3, Operand(rbx, rcx, times_4, 10000)));
     COMPARE("c4e27d79d3           vpbroadcastw ymm2,xmm3",
@@ -1509,6 +1527,24 @@ TEST_F(DisasmX64Test, DisasmX64YMMRegister) {
             vpabsb(ymm3, Operand(rbx, rcx, times_4, 10000)));
     COMPARE("c4e27d1df5           vpabsw ymm6,ymm5", vpabsw(ymm6, ymm5));
     COMPARE("c4c27d1efa           vpabsd ymm7,ymm10", vpabsd(ymm7, ymm10));
+    COMPARE("c4e3fd00ebd8         vpermq ymm5,ymm3,0xd8",
+            vpermq(ymm5, ymm3, 0xD8));
+    COMPARE("c463fd00848b102700001e vpermq ymm8,[rbx+rcx*4+0x2710],0x1e",
+            vpermq(ymm8, Operand(rbx, rcx, times_4, 10000), 0x1E));
+
+    // SSE4_UNOP
+    COMPARE("c4e27d20f5           vpmovsxbw ymm6,ymm5", vpmovsxbw(ymm6, ymm5));
+    COMPARE("c4e27d238c8b10270000 vpmovsxwd ymm1,[rbx+rcx*4+0x2710]",
+            vpmovsxwd(ymm1, Operand(rbx, rcx, times_4, 10000)));
+    COMPARE("c4627d25f6           vpmovsxdq ymm14,ymm6",
+            vpmovsxdq(ymm14, ymm6));
+    COMPARE("c4e27d30848b10270000 vpmovzxbw ymm0,[rbx+rcx*4+0x2710]",
+            vpmovzxbw(ymm0, Operand(rbx, rcx, times_4, 10000)));
+    COMPARE("c4627d31f6           vpmovzxbd ymm14,ymm6",
+            vpmovzxbd(ymm14, ymm6));
+    COMPARE("c4e27d33bc8b10270000 vpmovzxwd ymm7,[rbx+rcx*4+0x2710]",
+            vpmovzxwd(ymm7, Operand(rbx, rcx, times_4, 10000)));
+    COMPARE("c4627d35c6           vpmovzxdq ymm8,ymm6", vpmovzxdq(ymm8, ymm6));
   }
 }
 
