@@ -258,9 +258,9 @@ TEST_P(MicrotaskQueueTest, PromiseHandlerContext) {
   Handle<Context> context2 = Utils::OpenHandle(*v8_context2, isolate());
   Handle<Context> context3 = Utils::OpenHandle(*v8_context3, isolate());
   Handle<Context> context4 = Utils::OpenHandle(*v8_context3, isolate());
-  context2->native_context().set_microtask_queue(isolate(), microtask_queue());
-  context3->native_context().set_microtask_queue(isolate(), microtask_queue());
-  context4->native_context().set_microtask_queue(isolate(), microtask_queue());
+  context2->native_context()->set_microtask_queue(isolate(), microtask_queue());
+  context3->native_context()->set_microtask_queue(isolate(), microtask_queue());
+  context4->native_context()->set_microtask_queue(isolate(), microtask_queue());
 
   Handle<JSFunction> handler;
   Handle<JSProxy> proxy;
@@ -318,17 +318,17 @@ TEST_P(MicrotaskQueueTest, PromiseHandlerContext) {
 
   ASSERT_EQ(4, microtask_queue()->size());
   Handle<Microtask> microtask1(microtask_queue()->get(0), isolate());
-  ASSERT_TRUE(microtask1->IsPromiseFulfillReactionJobTask());
+  ASSERT_TRUE(IsPromiseFulfillReactionJobTask(*microtask1));
   EXPECT_EQ(*context2,
             Handle<PromiseFulfillReactionJobTask>::cast(microtask1)->context());
 
   Handle<Microtask> microtask2(microtask_queue()->get(1), isolate());
-  ASSERT_TRUE(microtask2->IsPromiseRejectReactionJobTask());
+  ASSERT_TRUE(IsPromiseRejectReactionJobTask(*microtask2));
   EXPECT_EQ(*context2,
             Handle<PromiseRejectReactionJobTask>::cast(microtask2)->context());
 
   Handle<Microtask> microtask3(microtask_queue()->get(2), isolate());
-  ASSERT_TRUE(microtask3->IsPromiseFulfillReactionJobTask());
+  ASSERT_TRUE(IsPromiseFulfillReactionJobTask(*microtask3));
   // |microtask3| corresponds to a PromiseReaction for |revoked_proxy|.
   // As |revoked_proxy| doesn't have a context, the current context should be
   // used as the fallback context.
@@ -336,7 +336,7 @@ TEST_P(MicrotaskQueueTest, PromiseHandlerContext) {
             Handle<PromiseFulfillReactionJobTask>::cast(microtask3)->context());
 
   Handle<Microtask> microtask4(microtask_queue()->get(3), isolate());
-  ASSERT_TRUE(microtask4->IsPromiseFulfillReactionJobTask());
+  ASSERT_TRUE(IsPromiseFulfillReactionJobTask(*microtask4));
   EXPECT_EQ(*context2,
             Handle<PromiseFulfillReactionJobTask>::cast(microtask4)->context());
 
@@ -378,7 +378,7 @@ TEST_P(MicrotaskQueueTest, DetachGlobal_Run) {
   const int kNumExpectedTasks = 3;
   for (int i = 0; i < kNumExpectedTasks; ++i) {
     EXPECT_TRUE(
-        Object::GetElement(isolate(), ran, i).ToHandleChecked()->IsFalse());
+        IsFalse(*Object::GetElement(isolate(), ran, i).ToHandleChecked()));
   }
   EXPECT_EQ(kNumExpectedTasks, microtask_queue()->size());
 
@@ -392,7 +392,7 @@ TEST_P(MicrotaskQueueTest, DetachGlobal_Run) {
   EXPECT_EQ(0, microtask_queue()->size());
   for (int i = 0; i < kNumExpectedTasks; ++i) {
     EXPECT_TRUE(
-        Object::GetElement(isolate(), ran, i).ToHandleChecked()->IsFalse());
+        IsFalse(*Object::GetElement(isolate(), ran, i).ToHandleChecked()));
   }
 }
 
@@ -450,26 +450,23 @@ TEST_P(MicrotaskQueueTest, DetachGlobal_ResolveThenableForeignThen) {
 
       ASSERT_EQ(0, microtask_queue()->size());
       ASSERT_EQ(0, sub_microtask_queue->size());
-      ASSERT_TRUE(Object::GetElement(isolate(), result, 0)
-                      .ToHandleChecked()
-                      ->IsFalse());
+      ASSERT_TRUE(
+          IsFalse(*Object::GetElement(isolate(), result, 0).ToHandleChecked()));
 
       // With a regular thenable, a microtask is queued on the sub-context.
       RunJS<JSPromise>("Promise.resolve({ then: cb => cb(1) })");
       EXPECT_EQ(0, microtask_queue()->size());
       EXPECT_EQ(1, sub_microtask_queue->size());
-      EXPECT_TRUE(Object::GetElement(isolate(), result, 0)
-                      .ToHandleChecked()
-                      ->IsFalse());
+      EXPECT_TRUE(
+          IsFalse(*Object::GetElement(isolate(), result, 0).ToHandleChecked()));
 
       // But when the `then` method comes from another context, a microtask is
       // instead queued on the main context.
       stale_promise = RunJS<JSPromise>("Promise.resolve({ then })");
       EXPECT_EQ(1, microtask_queue()->size());
       EXPECT_EQ(1, sub_microtask_queue->size());
-      EXPECT_TRUE(Object::GetElement(isolate(), result, 0)
-                      .ToHandleChecked()
-                      ->IsFalse());
+      EXPECT_TRUE(
+          IsFalse(*Object::GetElement(isolate(), result, 0).ToHandleChecked()));
     }
 
     sub_context->DetachGlobal();
@@ -477,12 +474,12 @@ TEST_P(MicrotaskQueueTest, DetachGlobal_ResolveThenableForeignThen) {
 
   EXPECT_EQ(1, microtask_queue()->size());
   EXPECT_TRUE(
-      Object::GetElement(isolate(), result, 0).ToHandleChecked()->IsFalse());
+      IsFalse(*Object::GetElement(isolate(), result, 0).ToHandleChecked()));
 
   EXPECT_EQ(1, microtask_queue()->RunMicrotasks(isolate()));
   EXPECT_EQ(0, microtask_queue()->size());
   EXPECT_TRUE(
-      Object::GetElement(isolate(), result, 0).ToHandleChecked()->IsTrue());
+      IsTrue(*Object::GetElement(isolate(), result, 0).ToHandleChecked()));
 }
 
 TEST_P(MicrotaskQueueTest, DetachGlobal_HandlerContext) {
@@ -588,14 +585,14 @@ TEST_P(MicrotaskQueueTest, DetachGlobal_Chain) {
       "result");
   microtask_queue()->RunMicrotasks(isolate());
   EXPECT_TRUE(
-      Object::GetElement(isolate(), result, 0).ToHandleChecked()->IsTrue());
+      IsTrue(*Object::GetElement(isolate(), result, 0).ToHandleChecked()));
 }
 
 TEST_P(MicrotaskQueueTest, DetachGlobal_InactiveHandler) {
   Local<v8::Context> sub_context = v8::Context::New(v8_isolate());
   Utils::OpenHandle(*sub_context)
       ->native_context()
-      .set_microtask_queue(isolate(), microtask_queue());
+      ->set_microtask_queue(isolate(), microtask_queue());
 
   Handle<JSArray> result;
   Handle<JSFunction> stale_handler;
@@ -628,9 +625,9 @@ TEST_P(MicrotaskQueueTest, DetachGlobal_InactiveHandler) {
 
   microtask_queue()->RunMicrotasks(isolate());
   EXPECT_TRUE(
-      Object::GetElement(isolate(), result, 0).ToHandleChecked()->IsFalse());
+      IsFalse(*Object::GetElement(isolate(), result, 0).ToHandleChecked()));
   EXPECT_TRUE(
-      Object::GetElement(isolate(), result, 1).ToHandleChecked()->IsFalse());
+      IsFalse(*Object::GetElement(isolate(), result, 1).ToHandleChecked()));
 }
 
 TEST_P(MicrotaskQueueTest, MicrotasksScope) {
