@@ -150,7 +150,7 @@ TEST_F(UnifiedHeapDetachedTest, AllocationBeforeConfigureHeap) {
   js_heap.AttachCppHeap(heap.get());
   auto& cpp_heap = *CppHeap::From(isolate()->heap()->cpp_heap());
   {
-    CollectGarbage(OLD_SPACE);
+    InvokeMajorGC();
     cpp_heap.AsBase().sweeper().FinishIfRunning();
     EXPECT_TRUE(weak_holder);
   }
@@ -159,7 +159,7 @@ TEST_F(UnifiedHeapDetachedTest, AllocationBeforeConfigureHeap) {
     EmbedderStackStateScope stack_scope(
         &js_heap, EmbedderStackStateScope::kExplicitInvocation,
         StackState::kNoHeapPointers);
-    CollectGarbage(OLD_SPACE);
+    InvokeMajorGC();
     cpp_heap.AsBase().sweeper().FinishIfRunning();
     EXPECT_FALSE(weak_holder);
   }
@@ -439,7 +439,7 @@ TEST_F(UnifiedHeapTest, TracedReferenceOnStack) {
     observer.SetWeak();
   }
   EXPECT_FALSE(observer.IsEmpty());
-  FullGC();
+  InvokeMajorGC();
   EXPECT_FALSE(observer.IsEmpty());
 }
 
@@ -478,6 +478,7 @@ class GCedWithHeapRef final : public cppgc::GarbageCollected<GCedWithHeapRef> {
 
 V8_NOINLINE void StackToHeapTest(v8::Isolate* v8_isolate, Operation op,
                                  TargetHandling target_handling) {
+  i::Isolate* i_isolate = reinterpret_cast<i::Isolate*>(v8_isolate);
   v8::Global<v8::Object> observer;
   v8::TracedReference<v8::Value> stack_handle;
   v8::CppHeap* cpp_heap = v8_isolate->GetCppHeap();
@@ -492,7 +493,7 @@ V8_NOINLINE void StackToHeapTest(v8::Isolate* v8_isolate, Operation op,
         IsNewObjectInCorrectGeneration(*v8::Utils::OpenHandle(*to_object)));
     if (!v8_flags.single_generation &&
         target_handling == TargetHandling::kInitializedOldGen) {
-      FullGC(v8_isolate);
+      InvokeMajorGC(i_isolate);
       EXPECT_FALSE(
           i::Heap::InYoungGeneration(*v8::Utils::OpenHandle(*to_object)));
     }
@@ -507,24 +508,25 @@ V8_NOINLINE void StackToHeapTest(v8::Isolate* v8_isolate, Operation op,
     observer.SetWeak();
   }
   EXPECT_FALSE(observer.IsEmpty());
-  FullGC(v8_isolate);
+  InvokeMajorGC(i_isolate);
   EXPECT_FALSE(observer.IsEmpty());
   PerformOperation(op, &cpp_heap_obj->heap_handle, &stack_handle);
-  FullGC(v8_isolate);
+  InvokeMajorGC(i_isolate);
   EXPECT_FALSE(observer.IsEmpty());
   cpp_heap_obj.Clear();
   {
     // Conservative scanning may find stale pointers to on-stack handles.
     // Disable scanning, assuming the slots are overwritten.
     DisableConservativeStackScanningScopeForTesting no_stack_scanning(
-        reinterpret_cast<i::Isolate*>(v8_isolate)->heap());
-    FullGC(v8_isolate);
+        i_isolate->heap());
+    InvokeMajorGC(i_isolate);
   }
   ASSERT_TRUE(observer.IsEmpty());
 }
 
 V8_NOINLINE void HeapToStackTest(v8::Isolate* v8_isolate, Operation op,
                                  TargetHandling target_handling) {
+  i::Isolate* i_isolate = reinterpret_cast<i::Isolate*>(v8_isolate);
   v8::Global<v8::Object> observer;
   v8::TracedReference<v8::Value> stack_handle;
   v8::CppHeap* cpp_heap = v8_isolate->GetCppHeap();
@@ -539,7 +541,7 @@ V8_NOINLINE void HeapToStackTest(v8::Isolate* v8_isolate, Operation op,
         IsNewObjectInCorrectGeneration(*v8::Utils::OpenHandle(*to_object)));
     if (!v8_flags.single_generation &&
         target_handling == TargetHandling::kInitializedOldGen) {
-      FullGC(v8_isolate);
+      InvokeMajorGC(i_isolate);
       EXPECT_FALSE(
           i::Heap::InYoungGeneration(*v8::Utils::OpenHandle(*to_object)));
     }
@@ -554,24 +556,25 @@ V8_NOINLINE void HeapToStackTest(v8::Isolate* v8_isolate, Operation op,
     observer.SetWeak();
   }
   EXPECT_FALSE(observer.IsEmpty());
-  FullGC(v8_isolate);
+  InvokeMajorGC(i_isolate);
   EXPECT_FALSE(observer.IsEmpty());
   PerformOperation(op, &stack_handle, &cpp_heap_obj->heap_handle);
-  FullGC(v8_isolate);
+  InvokeMajorGC(i_isolate);
   EXPECT_FALSE(observer.IsEmpty());
   stack_handle.Reset();
   {
     // Conservative scanning may find stale pointers to on-stack handles.
     // Disable scanning, assuming the slots are overwritten.
     DisableConservativeStackScanningScopeForTesting no_stack_scanning(
-        reinterpret_cast<i::Isolate*>(v8_isolate)->heap());
-    FullGC(v8_isolate);
+        i_isolate->heap());
+    InvokeMajorGC(i_isolate);
   }
   EXPECT_TRUE(observer.IsEmpty());
 }
 
 V8_NOINLINE void StackToStackTest(v8::Isolate* v8_isolate, Operation op,
                                   TargetHandling target_handling) {
+  i::Isolate* i_isolate = reinterpret_cast<i::Isolate*>(v8_isolate);
   v8::Global<v8::Object> observer;
   v8::TracedReference<v8::Value> stack_handle1;
   v8::TracedReference<v8::Value> stack_handle2;
@@ -583,7 +586,7 @@ V8_NOINLINE void StackToStackTest(v8::Isolate* v8_isolate, Operation op,
         IsNewObjectInCorrectGeneration(*v8::Utils::OpenHandle(*to_object)));
     if (!v8_flags.single_generation &&
         target_handling == TargetHandling::kInitializedOldGen) {
-      FullGC(v8_isolate);
+      InvokeMajorGC(i_isolate);
       EXPECT_FALSE(
           i::Heap::InYoungGeneration(*v8::Utils::OpenHandle(*to_object)));
     }
@@ -598,18 +601,18 @@ V8_NOINLINE void StackToStackTest(v8::Isolate* v8_isolate, Operation op,
     observer.SetWeak();
   }
   EXPECT_FALSE(observer.IsEmpty());
-  FullGC(v8_isolate);
+  InvokeMajorGC(i_isolate);
   EXPECT_FALSE(observer.IsEmpty());
   PerformOperation(op, &stack_handle2, &stack_handle1);
-  FullGC(v8_isolate);
+  InvokeMajorGC(i_isolate);
   EXPECT_FALSE(observer.IsEmpty());
   stack_handle2.Reset();
   {
     // Conservative scanning may find stale pointers to on-stack handles.
     // Disable scanning, assuming the slots are overwritten.
     DisableConservativeStackScanningScopeForTesting no_stack_scanning(
-        reinterpret_cast<i::Isolate*>(v8_isolate)->heap());
-    FullGC(v8_isolate);
+        i_isolate->heap());
+    InvokeMajorGC(i_isolate);
   }
   EXPECT_TRUE(observer.IsEmpty());
 }
@@ -684,7 +687,7 @@ TEST_F(UnifiedHeapTest, TracingInEphemerons) {
     Handle<JSObject> js_key =
         handle(JSObject::cast(*v8::Utils::OpenHandle(*key)), i_isolate());
     Handle<JSReceiver> js_value = v8::Utils::OpenHandle(*value);
-    int32_t hash = js_key->GetOrCreateHash(i_isolate()).value();
+    int32_t hash = Object::GetOrCreateHash(*js_key, i_isolate()).value();
     JSWeakCollection::Set(weak_map, js_key, js_value, hash);
   }
   CollectGarbageWithoutEmbedderStack(cppgc::Heap::SweepingType::kAtomic);
@@ -707,4 +710,45 @@ TEST_F(UnifiedHeapTest, TracedReferenceHandlesDoNotLeak) {
   EXPECT_EQ(initial_count, final_count + 1);
 }
 
+namespace {
+class Wrappable2 final : public cppgc::GarbageCollected<Wrappable2> {
+ public:
+  static size_t destructor_call_count;
+  void Trace(cppgc::Visitor* visitor) const {}
+  ~Wrappable2() { destructor_call_count++; }
+};
+
+size_t Wrappable2::destructor_call_count = 0;
+}  // namespace
+
+TEST_F(UnifiedHeapTest, WrapperDescriptorGetter) {
+  v8::Isolate* isolate = v8_isolate();
+  v8::HandleScope scope(isolate);
+  auto* wrappable_object =
+      cppgc::MakeGarbageCollected<Wrappable2>(allocation_handle());
+  v8::WrapperDescriptor descriptor =
+      isolate->GetCppHeap()->wrapper_descriptor();
+  v8::Local<v8::ObjectTemplate> tmpl = v8::ObjectTemplate::New(isolate);
+  int size = std::max(descriptor.wrappable_type_index,
+                      descriptor.wrappable_instance_index) +
+             1;
+  tmpl->SetInternalFieldCount(size);
+  v8::Local<v8::Object> api_object =
+      tmpl->NewInstance(isolate->GetCurrentContext()).ToLocalChecked();
+  api_object->SetAlignedPointerInInternalField(
+      descriptor.wrappable_type_index,
+      &descriptor.embedder_id_for_garbage_collected);
+  api_object->SetAlignedPointerInInternalField(
+      descriptor.wrappable_instance_index, wrappable_object);
+
+  Wrappable2::destructor_call_count = 0;
+  EXPECT_EQ(0u, Wrappable2::destructor_call_count);
+  CollectGarbageWithoutEmbedderStack(cppgc::Heap::SweepingType::kAtomic);
+  EXPECT_EQ(0u, Wrappable2::destructor_call_count);
+  api_object->SetAlignedPointerInInternalField(
+      descriptor.wrappable_instance_index, nullptr);
+  CollectGarbageWithoutEmbedderStack(cppgc::Heap::SweepingType::kAtomic);
+  EXPECT_EQ(1u, Wrappable2::destructor_call_count);
+}
+
 }  // namespace v8::internal
