@@ -31,79 +31,106 @@ using HeapTest = TestWithHeapInternalsAndContext;
 TEST(Heap, YoungGenerationSizeFromOldGenerationSize) {
   const size_t pm = i::Heap::kPointerMultiplier;
   const size_t hlm = i::Heap::kHeapLimitMultiplier;
-  const size_t semi_space_multiplier = v8_flags.minor_mc ? 4 : 3;
-  ASSERT_EQ(semi_space_multiplier * 512u * pm * KB,
+  // Low memory
+  ASSERT_EQ((v8_flags.minor_ms ? 4 : 3) * 512u * pm * KB,
             i::Heap::YoungGenerationSizeFromOldGenerationSize(128u * hlm * MB));
-  ASSERT_EQ(semi_space_multiplier * 2048u * pm * KB,
+  // High memory
+  ASSERT_EQ(v8_flags.minor_ms ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                              : 3 * 2u * pm * MB,
             i::Heap::YoungGenerationSizeFromOldGenerationSize(256u * hlm * MB));
-  ASSERT_EQ(semi_space_multiplier * 4096u * pm * KB,
+  ASSERT_EQ(v8_flags.minor_ms ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                              : 3 * 4u * pm * MB,
             i::Heap::YoungGenerationSizeFromOldGenerationSize(512u * hlm * MB));
-  ASSERT_EQ(
-      semi_space_multiplier * 8192u * pm * KB,
-      i::Heap::YoungGenerationSizeFromOldGenerationSize(1024u * hlm * MB));
+  ASSERT_EQ(v8_flags.minor_ms ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                              : 3 * 8u * pm * MB,
+            i::Heap::YoungGenerationSizeFromOldGenerationSize(1u * hlm * GB));
 }
 
 TEST(Heap, GenerationSizesFromHeapSize) {
   const size_t pm = i::Heap::kPointerMultiplier;
   const size_t hlm = i::Heap::kHeapLimitMultiplier;
-  const size_t semi_space_multiplier = v8_flags.minor_mc ? 4 : 3;
-  // On tiny heap max semi space capacity is set to the default capacity which
-  // MinorMC does not double.
-  const size_t tiny_heap_semi_space_multiplier = v8_flags.minor_mc ? 2 : 3;
 
   size_t old, young;
 
+  // Low memory
   i::Heap::GenerationSizesFromHeapSize(1 * KB, &young, &old);
   ASSERT_EQ(0u, old);
   ASSERT_EQ(0u, young);
 
+  // On tiny heap max semi space capacity is set to the default capacity which
+  // MinorMS does not double.
   i::Heap::GenerationSizesFromHeapSize(
-      1 * KB + tiny_heap_semi_space_multiplier * 512u * pm * KB, &young, &old);
+      1 * KB + (v8_flags.minor_ms ? 2 : 3) * 512u * pm * KB, &young, &old);
   ASSERT_EQ(1u * KB, old);
-  ASSERT_EQ(tiny_heap_semi_space_multiplier * 512u * pm * KB, young);
+  ASSERT_EQ((v8_flags.minor_ms ? 2 : 3) * 512u * pm * KB, young);
 
   i::Heap::GenerationSizesFromHeapSize(
-      128 * hlm * MB + semi_space_multiplier * 512 * pm * KB, &young, &old);
+      128 * hlm * MB + (v8_flags.minor_ms ? 4 : 3) * 512 * pm * KB, &young,
+      &old);
   ASSERT_EQ(128u * hlm * MB, old);
-  ASSERT_EQ(semi_space_multiplier * 512u * pm * KB, young);
+  ASSERT_EQ((v8_flags.minor_ms ? 4 : 3) * 512u * pm * KB, young);
 
+  // High memory
   i::Heap::GenerationSizesFromHeapSize(
-      256u * hlm * MB + semi_space_multiplier * 2048 * pm * KB, &young, &old);
+      256u * hlm * MB + (v8_flags.minor_ms
+                             ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                             : 3 * 2 * pm * MB),
+      &young, &old);
   ASSERT_EQ(256u * hlm * MB, old);
-  ASSERT_EQ(semi_space_multiplier * 2048u * pm * KB, young);
+  ASSERT_EQ(v8_flags.minor_ms ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                              : 3 * 2 * pm * MB,
+            young);
 
   i::Heap::GenerationSizesFromHeapSize(
-      512u * hlm * MB + semi_space_multiplier * 4096 * pm * KB, &young, &old);
+      512u * hlm * MB + (v8_flags.minor_ms
+                             ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                             : 3 * 4 * pm * MB),
+      &young, &old);
   ASSERT_EQ(512u * hlm * MB, old);
-  ASSERT_EQ(semi_space_multiplier * 4096u * pm * KB, young);
+  ASSERT_EQ(v8_flags.minor_ms ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                              : 3 * 4U * pm * MB,
+            young);
 
   i::Heap::GenerationSizesFromHeapSize(
-      1024u * hlm * MB + semi_space_multiplier * 8192 * pm * KB, &young, &old);
-  ASSERT_EQ(1024u * hlm * MB, old);
-  ASSERT_EQ(semi_space_multiplier * 8192u * pm * KB, young);
+      1u * hlm * GB + (v8_flags.minor_ms
+                           ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                           : 3 * 8 * pm * MB),
+      &young, &old);
+  ASSERT_EQ(1u * hlm * GB, old);
+  ASSERT_EQ(v8_flags.minor_ms ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                              : 3 * 8U * pm * MB,
+            young);
 }
 
 TEST(Heap, HeapSizeFromPhysicalMemory) {
   const size_t pm = i::Heap::kPointerMultiplier;
   const size_t hlm = i::Heap::kHeapLimitMultiplier;
 
-  const size_t semi_space_multiplier = v8_flags.minor_mc ? 4 : 3;
   // The expected value is old_generation_size + semi_space_multiplier *
   // semi_space_size.
-  ASSERT_EQ(128 * hlm * MB + semi_space_multiplier * 512 * pm * KB,
+
+  // Low memory
+  ASSERT_EQ(128 * hlm * MB + (v8_flags.minor_ms ? 4 : 3) * 512 * pm * KB,
             i::Heap::HeapSizeFromPhysicalMemory(0u));
-  ASSERT_EQ(128 * hlm * MB + semi_space_multiplier * 512 * pm * KB,
+  ASSERT_EQ(128 * hlm * MB + (v8_flags.minor_ms ? 4 : 3) * 512 * pm * KB,
             i::Heap::HeapSizeFromPhysicalMemory(512u * MB));
-  ASSERT_EQ(256 * hlm * MB + semi_space_multiplier * 2048 * pm * KB,
-            i::Heap::HeapSizeFromPhysicalMemory(1024u * MB));
-  ASSERT_EQ(512 * hlm * MB + semi_space_multiplier * 4096 * pm * KB,
-            i::Heap::HeapSizeFromPhysicalMemory(2048u * MB));
+  // High memory
+  ASSERT_EQ(256 * hlm * MB + (v8_flags.minor_ms
+                                  ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                                  : 3 * 2 * pm * MB),
+            i::Heap::HeapSizeFromPhysicalMemory(1u * GB));
+  ASSERT_EQ(512 * hlm * MB + (v8_flags.minor_ms
+                                  ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                                  : 3 * 4 * pm * MB),
+            i::Heap::HeapSizeFromPhysicalMemory(2u * GB));
   ASSERT_EQ(
-      1024 * hlm * MB + semi_space_multiplier * 8192 * pm * KB,
-      i::Heap::HeapSizeFromPhysicalMemory(static_cast<uint64_t>(4096u) * MB));
+      1 * hlm * GB + (v8_flags.minor_ms ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                                        : 3 * 8 * pm * MB),
+      i::Heap::HeapSizeFromPhysicalMemory(static_cast<uint64_t>(4u) * GB));
   ASSERT_EQ(
-      1024 * hlm * MB + semi_space_multiplier * 8192 * pm * KB,
-      i::Heap::HeapSizeFromPhysicalMemory(static_cast<uint64_t>(8192u) * MB));
+      1 * hlm * GB + (v8_flags.minor_ms ? 2 * i::Heap::DefaultMaxSemiSpaceSize()
+                                        : 3 * 8 * pm * MB),
+      i::Heap::HeapSizeFromPhysicalMemory(static_cast<uint64_t>(8u) * GB));
 }
 
 TEST_F(HeapTest, ASLR) {
@@ -178,8 +205,7 @@ TEST_F(HeapTest, HeapLayout) {
     Address address = chunk->address();
     size_t size = chunk->area_end() - address;
     AllocationSpace owner_id = chunk->owner_identity();
-    if (V8_EXTERNAL_CODE_SPACE_BOOL &&
-        (owner_id == CODE_SPACE || owner_id == CODE_LO_SPACE)) {
+    if (V8_EXTERNAL_CODE_SPACE_BOOL && IsAnyCodeSpace(owner_id)) {
       EXPECT_TRUE(code_reservation.contains(address, size));
     } else {
       EXPECT_TRUE(heap_reservation.contains(address, size));
@@ -190,17 +216,17 @@ TEST_F(HeapTest, HeapLayout) {
 
 namespace {
 void ShrinkNewSpace(NewSpace* new_space) {
-  if (!v8_flags.minor_mc) {
+  if (!v8_flags.minor_ms) {
     SemiSpaceNewSpace::From(new_space)->Shrink();
     return;
   }
-  // MinorMC shrinks the space as part of sweeping. Here we fake a GC cycle, in
+  // MinorMS shrinks the space as part of sweeping. Here we fake a GC cycle, in
   // which we just shrink without marking or sweeping.
   PagedNewSpace* paged_new_space = PagedNewSpace::From(new_space);
   Heap* heap = paged_new_space->heap();
   heap->EnsureSweepingCompleted(Heap::SweepingForcedFinalizationMode::kV8Only);
   GCTracer* tracer = heap->tracer();
-  tracer->StartObservablePause();
+  tracer->StartObservablePause(base::TimeTicks::Now());
   tracer->StartCycle(GarbageCollector::MARK_COMPACTOR,
                      GarbageCollectionReason::kTesting, "heap unittest",
                      GCTracer::MarkingType::kAtomic);
@@ -215,21 +241,21 @@ void ShrinkNewSpace(NewSpace* new_space) {
     } else {
       // The number of live bytes should be zero, because at this point we're
       // after a GC.
-      DCHECK_EQ(0, heap->non_atomic_marking_state()->live_bytes(page));
+      DCHECK_EQ(0, page->live_bytes());
       // We set it to the number of allocated bytes, because FinishShrinking
       // below expects that all pages have been swept and those that remain
       // contain live bytes.
-      heap->non_atomic_marking_state()->SetLiveBytes(page,
-                                                     page->allocated_bytes());
+      page->SetLiveBytes(page->allocated_bytes());
     }
   }
   paged_new_space->FinishShrinking();
   for (Page* page : *paged_new_space) {
     // We reset the number of live bytes to zero, as is expected after a GC.
-    heap->non_atomic_marking_state()->SetLiveBytes(page, 0);
+    page->SetLiveBytes(0);
   }
   tracer->StopAtomicPause();
-  tracer->StopObservablePause();
+  tracer->StopObservablePause(GarbageCollector::MARK_COMPACTOR,
+                              base::TimeTicks::Now());
   tracer->NotifyFullSweepingCompleted();
 }
 }  // namespace
@@ -250,8 +276,8 @@ TEST_F(HeapTest, GrowAndShrinkNewSpace) {
   }
 
   // Make sure we're in a consistent state to start out.
-  CollectAllGarbage();
-  CollectAllGarbage();
+  InvokeMajorGC();
+  InvokeMajorGC();
   ShrinkNewSpace(new_space);
 
   // Explicitly growing should double the space capacity.
@@ -276,14 +302,14 @@ TEST_F(HeapTest, GrowAndShrinkNewSpace) {
   CHECK_EQ(old_capacity, new_capacity);
 
   // Let the scavenger empty the new space.
-  CollectGarbage(NEW_SPACE);
+  EmptyNewSpaceUsingGC();
   CHECK_LE(new_space->Size(), old_capacity);
 
   // Explicitly shrinking should halve the space capacity.
   old_capacity = new_space->TotalCapacity();
   ShrinkNewSpace(new_space);
   new_capacity = new_space->TotalCapacity();
-  if (v8_flags.minor_mc) {
+  if (v8_flags.minor_ms) {
     // Shrinking may not be able to remove any pages if all contain live
     // objects.
     CHECK_GE(old_capacity, new_capacity);
@@ -319,7 +345,7 @@ TEST_F(HeapTest, CollectingAllAvailableGarbageShrinksNewSpace) {
     v8::HandleScope temporary_scope(iso);
     SimulateFullSpace(new_space);
   }
-  CollectAllAvailableGarbage();
+  InvokeMemoryReducingMajorGCs();
   new_capacity = new_space->TotalCapacity();
   CHECK_EQ(old_capacity, new_capacity);
 }
@@ -390,17 +416,17 @@ TEST_F(HeapTest, RememberedSet_InsertOnPromotingObjectToOld) {
   // Create a young object and age it one generation inside the new space.
   Handle<FixedArray> arr = factory->NewFixedArray(1);
   std::vector<Handle<FixedArray>> handles;
-  if (v8_flags.minor_mc) {
+  if (v8_flags.minor_ms) {
     NewSpace* new_space = heap->new_space();
     CHECK_NE(new_space->TotalCapacity(), new_space->MaximumCapacity());
-    // Fill current pages to force MinorMC to promote them.
+    // Fill current pages to force MinorMS to promote them.
     SimulateFullSpace(new_space, &handles);
     IsolateSafepointScope scope(heap);
     // New empty pages should remain in new space.
     new_space->Grow();
     CHECK(new_space->EnsureCurrentCapacity());
   }
-  CollectGarbage(i::NEW_SPACE);
+  InvokeMinorGC();
   CHECK(Heap::InYoungGeneration(*arr));
 
   // Add into 'arr' a reference to an object one generation younger.
@@ -412,12 +438,16 @@ TEST_F(HeapTest, RememberedSet_InsertOnPromotingObjectToOld) {
 
   // Promote 'arr' into old, its element is still in new, the old to new
   // refs are inserted into the remembered sets during GC.
-  CollectGarbage(i::NEW_SPACE);
+  InvokeMinorGC();
   heap->EnsureSweepingCompleted(Heap::SweepingForcedFinalizationMode::kV8Only);
 
   CHECK(heap->InOldSpace(*arr));
   CHECK(heap->InYoungGeneration(arr->get(0)));
-  CHECK_EQ(1, GetRememberedSetSize<OLD_TO_NEW>(*arr));
+  if (v8_flags.minor_ms) {
+    CHECK_EQ(1, GetRememberedSetSize<OLD_TO_NEW_BACKGROUND>(*arr));
+  } else {
+    CHECK_EQ(1, GetRememberedSetSize<OLD_TO_NEW>(*arr));
+  }
 }
 
 TEST_F(HeapTest, Regress978156) {
@@ -429,7 +459,7 @@ TEST_F(HeapTest, Regress978156) {
   Heap* heap = isolate()->heap();
 
   // 1. Ensure that the new space is empty.
-  GcAndSweep(OLD_SPACE);
+  EmptyNewSpaceUsingGC();
   // 2. Fill the new space with FixedArrays.
   std::vector<Handle<FixedArray>> arrays;
   SimulateFullSpace(heap->new_space(), &arrays);
@@ -441,7 +471,7 @@ TEST_F(HeapTest, Regress978156) {
   HeapObject filler = HeapObject::FromAddress(
       MemoryChunk::FromHeapObject(*last)->area_end() - kTaggedSize);
   HeapObject::FromAddress(last->address() + last->Size());
-  CHECK(filler.IsFiller());
+  CHECK(IsFiller(filler));
   // 5. Start incremental marking.
   i::IncrementalMarking* marking = heap->incremental_marking();
   if (marking->IsStopped()) {
