@@ -441,30 +441,7 @@ TEST_F(LogTest, LogVersion) {
 // https://crbug.com/539892
 // CodeCreateEvents with really large names should not crash.
 TEST_F(LogTest, Issue539892) {
-  class FakeCodeEventLogger : public i::CodeEventLogger {
-   public:
-    explicit FakeCodeEventLogger(i::Isolate* isolate)
-        : CodeEventLogger(isolate) {}
-
-    void CodeMoveEvent(i::InstructionStream from,
-                       i::InstructionStream to) override {}
-    void BytecodeMoveEvent(i::BytecodeArray from,
-                           i::BytecodeArray to) override {}
-    void CodeDisableOptEvent(i::Handle<i::AbstractCode> code,
-                             i::Handle<i::SharedFunctionInfo> shared) override {
-    }
-
-   private:
-    void LogRecordedBuffer(i::AbstractCode code,
-                           i::MaybeHandle<i::SharedFunctionInfo> maybe_shared,
-                           const char* name, int length) override {}
-#if V8_ENABLE_WEBASSEMBLY
-    void LogRecordedBuffer(const i::wasm::WasmCode* code, const char* name,
-                           int length) override {}
-#endif  // V8_ENABLE_WEBASSEMBLY
-  };
-
-  FakeCodeEventLogger code_event_logger(i_isolate());
+  i::FakeCodeEventLogger code_event_logger(i_isolate());
 
   {
     ScopedLoggerInitializer logger(isolate());
@@ -506,7 +483,7 @@ class LogAllTest : public LogTest {
     i::v8_flags.log_all = true;
     i::v8_flags.log_deopt = true;
     i::v8_flags.turbo_inlining = false;
-    i::v8_flags.log_internal_timer_events = true;
+    i::v8_flags.log_timer_events = true;
     i::v8_flags.allow_natives_syntax = true;
     LogTest::SetUpTestSuite();
   }
@@ -830,6 +807,9 @@ TEST_F(LogExternalInterpretedFramesNativeStackTest,
     v8::Local<v8::Context> context = v8::Context::New(isolate());
     context->Enter();
 
+    i::FakeCodeEventLogger code_event_logger(i_isolate());
+    i_isolate()->v8_file_logger()->AddLogEventListener(&code_event_logger);
+
     TestCodeEventHandler code_event_handler(isolate());
 
     const char* source_text_before_start =
@@ -861,6 +841,7 @@ TEST_F(LogExternalInterpretedFramesNativeStackTest,
         1);
 
     context->Exit();
+    i_isolate()->v8_file_logger()->RemoveLogEventListener(&code_event_logger);
   }
 }
 #endif  // V8_TARGET_ARCH_ARM
@@ -922,14 +903,14 @@ void ValidateMapDetailsLogging(v8::Isolate* isolate,
   i::HeapObjectIterator iterator(heap);
   i::DisallowGarbageCollection no_gc;
   size_t i = 0;
-  for (i::HeapObject obj = iterator.Next(); !obj.is_null();
+  for (i::Tagged<i::HeapObject> obj = iterator.Next(); !obj.is_null();
        obj = iterator.Next()) {
-    if (!obj.IsMap()) continue;
+    if (!IsMap(obj)) continue;
     i++;
     uintptr_t address = obj.ptr();
     if (map_create_addresses.find(address) == map_create_addresses.end()) {
       // logger->PrintLog();
-      i::Map::cast(obj).Print();
+      i::Print(i::Map::cast(obj));
       FATAL(
           "Map (%p, #%zu) creation not logged during startup with "
           "--log-maps!"
@@ -939,7 +920,7 @@ void ValidateMapDetailsLogging(v8::Isolate* isolate,
     } else if (map_details_addresses.find(address) ==
                map_details_addresses.end()) {
       // logger->PrintLog();
-      i::Map::cast(obj).Print();
+      i::Print(i::Map::cast(obj));
       FATAL(
           "Map (%p, #%zu) details not logged during startup with "
           "--log-maps!"
@@ -1094,7 +1075,15 @@ TEST_F(LogMapsTest, LogMapsDetailsContexts) {
   }
 }
 
-TEST_F(LogTest, ConsoleTimeEvents) {
+class LogTimerTest : public LogTest {
+ public:
+  static void SetUpTestSuite() {
+    i::v8_flags.log_timer_events = true;
+    LogTest::SetUpTestSuite();
+  }
+};
+
+TEST_F(LogTimerTest, ConsoleTimeEvents) {
   {
     ScopedLoggerInitializer logger(isolate());
     {
