@@ -10,7 +10,7 @@ namespace wasm {
 
 TEST_F(WasmCapiTest, Regressions) {
   FunctionSig sig(0, 0, nullptr);
-  byte code[] = {WASM_UNREACHABLE};
+  uint8_t code[] = {WASM_UNREACHABLE};
   WasmFunctionBuilder* start_func = builder()->AddFunction(&sig);
   start_func->EmitCode(code, static_cast<uint32_t>(sizeof(code)));
   start_func->Emit(kExprEnd);
